@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -117,6 +117,7 @@ createDynamicHttpResponse(HttpRequestPtr request) {
     }
 
     // Callout point for "auth".
+    bool reset_handle = false;
     if (HooksManager::calloutsPresent(Hooks.hook_index_auth_)) {
         // Get callout handle.
         CalloutHandlePtr callout_handle = request->getCalloutHandle();
@@ -131,7 +132,10 @@ createDynamicHttpResponse(HttpRequestPtr request) {
         callout_handle->getArgument("request", request);
         callout_handle->getArgument("response", http_response);
 
-        // Ignore status as the HTTP response is used instead.
+        // Status other than continue means 'please reset the handle'.
+        if (callout_handle->getStatus() != CalloutHandle::NEXT_STEP_CONTINUE) {
+            reset_handle = true;
+        }
     }
 
     // The basic HTTP authentication check or a callout failed and
@@ -140,6 +144,11 @@ createDynamicHttpResponse(HttpRequestPtr request) {
         return (http_response);
     }
 
+    // Reset the handle when a hook asks for.
+    if (reset_handle) {
+        request->resetCalloutHandle();
+    }
+
     // The request is always non-null, because this is verified by the
     // createHttpResponse method. Let's try to convert it to the
     // PostHttpRequestJson type as this is the type generated by the
