@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -47,8 +47,9 @@ const SimpleDefaults AgentSimpleParser::AGENT_DEFAULTS = {
 
 /// @brief This table defines default values for authentication.
 const SimpleDefaults AgentSimpleParser::AUTH_DEFAULTS = {
-    { "type",   Element::string,  "basic" },
-    { "realm",  Element::string,  "kea-control-agent" }
+    { "type",       Element::string,  "basic" },
+    { "realm",      Element::string,  "kea-control-agent" },
+    { "directory",  Element::string,  "" }
 };
 
 /// @brief This table defines default values for control sockets.
@@ -173,7 +174,7 @@ AgentSimpleParser::parse(const CtrlAgentCfgContextPtr& ctx,
     if (hooks) {
         HooksLibrariesParser hooks_parser;
         hooks_parser.parse(libraries, hooks);
-        libraries.verifyLibraries(hooks->getPosition());
+        libraries.verifyLibraries(hooks->getPosition(), false);
     }
 
     if (!check_only) {
@@ -182,7 +183,7 @@ AgentSimpleParser::parse(const CtrlAgentCfgContextPtr& ctx,
         // change causes problems when trying to roll back.
         HooksManager::prepareUnloadLibraries();
         static_cast<void>(HooksManager::unloadLibraries());
-        libraries.loadLibraries();
+        libraries.loadLibraries(false);
     }
 }
 
