@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,13 +6,13 @@
 
 #include <config.h>
 
-#include <cc/data.h>
 #include <cc/command_interpreter.h>
-#include <testutils/user_context_utils.h>
-#include <process/testutils/d_test_stubs.h>
-#include <d2/d2_config.h>
-#include <d2/d2_cfg_mgr.h>
+#include <cc/data.h>
 #include <d2/parser_context.h>
+#include <d2srv/d2_cfg_mgr.h>
+#include <d2srv/d2_config.h>
+#include <process/testutils/d_test_stubs.h>
+#include <testutils/user_context_utils.h>
 #include <gtest/gtest.h>
 
 #include <iostream>
@@ -21,6 +21,7 @@
 #include <sstream>
 
 #include "test_data_files_config.h"
+#include "test_callout_libraries.h"
 
 using namespace isc::config;
 using namespace isc::d2;
@@ -95,6 +96,17 @@ parseDHCPDDNS(const std::string& in,  bool verbose = false) {
     }
 }
 
+/// @brief Replace the library path
+void pathReplacer(ConstElementPtr d2_cfg) {
+    ConstElementPtr hooks_libs = d2_cfg->get("hooks-libraries");
+    if (!hooks_libs || hooks_libs->empty()) {
+        return;
+    }
+    ElementPtr first_lib = hooks_libs->getNonConst(0);
+    std::string lib_path(CALLOUT_LIBRARY);
+    first_lib->set("library", Element::create(lib_path));
+}
+
 }
 
 /// Test fixture class
@@ -153,6 +165,9 @@ public:
             return (false);
         }
 
+        // update hooks-libraries
+        pathReplacer(d2);
+
         // try DHCPDDNS configure
         ConstElementPtr status;
         try {
@@ -191,7 +206,7 @@ public:
     /// @brief Reset configuration database.
     ///
     /// This function resets configuration data base by
-    /// removing control sockets and domain lists. Reset must
+    /// removing control sockets, hooks, etc. Reset must
     /// be performed after each test to make sure that
     /// contents of the database do not affect result of
     /// subsequent tests.
@@ -250,13 +265,19 @@ TEST_F(D2GetConfigTest, sample1) {
         ASSERT_NO_THROW(jsonj = parseJSON(expected));
         // the generic JSON parser does not handle comments
         EXPECT_TRUE(isEquivalent(jsond, moveComments(jsonj)));
+        // replace the path by its actual value
+        ConstElementPtr d2;
+        ASSERT_NO_THROW(d2 = jsonj->get("DhcpDdns"));
+        ASSERT_TRUE(d2);
+        pathReplacer(d2);
         // check that unparsed and expected values match
         EXPECT_TRUE(isEquivalent(unparsed, jsonj));
         // check on pretty prints too
-        std::string current = prettyPrint(unparsed, 0, 4) + "\n";
-        EXPECT_EQ(expected, current);
-        if (expected != current) {
-            expected = current;
+        std::string current = prettyPrint(unparsed, 0, 4);
+        std::string expected2 = prettyPrint(jsonj, 0, 4);
+        EXPECT_EQ(expected2, current);
+        if (expected2 != current) {
+            expected = current + "\n";
         }
     }
 
