@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,11 +7,10 @@
 #include <config.h>
 
 #include <asiolink/io_service.h>
-#include <d2/d2_cfg_mgr.h>
-#include <d2/d2_cfg_mgr.h>
 #include <d2/nc_remove.h>
+#include <d2srv/d2_cfg_mgr.h>
+#include <d2srv/testutils/nc_test_utils.h>
 #include <dns/messagerenderer.h>
-#include <nc_test_utils.h>
 
 #include <gtest/gtest.h>
 
@@ -275,7 +274,8 @@ TEST(NameRemoveTransaction, construction) {
         " \"ip-address\" : \"192.168.2.1\" , "
         " \"dhcid\" : \"0102030405060708\" , "
         " \"lease-expires-on\" : \"20130121132405\" , "
-        " \"lease-length\" : 1300 "
+        " \"lease-length\" : 1300, "
+        " \"use-conflict-resolution\" : true "
         "}";
 
     dhcp_ddns::NameChangeRequestPtr ncr;
@@ -582,7 +582,7 @@ TEST_F(NameRemoveTransactionTest, removingFwdAddrsHandler_FwdOnlyOK) {
               name_remove->getNextEvent());
 }
 
-// Tests addingFwdAddrsHandler with the following scenario:
+// Tests removingFwdAddrsHandler with the following scenario:
 //
 //  The request includes only a forward change.
 //  Initial posted event is SERVER_SELECTED_EVT.
@@ -692,24 +692,9 @@ TEST_F(NameRemoveTransactionTest, removingFwdAddrsHandler_Timeout) {
     // and then transition to selecting a new server.
     int max_tries = NameChangeTransaction::MAX_UPDATE_TRIES_PER_SERVER;
     for (int i = 1; i <= max_tries; ++i) {
-        const D2UpdateMessagePtr prev_msg = name_remove->getDnsUpdateRequest();
-
         // Run removingFwdAddrsHandler to send the request.
         EXPECT_NO_THROW(name_remove->removingFwdAddrsHandler());
 
-        const D2UpdateMessagePtr curr_msg = name_remove->getDnsUpdateRequest();
-        if (i == 1) {
-            // First time around we should build the message.
-            EXPECT_FALSE(prev_msg);
-            EXPECT_TRUE(curr_msg);
-        } else {
-            // Subsequent passes should reuse the request. We are only
-            // looking to check that we have not replaced the pointer value
-            // with a new pointer.  This tests the on_entry() logic which
-            // clears the request ONLY upon initial entry into the state.
-            EXPECT_TRUE(prev_msg == curr_msg);
-        }
-
         // Simulate a server IO timeout.
         name_remove->setDnsUpdateStatus(DNSClient::TIMEOUT);
         name_remove->postNextEvent(NameChangeTransaction::IO_COMPLETED_EVT);
@@ -1045,24 +1030,9 @@ TEST_F(NameRemoveTransactionTest, removingFwdRRsHandler_Timeout) {
     // and then transition to selecting a new server.
     int max_tries = NameChangeTransaction::MAX_UPDATE_TRIES_PER_SERVER;
     for (int i = 1; i <= max_tries; ++i) {
-        const D2UpdateMessagePtr prev_msg = name_remove->getDnsUpdateRequest();
-
         // Run removingFwdRRsHandler to send the request.
         EXPECT_NO_THROW(name_remove->removingFwdRRsHandler());
 
-        const D2UpdateMessagePtr curr_msg = name_remove->getDnsUpdateRequest();
-        if (i == 1) {
-            // First time around we should build the message.
-            EXPECT_FALSE(prev_msg);
-            EXPECT_TRUE(curr_msg);
-        } else {
-            // Subsequent passes should reuse the request. We are only
-            // looking to check that we have not replaced the pointer value
-            // with a new pointer.  This tests the on_entry() logic which
-            // clears the request ONLY upon initial entry into the state.
-            EXPECT_TRUE(prev_msg == curr_msg);
-        }
-
         // Simulate a server IO timeout.
         name_remove->setDnsUpdateStatus(DNSClient::TIMEOUT);
         name_remove->postNextEvent(NameChangeTransaction::IO_COMPLETED_EVT);
@@ -1115,24 +1085,9 @@ TEST_F(NameRemoveTransactionTest, removingFwdRRsHandler_InvalidResponse) {
     // and then transition to selecting a new server.
     int max_tries = NameChangeTransaction::MAX_UPDATE_TRIES_PER_SERVER;
     for (int i = 1; i <= max_tries; ++i) {
-        const D2UpdateMessagePtr prev_msg = name_remove->getDnsUpdateRequest();
-
         // Run removingFwdRRsHandler to send the request.
         EXPECT_NO_THROW(name_remove->removingFwdRRsHandler());
 
-        const D2UpdateMessagePtr curr_msg = name_remove->getDnsUpdateRequest();
-        if (i == 1) {
-            // First time around we should build the message.
-            EXPECT_FALSE(prev_msg);
-            EXPECT_TRUE(curr_msg);
-        } else {
-            // Subsequent passes should reuse the request. We are only
-            // looking to check that we have not replaced the pointer value
-            // with a new pointer.  This tests the on_entry() logic which
-            // clears the request ONLY upon initial entry into the state.
-            EXPECT_TRUE(prev_msg == curr_msg);
-        }
-
         // Simulate a corrupt server response.
         name_remove->setDnsUpdateStatus(DNSClient::INVALID_RESPONSE);
         name_remove->postNextEvent(NameChangeTransaction::IO_COMPLETED_EVT);
@@ -1419,24 +1374,9 @@ TEST_F(NameRemoveTransactionTest, removingRevPtrsHandler_Timeout) {
     // and then transition to selecting a new server.
     int max_tries = NameChangeTransaction::MAX_UPDATE_TRIES_PER_SERVER;
     for (int i = 1; i <= max_tries; ++i) {
-        const D2UpdateMessagePtr prev_msg = name_remove->getDnsUpdateRequest();
-
         // Run removingRevPtrsHandler to send the request.
         EXPECT_NO_THROW(name_remove->removingRevPtrsHandler());
 
-        const D2UpdateMessagePtr curr_msg = name_remove->getDnsUpdateRequest();
-        if (i == 1) {
-            // First time around we should build the message.
-            EXPECT_FALSE(prev_msg);
-            EXPECT_TRUE(curr_msg);
-        } else {
-            // Subsequent passes should reuse the request. We are only
-            // looking to check that we have not replaced the pointer value
-            // with a new pointer.  This tests the on_entry() logic which
-            // clears the request ONLY upon initial entry into the state.
-            EXPECT_TRUE(prev_msg == curr_msg);
-        }
-
         // Simulate a server IO timeout.
         name_remove->setDnsUpdateStatus(DNSClient::TIMEOUT);
         name_remove->postNextEvent(NameChangeTransaction::IO_COMPLETED_EVT);
@@ -1488,24 +1428,9 @@ TEST_F(NameRemoveTransactionTest, removingRevPtrsHandler_CorruptResponse) {
     // and then transition to selecting a new server.
     int max_tries = NameChangeTransaction::MAX_UPDATE_TRIES_PER_SERVER;
     for (int i = 1; i <= max_tries; ++i) {
-        const D2UpdateMessagePtr prev_msg = name_remove->getDnsUpdateRequest();
-
         // Run removingRevPtrsHandler to send the request.
         EXPECT_NO_THROW(name_remove->removingRevPtrsHandler());
 
-        const D2UpdateMessagePtr curr_msg = name_remove->getDnsUpdateRequest();
-        if (i == 1) {
-            // First time around we should build the message.
-            EXPECT_FALSE(prev_msg);
-            EXPECT_TRUE(curr_msg);
-        } else {
-            // Subsequent passes should reuse the request. We are only
-            // looking to check that we have not replaced the pointer value
-            // with a new pointer.  This tests the on_entry() logic which
-            // clears the request ONLY upon initial entry into the state.
-            EXPECT_TRUE(prev_msg == curr_msg);
-        }
-
         // Simulate a server corrupt response.
         name_remove->setDnsUpdateStatus(DNSClient::INVALID_RESPONSE);
         name_remove->postNextEvent(NameChangeTransaction::IO_COMPLETED_EVT);
