@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -50,10 +50,15 @@ const char* CONFIGS[] = {
         "},\n"
         "{\n"
         "   \"hw-address\": \"01:02:03:04:05:06\",\n"
-        "   \"hostname\": \"hw-host-fixed\",\n"
+        "   \"hostname\": \"hw-host-fixed-out-of-range\",\n"
         "   \"ip-address\": \"192.0.1.77\"\n"
         "},\n"
         "{\n"
+        "   \"hw-address\": \"02:02:03:04:05:06\",\n"
+        "   \"hostname\": \"hw-host-fixed-in-range\",\n"
+        "   \"ip-address\": \"10.0.0.77\"\n"
+        "},\n"
+        "{\n"
         "   \"duid\": \"01:02:03:04:05\",\n"
         "   \"hostname\": \"duid-host\"\n"
         "},\n"
@@ -69,6 +74,7 @@ const char* CONFIGS[] = {
         "\"valid-lifetime\": 600,\n"
         "\"subnet4\": [ { \n"
         "    \"subnet\": \"10.0.0.0/24\",\n"
+        "    \"id\": 10,\n"
         "    \"reservations-global\": true,\n"
         "    \"reservations-in-subnet\": false,\n"
         "    \"pools\": [ { \"pool\": \"10.0.0.10-10.0.0.100\" } ]\n"
@@ -92,7 +98,7 @@ const char* CONFIGS[] = {
         "\"subnet4\": [\n"
         "    {\n"
         "        \"subnet\": \"10.0.0.0/24\", \n"
-        "        \"id\": 10,"
+        "        \"id\": 10, \n"
         "        \"pools\": [ { \"pool\": \"10.0.0.10-10.0.0.100\" } ],\n"
         "        \"interface\": \"eth0\",\n"
         "        \"reservations\": [ \n"
@@ -131,7 +137,7 @@ const char* CONFIGS[] = {
         "\"subnet4\": [\n"
         "    {\n"
         "        \"subnet\": \"10.0.0.0/24\", \n"
-        "        \"id\": 10,"
+        "        \"id\": 10, \n"
         "        \"pools\": [ { \"pool\": \"10.0.0.10-10.0.0.100\" } ],\n"
         "        \"interface\": \"eth0\",\n"
         "        \"reservations-global\": false,\n"
@@ -161,7 +167,7 @@ const char* CONFIGS[] = {
         "\"subnet4\": [\n"
         "    {\n"
         "        \"subnet\": \"10.0.0.0/24\", \n"
-        "        \"id\": 10,"
+        "        \"id\": 10, \n"
         "        \"pools\": [ { \"pool\": \"10.0.0.10-10.0.0.100\" } ],\n"
         "        \"interface\": \"eth0\",\n"
         "        \"reservations-global\": false,\n"
@@ -206,7 +212,7 @@ const char* CONFIGS[] = {
         "    \"subnet4\": [\n"
         "        {\n"
         "            \"subnet\": \"10.0.0.0/24\", \n"
-        "            \"id\": 10,"
+        "            \"id\": 10, \n"
         "            \"pools\": ["
         "                {"
         "                    \"pool\": \"10.0.0.10-10.0.0.11\","
@@ -258,7 +264,7 @@ const char* CONFIGS[] = {
         "    \"subnet4\": [\n"
         "        {\n"
         "            \"subnet\": \"10.0.0.0/24\", \n"
-        "            \"id\": 10,"
+        "            \"id\": 10, \n"
         "            \"client-class\": \"reserved_class\","
         "            \"pools\": ["
         "                {"
@@ -299,7 +305,7 @@ const char* CONFIGS[] = {
         "\"subnet4\": [\n"
         "    {\n"
         "        \"subnet\": \"10.0.0.0/24\", \n"
-        "        \"id\": 10,"
+        "        \"id\": 10, \n"
         "        \"reservations\": [{ \n"
         "            \"hw-address\": \"aa:bb:cc:dd:ee:fe\",\n"
         "            \"client-classes\": [ \"reserved_class\" ]\n"
@@ -367,7 +373,7 @@ const char* CONFIGS[] = {
         "\"subnet4\": [\n"
         "    {\n"
         "        \"subnet\": \"10.0.0.0/24\", \n"
-        "        \"id\": 10,"
+        "        \"id\": 10, \n"
         "        \"pools\": [ { \"pool\": \"10.0.0.10-10.0.0.100\" } ],\n"
         "        \"interface\": \"eth0\",\n"
         "        \"reservations\": [ \n"
@@ -587,15 +593,24 @@ TEST_F(HostTest, globalHardwareDynamicAddress) {
     runDoraTest(CONFIGS[0], client, "hw-host-dynamic", "10.0.0.10");
 }
 
-// Verifies that a client matched to a global address reservation
-// gets both the hostname and the reserved address
-// when the subnet reservations flags are global only.
-TEST_F(HostTest, globalHardwareFixedAddress) {
+// Verifies that a client matched to a global in-subnet address reservation
+// gets both the hostname and the reserved address when the subnet reservations
+// flags are global only.
+TEST_F(HostTest, globalHardwareFixedAddressInRange) {
+    Dhcp4Client client(Dhcp4Client::SELECTING);
+
+    client.setHWAddress("02:02:03:04:05:06");
+    runDoraTest(CONFIGS[0], client, "hw-host-fixed-in-range", "10.0.0.77");
+}
+
+// Verifies that a client matched to a global out-of-range address reservation
+// gets the hostname and a dynamic address when the subnet reservations
+// flags are global only.
+TEST_F(HostTest, globalHardwareFixedAddressOutOfRange) {
     Dhcp4Client client(Dhcp4Client::SELECTING);
 
-    //client.includeClientId(clientid_a);
     client.setHWAddress("01:02:03:04:05:06");
-    runDoraTest(CONFIGS[0], client, "hw-host-fixed", "192.0.1.77");
+    runDoraTest(CONFIGS[0], client, "hw-host-fixed-out-of-range", "10.0.0.10");
 }
 
 // Verifies that a client can be matched to a global reservation by DUID
