@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -230,10 +230,10 @@ public:
     void runTimersWithTimeout(const IOServicePtr& io_service, const long timeout_ms,
                               std::function<bool()> cond = std::function<bool()>()) {
         IntervalTimer timer(*io_service);
-        bool stopped = false;
+        std::atomic<bool> stopped(false);
         timer.setup([&io_service, &stopped]() {
-            io_service->stop();
             stopped = true;
+            io_service->stop();
         }, timeout_ms, IntervalTimer::ONE_SHOT);
 
         // Run as long as the timeout hasn't occurred and the interrupting
@@ -398,15 +398,17 @@ TEST_F(JSONFileBackendTest, jsonFile) {
         "\"renew-timer\": 1000, "
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.1 - 192.0.2.100\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/24\" "
         " },"
         " {"
         "    \"pools\": [ { \"pool\": \"192.0.3.101 - 192.0.3.150\" } ],"
         "    \"subnet\": \"192.0.3.0/24\", "
-        "    \"id\": 0 "
+        "    \"id\": 2 "
         " },"
         " {"
         "    \"pools\": [ { \"pool\": \"192.0.4.101 - 192.0.4.150\" } ],"
+        "    \"id\": 3, "
         "    \"subnet\": \"192.0.4.0/24\" "
         " } ],"
         "\"valid-lifetime\": 4000 }"
@@ -483,6 +485,7 @@ TEST_F(JSONFileBackendTest, hashComments) {
         "# comments in the middle should be ignored, too\n"
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.0/24\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/22\" "
         " } ],"
         "\"valid-lifetime\": 4000 }"
@@ -534,6 +537,7 @@ TEST_F(JSONFileBackendTest, cppLineComments) {
         "// comments in the middle should be ignored, too\n"
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.0/24\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/22\" "
         " } ],"
         "\"valid-lifetime\": 4000 }"
@@ -585,6 +589,7 @@ TEST_F(JSONFileBackendTest, cBlockComments) {
         "/* comments in the middle should be ignored, too*/\n"
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.0/24\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/22\" "
         " } ],"
         "\"valid-lifetime\": 4000 }"
@@ -637,6 +642,7 @@ TEST_F(JSONFileBackendTest, include) {
     string include = "\n"
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.0/24\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/22\" "
         " } ]\n";
 
@@ -683,6 +689,7 @@ TEST_F(JSONFileBackendTest, recursiveInclude) {
         "\"renew-timer\": 1000, \n"
         "\"subnet4\": [ { "
         "    \"pools\": [ { \"pool\": \"192.0.2.0/24\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"192.0.2.0/22\" "
         " } ],"
         "\"valid-lifetime\": 4000 }"
@@ -721,12 +728,13 @@ TEST_F(JSONFileBackendTest, configBroken) {
     string config_empty = "";
 
     // This config does not have mandatory Dhcp4 element
-    string config_v4 = "{ \"Dhcp6\": { \"interfaces\": [ \"*\" ],"
+    string config_v6 = "{ \"Dhcp6\": { \"interfaces\": [ \"*\" ],"
         "\"preferred-lifetime\": 3000,"
         "\"rebind-timer\": 2000, "
         "\"renew-timer\": 1000, "
         "\"subnet4\": [ { "
         "    \"pool\": [ \"2001:db8::/80\" ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8::/64\" "
         " } ]}";
 
@@ -747,7 +755,7 @@ TEST_F(JSONFileBackendTest, configBroken) {
     EXPECT_THROW(srv->init(TEST_FILE), BadValue);
 
     // Now try to load a config that does not have Dhcp4 component.
-    writeFile(TEST_FILE, config_v4);
+    writeFile(TEST_FILE, config_v6);
     EXPECT_THROW(srv->init(TEST_FILE), BadValue);
 
     // Now try to load a config with Dhcp4 full of nonsense.
