@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -61,7 +61,8 @@ class ReleaseTest : public Dhcpv4SrvTest {
 public:
 
     enum ExpectedResult {
-        SHOULD_PASS,
+        SHOULD_PASS_EXPIRED,
+        SHOULD_PASS_DELETED,
         SHOULD_FAIL
     };
 
@@ -69,9 +70,7 @@ public:
     ///
     /// Sets up fake interfaces.
     ReleaseTest()
-        : Dhcpv4SrvTest(),
-          iface_mgr_test_config_(true) {
-        IfaceMgr::instance().openSockets4();
+        : Dhcpv4SrvTest(), iface_mgr_test_config_(true) {
     }
 
     /// @brief Performs 4-way exchange to obtain new lease.
@@ -85,18 +84,21 @@ public:
     /// @param client_id_1 Client id to be used to acquire the lease.
     /// @param hw_address_2 HW Address to be used to release the lease.
     /// @param client_id_2 Client id to be used to release the lease.
-    /// @param expected_result SHOULD_PASS if the lease is expected to
-    /// be successfully released, or SHOULD_FAIL if the lease is expected
-    /// to not be released.
+    /// @param expected_result SHOULD_PASS_EXPIRED if the lease is expected to
+    /// be successfully released and expired, SHOULD_PASS_DELETED if the lease
+    /// is expected to be successfully released and deleted, or SHOULD_FAIL if
+    /// the lease is expected to not be released.
+    /// @param lease_affinity A flag which indicates if lease affinity should
+    /// be enabled or disabled.
     void acquireAndRelease(const std::string& hw_address_1,
                            const std::string& client_id_1,
                            const std::string& hw_address_2,
                            const std::string& client_id_2,
-                           ExpectedResult expected_result);
+                           ExpectedResult expected_result,
+                           const LeaseAffinity lease_affinity);
 
     /// @brief Interface Manager's fake configuration control.
     IfaceMgrTestConfig iface_mgr_test_config_;
-
 };
 
 void
@@ -124,11 +126,12 @@ ReleaseTest::acquireAndRelease(const std::string& hw_address_1,
                                const std::string& client_id_1,
                                const std::string& hw_address_2,
                                const std::string& client_id_2,
-                               ExpectedResult expected_result) {
+                               ExpectedResult expected_result,
+                               const LeaseAffinity lease_affinity) {
     CfgMgr::instance().clear();
     Dhcp4Client client(Dhcp4Client::SELECTING);
     // Configure DHCP server.
-    configure(RELEASE_CONFIGS[0], *client.getServer());
+    configure(RELEASE_CONFIGS[0], *client.getServer(), true, true, true, false, lease_affinity);
     // Explicitly set the client id.
     client.includeClientId(client_id_1);
     // Explicitly set the HW address.
@@ -142,7 +145,7 @@ ReleaseTest::acquireAndRelease(const std::string& hw_address_1,
     const Subnet4Collection* subnets =
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
     ASSERT_EQ(1, subnets->size());
-    name << "subnet[" << subnets->at(0)->getID() << "].assigned-addresses";
+    name << "subnet[" << (*subnets->begin())->getID() << "].assigned-addresses";
 
     ObservationPtr assigned_cnt = StatsMgr::instance().getObservation(name.str());
     ASSERT_TRUE(assigned_cnt);
@@ -168,7 +171,16 @@ ReleaseTest::acquireAndRelease(const std::string& hw_address_1,
     // We check if the release process was successful by checking if the
     // lease is in the database. It is expected that it is not present,
     // i.e. has been deleted with the release.
-    if (expected_result == SHOULD_PASS) {
+    if (expected_result == SHOULD_PASS_EXPIRED) {
+        ASSERT_TRUE(lease);
+
+        // The update succeeded, so the assigned-address should be expired
+        EXPECT_EQ(lease->valid_lft_, 0);
+
+        // No lease has been removed, so the assigned-address should be the same
+        // as before
+        EXPECT_EQ(before, after);
+    } else if (expected_result == SHOULD_PASS_DELETED) {
         EXPECT_FALSE(lease);
 
         // The removal succeeded, so the assigned-addresses statistic should
@@ -187,7 +199,14 @@ ReleaseTest::acquireAndRelease(const std::string& hw_address_1,
 TEST_F(ReleaseTest, releaseNoIdentifierChange) {
     acquireAndRelease("01:02:03:04:05:06", "12:14",
                       "01:02:03:04:05:06", "12:14",
-                      SHOULD_PASS);
+                      SHOULD_PASS_DELETED, LEASE_AFFINITY_DISABLED);
+}
+
+// This test checks that the client can acquire and release the lease.
+TEST_F(ReleaseTest, releaseNoDeleteNoIdentifierChange) {
+    acquireAndRelease("01:02:03:04:05:06", "12:14",
+                      "01:02:03:04:05:06", "12:14",
+                      SHOULD_PASS_EXPIRED, LEASE_AFFINITY_ENABLED);
 }
 
 // This test verifies the release correctness in the following case:
@@ -198,7 +217,18 @@ TEST_F(ReleaseTest, releaseNoIdentifierChange) {
 TEST_F(ReleaseTest, releaseHWAddressOnly) {
     acquireAndRelease("01:02:03:04:05:06", "",
                       "01:02:03:04:05:06", "",
-                      SHOULD_PASS);
+                      SHOULD_PASS_DELETED, LEASE_AFFINITY_DISABLED);
+}
+
+// This test verifies the release correctness in the following case:
+// - Client acquires new lease using HW address only
+// - Client sends the DHCPRELEASE with valid HW address and without
+//   client identifier.
+// - The server successfully releases the lease.
+TEST_F(ReleaseTest, releaseNoDeleteHWAddressOnly) {
+    acquireAndRelease("01:02:03:04:05:06", "",
+                      "01:02:03:04:05:06", "",
+                      SHOULD_PASS_EXPIRED, LEASE_AFFINITY_ENABLED);
 }
 
 // This test verifies the release correctness in the following case:
@@ -209,7 +239,18 @@ TEST_F(ReleaseTest, releaseHWAddressOnly) {
 TEST_F(ReleaseTest, releaseNoClientId) {
     acquireAndRelease("01:02:03:04:05:06", "12:14",
                       "01:02:03:04:05:06", "",
-                      SHOULD_PASS);
+                      SHOULD_PASS_DELETED, LEASE_AFFINITY_DISABLED);
+}
+
+// This test verifies the release correctness in the following case:
+// - Client acquires new lease using the client identifier and HW address
+// - Client sends the DHCPRELEASE with valid HW address but with no
+//   client identifier.
+// - The server successfully releases the lease.
+TEST_F(ReleaseTest, releaseNoDeleteNoClientId) {
+    acquireAndRelease("01:02:03:04:05:06", "12:14",
+                      "01:02:03:04:05:06", "",
+                      SHOULD_PASS_EXPIRED, LEASE_AFFINITY_ENABLED);
 }
 
 // This test verifies the release correctness in the following case:
@@ -221,7 +262,19 @@ TEST_F(ReleaseTest, releaseNoClientId) {
 TEST_F(ReleaseTest, releaseNoClientId2) {
     acquireAndRelease("01:02:03:04:05:06", "",
                       "01:02:03:04:05:06", "12:14",
-                      SHOULD_PASS);
+                      SHOULD_PASS_DELETED, LEASE_AFFINITY_DISABLED);
+}
+
+// This test verifies the release correctness in the following case:
+// - Client acquires new lease using HW address
+// - Client sends the DHCPRELEASE with valid HW address and some
+//   client identifier.
+// - The server identifies the lease using HW address and releases
+//   this lease.
+TEST_F(ReleaseTest, releaseNoDeleteNoClientId2) {
+    acquireAndRelease("01:02:03:04:05:06", "",
+                      "01:02:03:04:05:06", "12:14",
+                      SHOULD_PASS_EXPIRED, LEASE_AFFINITY_ENABLED);
 }
 
 // This test checks the server's behavior in the following case:
@@ -232,7 +285,7 @@ TEST_F(ReleaseTest, releaseNoClientId2) {
 TEST_F(ReleaseTest, releaseNonMatchingClientId) {
     acquireAndRelease("01:02:03:04:05:06", "12:14",
                       "01:02:03:04:05:06", "12:15:16",
-                      SHOULD_FAIL);
+                      SHOULD_FAIL, LEASE_AFFINITY_DISABLED);
 }
 
 // This test checks the server's behavior in the following case:
@@ -244,7 +297,19 @@ TEST_F(ReleaseTest, releaseNonMatchingClientId) {
 TEST_F(ReleaseTest, releaseNonMatchingHWAddress) {
     acquireAndRelease("01:02:03:04:05:06", "12:14",
                       "06:06:06:06:06:06", "12:14",
-                      SHOULD_PASS);
+                      SHOULD_PASS_DELETED, LEASE_AFFINITY_DISABLED);
+}
+
+// This test checks the server's behavior in the following case:
+// - Client acquires new lease using client identifier and HW address
+// - Client sends the DHCPRELEASE with the same client identifier but
+//   different HW address.
+// - The server uses client identifier to find the client's lease and
+//   releases it.
+TEST_F(ReleaseTest, releaseNoDeleteNonMatchingHWAddress) {
+    acquireAndRelease("01:02:03:04:05:06", "12:14",
+                      "06:06:06:06:06:06", "12:14",
+                      SHOULD_PASS_EXPIRED, LEASE_AFFINITY_ENABLED);
 }
 
 // This test checks the server's behavior in the following case:
@@ -274,6 +339,33 @@ TEST_F(ReleaseTest, releaseNonMatchingIPAddress) {
     ASSERT_TRUE(lease);
 }
 
+// This test checks the server's behavior in the following case:
+// - Client acquires new lease.
+// - Client sends DHCPRELEASE with the ciaddr set to a different
+//   address than it has acquired from the server.
+// - Server determines that the client is trying to release a
+//   wrong address and will refuse to release.
+TEST_F(ReleaseTest, releaseNoDeleteNonMatchingIPAddress) {
+    Dhcp4Client client(Dhcp4Client::SELECTING);
+    // Configure DHCP server.
+    configure(RELEASE_CONFIGS[0], *client.getServer(), true, true, true, false, LEASE_AFFINITY_ENABLED);
+    // Perform 4-way exchange to obtain a new lease.
+    acquireLease(client);
+
+    // Remember the acquired address.
+    IOAddress leased_address = client.config_.lease_.addr_;
+
+    // Modify the client's address to force it to release a different address
+    // than it has obtained from the server.
+    client.config_.lease_.addr_ = IOAddress(leased_address.toUint32() + 1);
+
+    // Send DHCPRELEASE and make sure it was unsuccessful, i.e. the lease
+    // remains in the database.
+    ASSERT_NO_THROW(client.doRelease());
+    Lease4Ptr lease = LeaseMgrFactory::instance().getLease4(leased_address);
+    ASSERT_TRUE(lease);
+}
+
 // This test verifies that an incoming RELEASE for an address within
 // a subnet that has been removed can still be released.
 TEST_F(ReleaseTest, releaseNoSubnet) {
@@ -297,4 +389,30 @@ TEST_F(ReleaseTest, releaseNoSubnet) {
     EXPECT_FALSE(lease);
 }
 
+// This test verifies that an incoming RELEASE for an address within
+// a subnet that has been removed can still be released.
+TEST_F(ReleaseTest, releaseNoDeleteNoSubnet) {
+    Dhcp4Client client(Dhcp4Client::SELECTING);
+    // Configure DHCP server.
+    configure(RELEASE_CONFIGS[0], *client.getServer(), true, true, true, false, LEASE_AFFINITY_ENABLED);
+    // Perform 4-way exchange to obtain a new lease.
+    acquireLease(client);
+
+    // Remember the acquired address.
+    IOAddress leased_address = client.config_.lease_.addr_;
+
+    // Release is as it was relayed
+    client.useRelay(true);
+
+    // Send the release
+    ASSERT_NO_THROW(client.doRelease());
+
+    // Check that the lease was not removed
+    Lease4Ptr lease = LeaseMgrFactory::instance().getLease4(leased_address);
+    ASSERT_TRUE(lease);
+
+    // Check That the lease has been expired
+    EXPECT_EQ(lease->valid_lft_, 0);
+}
+
 } // end of anonymous namespace
