@@ -55,6 +55,7 @@ extern const isc::log::MessageID DHCP6_DECLINE_FAIL_LEASE_WITHOUT_DUID;
 extern const isc::log::MessageID DHCP6_DECLINE_FAIL_NO_LEASE;
 extern const isc::log::MessageID DHCP6_DECLINE_LEASE;
 extern const isc::log::MessageID DHCP6_DECLINE_PROCESS_IA;
+extern const isc::log::MessageID DHCP6_DEPRECATED;
 extern const isc::log::MessageID DHCP6_DEVELOPMENT_VERSION;
 extern const isc::log::MessageID DHCP6_DHCP4O6_PACKET_RECEIVED;
 extern const isc::log::MessageID DHCP6_DHCP4O6_RECEIVE_FAIL;
@@ -137,10 +138,14 @@ extern const isc::log::MessageID DHCP6_QUERY_DATA;
 extern const isc::log::MessageID DHCP6_RAPID_COMMIT;
 extern const isc::log::MessageID DHCP6_RECLAIM_EXPIRED_LEASES_FAIL;
 extern const isc::log::MessageID DHCP6_RELEASE_NA;
+extern const isc::log::MessageID DHCP6_RELEASE_NA_DELETED;
+extern const isc::log::MessageID DHCP6_RELEASE_NA_EXPIRED;
 extern const isc::log::MessageID DHCP6_RELEASE_NA_FAIL;
 extern const isc::log::MessageID DHCP6_RELEASE_NA_FAIL_WRONG_DUID;
 extern const isc::log::MessageID DHCP6_RELEASE_NA_FAIL_WRONG_IAID;
 extern const isc::log::MessageID DHCP6_RELEASE_PD;
+extern const isc::log::MessageID DHCP6_RELEASE_PD_DELETED;
+extern const isc::log::MessageID DHCP6_RELEASE_PD_EXPIRED;
 extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL;
 extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL_WRONG_DUID;
 extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL_WRONG_IAID;
