@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,19 +11,17 @@
 #include <cc/command_interpreter.h>
 #include <config/command_mgr.h>
 #include <database/dbaccess_parser.h>
-#include <dhcp/libdhcp++.h>
-#include <dhcp6/json_config_parser.h>
+#include <dhcp6/ctrl_dhcp6_srv.h>
 #include <dhcp6/dhcp6_log.h>
 #include <dhcp6/dhcp6_srv.h>
+#include <dhcp6/json_config_parser.h>
+#include <dhcp/libdhcp++.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcpsrv/cb_ctl_dhcp4.h>
+#include <dhcpsrv/cfg_multi_threading.h>
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/db_type.h>
-#include <dhcpsrv/pool.h>
-#include <dhcpsrv/subnet.h>
-#include <dhcpsrv/timer_mgr.h>
-#include <dhcpsrv/triplet.h>
 #include <dhcpsrv/parsers/client_class_def_parser.h>
 #include <dhcpsrv/parsers/dhcp_parsers.h>
 #include <dhcpsrv/parsers/duid_config_parser.h>
@@ -38,13 +36,17 @@
 #include <dhcpsrv/parsers/shared_networks_list_parser.h>
 #include <dhcpsrv/parsers/sanity_checks_parser.h>
 #include <dhcpsrv/host_data_source_factory.h>
-#include <hooks/hooks_parser.h>
+#include <dhcpsrv/pool.h>
+#include <dhcpsrv/subnet.h>
+#include <dhcpsrv/timer_mgr.h>
 #include <hooks/hooks_manager.h>
+#include <hooks/hooks_parser.h>
 #include <log/logger_support.h>
 #include <process/config_ctl_parser.h>
-
 #include <util/encode/hex.h>
+#include <util/multi_threading_mgr.h>
 #include <util/strutil.h>
+#include <util/triplet.h>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/foreach.hpp>
@@ -67,6 +69,8 @@ using namespace isc::dhcp;
 using namespace isc::asiolink;
 using namespace isc::hooks;
 using namespace isc::process;
+using namespace isc::config;
+using namespace isc::util;
 
 namespace {
 
@@ -175,32 +179,32 @@ public:
     ///
     /// @throw DhcpConfigError if parameters are missing or
     /// or having incorrect values.
-    void parse(const SrvConfigPtr& srv_config, const ConstElementPtr& global) {
+    void parse(const SrvConfigPtr& cfg, const ConstElementPtr& global) {
 
         // Set the data directory for server id file.
         if (global->contains("data-directory")) {
-          CfgMgr::instance().setDataDir(getString(global, "data-directory"),
-                                        false);
+            CfgMgr::instance().setDataDir(getString(global, "data-directory"),
+                                          false);
         }
 
         // Set the probation period for decline handling.
         uint32_t probation_period =
             getUint32(global, "decline-probation-period");
-        srv_config->setDeclinePeriod(probation_period);
+        cfg->setDeclinePeriod(probation_period);
 
         // Set the DHCPv4-over-DHCPv6 interserver port.
         uint16_t dhcp4o6_port = getUint16(global, "dhcp4o6-port");
-        srv_config->setDhcp4o6Port(dhcp4o6_port);
+        cfg->setDhcp4o6Port(dhcp4o6_port);
 
         // Set the global user context.
         ConstElementPtr user_context = global->get("user-context");
         if (user_context) {
-            srv_config->setContext(user_context);
+            cfg->setContext(user_context);
         }
 
         // Set the server's logical name
         std::string server_tag = getString(global, "server-tag");
-        srv_config->setServerTag(server_tag);
+        cfg->setServerTag(server_tag);
     }
 
     /// @brief Sets global parameters before other parameters are parsed.
@@ -244,7 +248,7 @@ public:
         for (auto net = networks->begin(); net != networks->end(); ++net) {
 
             // For each network go through all the subnets in it.
-            const Subnet6Collection* subnets = (*net)->getAllSubnets();
+            const Subnet6SimpleCollection* subnets = (*net)->getAllSubnets();
             if (!subnets) {
                 // Shared network without subnets it weird, but we decided to
                 // accept such configurations.
@@ -264,7 +268,7 @@ public:
     /// in the future.
     ///
     /// @param cfg - the parsed structure
-    /// @param global global Dhcp4 scope
+    /// @param global global Dhcp6 scope
     /// @throw DhcpConfigError in case of issues found
     void
     sanityChecks(const SrvConfigPtr& cfg, const ConstElementPtr& global) {
@@ -308,7 +312,7 @@ public:
             // or don't have the interface specified at all.
             string iface = (*net)->getIface();
 
-            const Subnet6Collection* subnets = (*net)->getAllSubnets();
+            const Subnet6SimpleCollection* subnets = (*net)->getAllSubnets();
             if (subnets) {
 
                 bool rapid_commit = false;
@@ -414,37 +418,23 @@ void configureCommandChannel() {
         if (sock_cfg) {
             // This will create a control socket and install the external
             // socket in IfaceMgr. That socket will be monitored when
-            // Dhcp4Srv::receivePacket() calls IfaceMgr::receive4() and
+            // Dhcp6Srv::receivePacket() calls IfaceMgr::receive6() and
             // callback in CommandMgr will be called, if necessary.
             isc::config::CommandMgr::instance().openCommandSocket(sock_cfg);
         }
     }
 }
 
+/// @brief Process a DHCPv6 confguration and return an answer stating if the
+/// configuration is valid, or specifying details about the error otherwise.
+///
+/// @param config_set the configuration being processed
 isc::data::ConstElementPtr
-configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
-                     bool check_only) {
-    if (!config_set) {
-        ConstElementPtr answer = isc::config::createAnswer(1,
-                                 string("Can't parse NULL config"));
-        return (answer);
-    }
-
-    LOG_DEBUG(dhcp6_logger, DBG_DHCP6_COMMAND, DHCP6_CONFIG_START)
-        .arg(server.redactConfig(config_set)->str());
-
+processDhcp6Config(isc::data::ConstElementPtr config_set) {
     // Before starting any subnet operations, let's reset the subnet-id counter,
     // so newly recreated configuration starts with first subnet-id equal 1.
     Subnet::resetSubnetID();
 
-    // Close DHCP sockets and remove any existing timers.
-    if (!check_only) {
-        IfaceMgr::instance().closeSockets();
-        TimerMgr::instance()->unregisterTimers();
-        server.discardPackets();
-        server.getCBControl()->reset();
-    }
-
     // Revert any runtime option definitions configured so far and not committed.
     LibDHCP::revertRuntimeOptionDefs();
     // Let's set empty container in case a user hasn't specified any configuration
@@ -456,9 +446,7 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
 
     // Answer will hold the result.
     ConstElementPtr answer;
-    // Rollback informs whether error occurred and original data
-    // have to be restored to global storages.
-    bool rollback = false;
+
     // Global parameter name in case of an error.
     string parameter_name;
     ElementPtr mutable_cfg;
@@ -546,11 +534,28 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             parser.parse(*srv_config, multi_threading);
         }
 
+        bool multi_threading_enabled = true;
+        uint32_t thread_count = 0;
+        uint32_t queue_size = 0;
+        CfgMultiThreading::extract(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading(),
+                                   multi_threading_enabled, thread_count, queue_size);
+
+        /// depends on "multi-threading" being enabled, so it must come after.
         ConstElementPtr queue_control = mutable_cfg->get("dhcp-queue-control");
         if (queue_control) {
             parameter_name = "dhcp-queue-control";
             DHCPQueueControlParser parser;
-            srv_config->setDHCPQueueControl(parser.parse(queue_control));
+            srv_config->setDHCPQueueControl(parser.parse(queue_control, multi_threading_enabled));
+        }
+
+        /// depends on "multi-threading" being enabled, so it must come after.
+        ConstElementPtr reservations_lookup_first = mutable_cfg->get("reservations-lookup-first");
+        if (reservations_lookup_first) {
+            parameter_name = "reservations-lookup-first";
+            if (multi_threading_enabled) {
+                LOG_WARN(dhcp6_logger, DHCP6_RESERVATIONS_LOOKUP_FIRST_ENABLED);
+            }
+            srv_config->setReservationsLookupFirst(reservations_lookup_first->boolValue());
         }
 
         ConstElementPtr hr_identifiers =
@@ -569,14 +574,6 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             parser.parse(cfg, server_id);
         }
 
-        ConstElementPtr ifaces_config = mutable_cfg->get("interfaces-config");
-        if (ifaces_config) {
-            parameter_name = "interfaces-config";
-            IfacesConfigParser parser(AF_INET6, check_only);
-            CfgIfacePtr cfg_iface = srv_config->getCfgIface();
-            parser.parse(cfg_iface, ifaces_config);
-        }
-
         ConstElementPtr sanity_checks = mutable_cfg->get("sanity-checks");
         if (sanity_checks) {
             parameter_name = "sanity-checks";
@@ -601,7 +598,8 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             HooksLibrariesParser hooks_parser;
             HooksConfig& libraries = srv_config->getHooksConfig();
             hooks_parser.parse(libraries, hooks_libraries);
-            libraries.verifyLibraries(hooks_libraries->getPosition());
+            libraries.verifyLibraries(hooks_libraries->getPosition(),
+                                      multi_threading_enabled);
         }
 
         // D2 client configuration.
@@ -651,8 +649,8 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
         if (hosts_databases) {
             parameter_name = "hosts-databases";
             CfgDbAccessPtr cfg_db_access = srv_config->getCfgDbAccess();
-            db::DbAccessParser parser;
             for (auto it : hosts_databases->listValue()) {
+                db::DbAccessParser parser;
                 std::string access_string;
                 parser.parse(access_string, it);
                 cfg_db_access->setHostDbAccessString(access_string);
@@ -714,9 +712,20 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
         ConstElementPtr compatibility = mutable_cfg->get("compatibility");
         if (compatibility) {
             for (auto kv : compatibility->mapValue()) {
+                if (!kv.second || (kv.second->getType() != Element::boolean)) {
+                    isc_throw(DhcpConfigError,
+                              "compatibility parameter values must be "
+                              << "boolean (" << kv.first << " at "
+                              << kv.second->getPosition() << ")");
+                }
                 if (kv.first == "lenient-option-parsing") {
                     CfgMgr::instance().getStagingCfg()->setLenientOptionParsing(
                         kv.second->boolValue());
+                } else {
+                    isc_throw(DhcpConfigError,
+                              "unsupported compatibility parameter: "
+                              << kv.first << " (" << kv.second->getPosition()
+                              << ")");
                 }
             }
         }
@@ -754,6 +763,7 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
                 (config_pair.first == "reservations") ||
                 (config_pair.first == "config-control") ||
                 (config_pair.first == "relay-supplied-options") ||
+                (config_pair.first == "loggers") ||
                 (config_pair.first == "compatibility")) {
                 continue;
             }
@@ -788,7 +798,6 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
                  (config_pair.first == "t2-percent") ||
                  (config_pair.first == "cache-threshold") ||
                  (config_pair.first == "cache-max-age") ||
-                 (config_pair.first == "loggers") ||
                  (config_pair.first == "hostname-char-set") ||
                  (config_pair.first == "hostname-char-replacement") ||
                  (config_pair.first == "ddns-send-updates") ||
@@ -799,11 +808,16 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
                  (config_pair.first == "ddns-qualifying-suffix") ||
                  (config_pair.first == "ddns-update-on-renew") ||
                  (config_pair.first == "ddns-use-conflict-resolution") ||
+                 (config_pair.first == "ddns-ttl-percent") ||
                  (config_pair.first == "store-extended-info") ||
                  (config_pair.first == "statistic-default-sample-count") ||
                  (config_pair.first == "statistic-default-sample-age") ||
+                 (config_pair.first == "early-global-reservations-lookup") ||
                  (config_pair.first == "ip-reservations-unique") ||
-                 (config_pair.first == "parked-packet-limit")) {
+                 (config_pair.first == "reservations-lookup-first") ||
+                 (config_pair.first == "parked-packet-limit") ||
+                 (config_pair.first == "allocator") ||
+                 (config_pair.first == "pd-allocator") ) {
                 CfgMgr::instance().getStagingCfg()->addConfiguredGlobal(config_pair.first,
                                                                         config_pair.second);
                 continue;
@@ -840,26 +854,129 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
     } catch (const isc::Exception& ex) {
         LOG_ERROR(dhcp6_logger, DHCP6_PARSER_FAIL)
                   .arg(parameter_name).arg(ex.what());
-        answer = isc::config::createAnswer(1, ex.what());
-
-        // An error occurred, so make sure that we restore original data.
-        rollback = true;
+        answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
     } catch (...) {
         // For things like bad_cast in boost::lexical_cast
         LOG_ERROR(dhcp6_logger, DHCP6_PARSER_EXCEPTION).arg(parameter_name);
-        answer = isc::config::createAnswer(1, "undefined configuration"
-                                           " processing error");
-
-        // An error occurred, so make sure that we restore original data.
-        rollback = true;
+        answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration "
+                                           "processing error");
     }
 
-    if (check_only) {
-        rollback = true;
-        if (!answer) {
-            answer = isc::config::createAnswer(0,
-            "Configuration seems sane. Control-socket, hook-libraries, and D2 "
-            "configuration were sanity checked, but not applied.");
+    if (!answer) {
+        answer = isc::config::createAnswer(CONTROL_RESULT_SUCCESS, "Configuration seems sane. "
+                                           "Control-socket, hook-libraries, and D2 configuration "
+                                           "were sanity checked, but not applied.");
+    }
+
+    return (answer);
+}
+
+isc::data::ConstElementPtr
+configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
+                     bool check_only, bool extra_checks) {
+    if (!config_set) {
+        ConstElementPtr answer = isc::config::createAnswer(CONTROL_RESULT_ERROR,
+                                                           "Can't parse NULL config");
+        return (answer);
+    }
+
+    LOG_DEBUG(dhcp6_logger, DBG_DHCP6_COMMAND, DHCP6_CONFIG_START)
+        .arg(server.redactConfig(config_set)->str());
+
+    auto answer = processDhcp6Config(config_set);
+
+    int status_code = CONTROL_RESULT_SUCCESS;
+    isc::config::parseAnswer(status_code, answer);
+
+    SrvConfigPtr srv_config;
+
+    if (status_code == CONTROL_RESULT_SUCCESS) {
+        if (check_only) {
+            if (extra_checks) {
+                // Re-open lease and host database with new parameters.
+                try {
+                    // Get the staging configuration.
+                    srv_config = CfgMgr::instance().getStagingCfg();
+
+                    CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
+                    string params = "universe=6 persist=false";
+                    if (cfg_db->getExtendedInfoTablesEnabled()) {
+                        params += " extended-info-tables=true";
+                    }
+                    cfg_db->setAppendedParameters(params);
+                    cfg_db->createManagers();
+                } catch (const std::exception& ex) {
+                    answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+                    status_code = CONTROL_RESULT_ERROR;
+                }
+
+                if (status_code == CONTROL_RESULT_SUCCESS) {
+                    std::ostringstream err;
+                    // Configure DHCP packet queueing
+                    try {
+                        data::ConstElementPtr qc;
+                        qc = CfgMgr::instance().getStagingCfg()->getDHCPQueueControl();
+                        if (IfaceMgr::instance().configureDHCPPacketQueue(AF_INET6, qc)) {
+                            LOG_INFO(dhcp6_logger, DHCP6_CONFIG_PACKET_QUEUE)
+                                     .arg(IfaceMgr::instance().getPacketQueue6()->getInfoStr());
+                        }
+
+                    } catch (const std::exception& ex) {
+                        err << "Error setting packet queue controls after server reconfiguration: "
+                            << ex.what();
+                        answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str());
+                        status_code = CONTROL_RESULT_ERROR;
+                    }
+                }
+            }
+        } else {
+            // disable multi-threading (it will be applied by new configuration)
+            // this must be done in order to properly handle MT to ST transition
+            // when 'multi-threading' structure is missing from new config and
+            // to properly drop any task items stored in the thread pool which
+            // might reference some handles to loaded hooks, preventing them
+            // from being unloaded.
+            MultiThreadingMgr::instance().apply(false, 0, 0);
+
+            // Close DHCP sockets and remove any existing timers.
+            IfaceMgr::instance().closeSockets();
+            TimerMgr::instance()->unregisterTimers();
+            server.discardPackets();
+            server.getCBControl()->reset();
+        }
+
+        if (status_code == CONTROL_RESULT_SUCCESS) {
+            string parameter_name;
+            ElementPtr mutable_cfg;
+            try {
+                // Get the staging configuration.
+                srv_config = CfgMgr::instance().getStagingCfg();
+
+                // This is a way to convert ConstElementPtr to ElementPtr.
+                // We need a config that can be edited, because we will insert
+                // default values and will insert derived values as well.
+                mutable_cfg = boost::const_pointer_cast<Element>(config_set);
+
+                ConstElementPtr ifaces_config = mutable_cfg->get("interfaces-config");
+                if (ifaces_config) {
+                    parameter_name = "interfaces-config";
+                    IfacesConfigParser parser(AF_INET6, check_only);
+                    CfgIfacePtr cfg_iface = srv_config->getCfgIface();
+                    cfg_iface->reset();
+                    parser.parse(cfg_iface, ifaces_config);
+                }
+            } catch (const isc::Exception& ex) {
+                LOG_ERROR(dhcp6_logger, DHCP6_PARSER_FAIL)
+                          .arg(parameter_name).arg(ex.what());
+                answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+                status_code = CONTROL_RESULT_ERROR;
+            } catch (...) {
+                // For things like bad_cast in boost::lexical_cast
+                LOG_ERROR(dhcp6_logger, DHCP6_PARSER_EXCEPTION).arg(parameter_name);
+                answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
+                                                   " processing error");
+                status_code = CONTROL_RESULT_ERROR;
+            }
         }
     }
 
@@ -867,12 +984,26 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
     // configuration. This will add created subnets and option values into
     // the server's configuration.
     // This operation should be exception safe but let's make sure.
-    if (!rollback) {
+    if (status_code == CONTROL_RESULT_SUCCESS && (!check_only || extra_checks)) {
         try {
 
             // Setup the command channel.
             configureCommandChannel();
+        } catch (const isc::Exception& ex) {
+            LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_FAIL).arg(ex.what());
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+            status_code = CONTROL_RESULT_ERROR;
+        } catch (...) {
+            // For things like bad_cast in boost::lexical_cast
+            LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_EXCEPTION);
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
+                                               " parsing error");
+            status_code = CONTROL_RESULT_ERROR;
+        }
+    }
 
+    if (status_code == CONTROL_RESULT_SUCCESS && (!check_only || extra_checks)) {
+        try {
             // No need to commit interface names as this is handled by the
             // CfgMgr::commit() function.
 
@@ -880,7 +1011,21 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             D2ClientConfigPtr cfg;
             cfg = CfgMgr::instance().getStagingCfg()->getD2ClientConfig();
             CfgMgr::instance().setD2ClientConfig(cfg);
+        } catch (const isc::Exception& ex) {
+            LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_FAIL).arg(ex.what());
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+            status_code = CONTROL_RESULT_ERROR;
+        } catch (...) {
+            // For things like bad_cast in boost::lexical_cast
+            LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_EXCEPTION);
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
+                                               " parsing error");
+            status_code = CONTROL_RESULT_ERROR;
+        }
+    }
 
+    if (status_code == CONTROL_RESULT_SUCCESS && (!check_only || extra_checks)) {
+        try {
             // This occurs last as if it succeeds, there is no easy way to
             // revert it.  As a result, the failure to commit a subsequent
             // change causes problems when trying to roll back.
@@ -888,59 +1033,61 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             static_cast<void>(HooksManager::unloadLibraries());
             const HooksConfig& libraries =
                 CfgMgr::instance().getStagingCfg()->getHooksConfig();
-            libraries.loadLibraries();
-        }
-        catch (const isc::Exception& ex) {
+            bool multi_threading_enabled = true;
+            uint32_t thread_count = 0;
+            uint32_t queue_size = 0;
+            CfgMultiThreading::extract(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading(),
+                                       multi_threading_enabled, thread_count, queue_size);
+            libraries.loadLibraries(multi_threading_enabled);
+        } catch (const isc::Exception& ex) {
             LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_FAIL).arg(ex.what());
-            answer = isc::config::createAnswer(2, ex.what());
-
-            // An error occurred, so make sure to restore the original data.
-            rollback = true;
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+            status_code = CONTROL_RESULT_ERROR;
         } catch (...) {
             // For things like bad_cast in boost::lexical_cast
             LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_EXCEPTION);
-            answer = isc::config::createAnswer(2, "undefined configuration"
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
                                                " parsing error");
-
-            // An error occurred, so make sure to restore the original data.
-            rollback = true;
+            status_code = CONTROL_RESULT_ERROR;
         }
     }
 
     // Moved from the commit block to add the config backend indication.
-    if (!rollback) {
+    if (status_code == CONTROL_RESULT_SUCCESS && (!check_only || extra_checks)) {
         try {
-
             // If there are config backends, fetch and merge into staging config
             server.getCBControl()->databaseConfigFetch(srv_config,
                                                        CBControlDHCPv6::FetchMode::FETCH_ALL);
-        }
-        catch (const isc::Exception& ex) {
+        } catch (const isc::Exception& ex) {
             std::ostringstream err;
             err << "during update from config backend database: " << ex.what();
             LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_FAIL).arg(err.str());
-            answer = isc::config::createAnswer(2, err.str());
-
-            // An error occurred, so make sure to restore the original data.
-            rollback = true;
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str());
+            status_code = CONTROL_RESULT_ERROR;
         } catch (...) {
             // For things like bad_cast in boost::lexical_cast
             std::ostringstream err;
             err << "during update from config backend database: "
                 << "undefined configuration parsing error";
             LOG_ERROR(dhcp6_logger, DHCP6_PARSER_COMMIT_FAIL).arg(err.str());
-            answer = isc::config::createAnswer(2, err.str());
-
-            // An error occurred, so make sure to restore the original data.
-            rollback = true;
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str());
+            status_code = CONTROL_RESULT_ERROR;
         }
     }
 
     // Rollback changes as the configuration parsing failed.
-    if (rollback) {
+    if (check_only || status_code != CONTROL_RESULT_SUCCESS) {
         // Revert to original configuration of runtime option definitions
         // in the libdhcp++.
         LibDHCP::revertRuntimeOptionDefs();
+
+        if (status_code == CONTROL_RESULT_SUCCESS && extra_checks) {
+            auto notify_libraries = ControlledDhcpv6Srv::finishConfigHookLibraries(config_set);
+            if (notify_libraries) {
+                return (notify_libraries);
+            }
+        }
+
         return (answer);
     }
 
@@ -948,8 +1095,15 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
         .arg(CfgMgr::instance().getStagingCfg()->
              getConfigSummary(SrvConfig::CFGSEL_ALL6));
 
+    // Also calculate SHA256 hash of the config that was just set and
+    // append it to the response.
+    ConstElementPtr config = CfgMgr::instance().getStagingCfg()->toElement();
+    string hash = BaseCommandMgr::getHash(config);
+    ElementPtr hash_map = Element::createMap();
+    hash_map->set("hash", Element::create(hash));
+
     // Everything was fine. Configuration is successful.
-    answer = isc::config::createAnswer(0, "Configuration successful.");
+    answer = isc::config::createAnswer(CONTROL_RESULT_SUCCESS, "Configuration successful.", hash_map);
     return (answer);
 }
 
