@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,10 @@ static const int LIBRARY_NUMBER = 1;
 // issues related to namespaces.
 extern "C" {
 
+int (*do_load)(isc::hooks::LibraryHandle& handle);
+
+int (*do_unload)();
+
 /// @brief This function is called to retrieve the multi-threading compatibility.
 ///
 /// @return 1 which means compatible with multi-threading.
