@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -158,6 +158,7 @@ const char* CONFIGS[] = {
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
         "    \"subnet\": \"2001:db8:1::/48\", "
+        "    \"id\": 1, "
         "    \"interface\": \"eth1\","
         "    \"reservations\": ["
         "    {"
@@ -214,6 +215,7 @@ const char* CONFIGS[] = {
         "],"
         "\"subnet6\": [ "
         "{   \"subnet\": \"2001:db8:1::/48\", "
+        "    \"id\": 1, "
         "    \"interface\": \"eth1\","
         "    \"pools\": [ "
         "        { \"pool\": \"2001:db8:1:1::/64\","
@@ -274,6 +276,7 @@ const char* CONFIGS[] = {
         "],"
         "\"subnet6\": [ "
         "{   \"subnet\": \"2001:db8::/32\", "
+        "    \"id\": 1, "
         "    \"interface\": \"eth1\","
         "    \"pd-pools\": [ "
         "        { \"prefix\": \"2001:db8:1::\","
@@ -317,6 +320,7 @@ const char* CONFIGS[] = {
         "],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\""
         " } ],"
@@ -337,6 +341,7 @@ const char* CONFIGS[] = {
         "],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\","
         "    \"reservations\": ["
@@ -369,6 +374,7 @@ const char* CONFIGS[] = {
         "],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\","
         "    \"reservations\": ["
@@ -578,6 +584,7 @@ TEST_F(ClassifyTest, matchClassification) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\" } ],"
         "\"client-classes\": [ "
@@ -585,7 +592,13 @@ TEST_F(ClassifyTest, matchClassification) {
         "    \"option-data\": ["
         "        {    \"name\": \"ipv6-forwarding\", "
         "             \"data\": \"true\" } ], "
-        "    \"test\": \"option[host-name].text == 'foo'\" } ] }";
+        "    \"test\": \"option[host-name].text == 'foo'\" },"
+        "{   \"name\": \"template-client-id\","
+        "    \"template-test\": \"substring(option[1].hex,0,3)\" },"
+        "{   \"name\": \"SPAWN_template-hostname_foo\" },"
+        "{   \"name\": \"template-hostname\","
+        "    \"template-test\": \"option[host-name].text\"} ] }";
+
     ASSERT_NO_THROW(configure(config));
 
     // Create packets with enough to select the subnet
@@ -611,11 +624,35 @@ TEST_F(ClassifyTest, matchClassification) {
     srv.classifyPacket(query2);
     srv.classifyPacket(query3);
 
+    EXPECT_EQ(query1->classes_.size(), 6);
+    EXPECT_EQ(query2->classes_.size(), 3);
+    EXPECT_EQ(query3->classes_.size(), 6);
+
+    EXPECT_TRUE(query1->inClass("ALL"));
+    EXPECT_TRUE(query2->inClass("ALL"));
+    EXPECT_TRUE(query3->inClass("ALL"));
+
     // Packets with the exception of the second should be in the router class
     EXPECT_TRUE(query1->inClass("router"));
     EXPECT_FALSE(query2->inClass("router"));
     EXPECT_TRUE(query3->inClass("router"));
 
+    EXPECT_TRUE(query1->inClass("template-hostname"));
+    EXPECT_FALSE(query2->inClass("template-hostname"));
+    EXPECT_TRUE(query3->inClass("template-hostname"));
+
+    EXPECT_TRUE(query1->inClass("SPAWN_template-hostname_foo"));
+    EXPECT_FALSE(query2->inClass("SPAWN_template-hostname_foo"));
+    EXPECT_TRUE(query3->inClass("SPAWN_template-hostname_foo"));
+
+    EXPECT_TRUE(query1->inClass("template-client-id"));
+    EXPECT_TRUE(query2->inClass("template-client-id"));
+    EXPECT_TRUE(query3->inClass("template-client-id"));
+
+    EXPECT_TRUE(query1->inClass("SPAWN_template-client-id_def"));
+    EXPECT_TRUE(query2->inClass("SPAWN_template-client-id_def"));
+    EXPECT_TRUE(query3->inClass("SPAWN_template-client-id_def"));
+
     // Process queries
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
@@ -672,6 +709,7 @@ TEST_F(ClassifyTest, required) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\" } ],"
         "\"client-classes\": [ "
@@ -779,6 +817,7 @@ TEST_F(ClassifyTest, requiredClassification) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"require-client-classes\": [ \"router\" ], "
         "    \"interface\": \"eth1\" } ],"
@@ -892,6 +931,7 @@ TEST_F(ClassifyTest, subnetClassPriority) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\", "
         "    \"option-data\": ["
@@ -970,6 +1010,7 @@ TEST_F(ClassifyTest, subnetGlobalPriority) {
         "         \"data\": \"false\" } ], "
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\", "
         "    \"option-data\": ["
@@ -1033,6 +1074,7 @@ TEST_F(ClassifyTest, classGlobalPriority) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\" } ],"
         "\"option-data\": ["
@@ -1111,15 +1153,18 @@ TEST_F(ClassifyTest, classGlobalPersistency) {
         "\"option-data\": ["
         "    {    \"name\": \"ipv6-forwarding\", "
         "         \"data\": \"false\", "
-        "         \"always-send\": true } ], "
+        "         \"always-send\": true, "
+        "         \"never-send\": false } ], "
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\", "
         "    \"option-data\": ["
         "        {    \"name\": \"ipv6-forwarding\", "
         "             \"data\": \"false\", "
-        "             \"always-send\": false } ] } ] }";
+        "             \"always-send\": false, "
+        "             \"never-send\": false } ] } ] }";
     ASSERT_NO_THROW(configure(config));
 
     // Create a packet with enough to select the subnet and go through
@@ -1152,6 +1197,71 @@ TEST_F(ClassifyTest, classGlobalPersistency) {
     EXPECT_EQ(0, opt->getUint8());
 }
 
+// Checks class never-send options have the priority over everything else.
+TEST_F(ClassifyTest, classNeverSend) {
+    IfaceMgrTestConfig test_config(true);
+
+    NakedDhcpv6Srv srv(0);
+
+    // Subnet sets an ipv6-forwarding option in the response.
+    // The router class matches incoming packets with foo in a host-name
+    // option (code 1234) and sets an ipv6-forwarding option in the response.
+    // Note the cancellation flag follows a "OR" semantic so to set
+    // it to false (or to leave the default) has no effect.
+    std::string config = "{ \"interfaces-config\": {"
+        "    \"interfaces\": [ \"*\" ] }, "
+        "\"preferred-lifetime\": 3000,"
+        "\"rebind-timer\": 2000, "
+        "\"renew-timer\": 1000, "
+        "\"valid-lifetime\": 4000, "
+        "\"option-def\": [ "
+        "{   \"name\": \"host-name\","
+        "    \"code\": 1234,"
+        "    \"type\": \"string\" },"
+        "{   \"name\": \"ipv6-forwarding\","
+        "    \"code\": 2345,"
+        "    \"type\": \"boolean\" }],"
+        "\"option-data\": ["
+        "    {    \"name\": \"ipv6-forwarding\", "
+        "         \"data\": \"false\", "
+        "         \"always-send\": true, "
+        "         \"never-send\": false } ], "
+        "\"subnet6\": [ "
+        "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
+        "    \"subnet\": \"2001:db8:1::/48\", "
+        "    \"interface\": \"eth1\", "
+        "    \"option-data\": ["
+        "        {    \"name\": \"ipv6-forwarding\", "
+        "             \"always-send\": false, "
+        "             \"never-send\": true } ] } ] }";
+    ASSERT_NO_THROW(configure(config));
+
+    // Create a packet with enough to select the subnet and go through
+    // the SOLICIT processing
+    Pkt6Ptr query = createSolicit();
+
+    // Do not add an ORO.
+    OptionPtr oro = query->getOption(D6O_ORO);
+    EXPECT_FALSE(oro);
+
+    // Create and add a host-name option to the query
+    OptionStringPtr hostname(new OptionString(Option::V6, 1234, "foo"));
+    ASSERT_TRUE(hostname);
+    query->addOption(hostname);
+
+    // Process the query
+    AllocEngine::ClientContext6 ctx;
+    bool drop = !srv.earlyGHRLookup(query, ctx);
+    ASSERT_FALSE(drop);
+    srv.initContext(query, ctx,  drop);
+    ASSERT_FALSE(drop);
+    Pkt6Ptr response = srv.processSolicit(ctx);
+
+    // Processing should not add an ip-forwarding option
+    EXPECT_FALSE(response->getOption(2345));
+}
+
 // Checks if the client-class field is indeed used for subnet selection.
 // Note that packet classification is already checked in ClassifyTest
 // .*Classification above.
@@ -1174,10 +1284,12 @@ TEST_F(ClassifyTest, clientClassifySubnet) {
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ "
         " {  \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"client-class\": \"foo\" "
         " }, "
         " {  \"pools\": [ { \"pool\": \"2001:db8:2::/64\" } ],"
+        "    \"id\": 2, "
         "    \"subnet\": \"2001:db8:2::/48\", "
         "    \"client-class\": \"xyzzy\" "
         " } "
@@ -1244,6 +1356,7 @@ TEST_F(ClassifyTest, clientClassifyPool) {
         "       \"client-class\": \"xyzzy\" "
         "    } "
         "   ], "
+        "   \"id\": 1, "
         "   \"subnet\": \"2001:db8::/40\" "
         " } "
         "], "
@@ -1328,6 +1441,7 @@ TEST_F(ClassifyTest, clientClassifyPoolKnown) {
         "       \"client-class\": \"UNKNOWN\" "
         "    } "
         "   ], "
+        "   \"id\": 1, "
         "   \"subnet\": \"2001:db8::/40\", "
         "   \"reservations\": [ "
         "      { \"duid\": \"01:02:03:04\", \"hostname\": \"foo\" } ] "
@@ -1439,6 +1553,7 @@ TEST_F(ClassifyTest, relayOverrideAndClientClass) {
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ "
         " {  \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"client-class\": \"foo\", "
         "    \"relay\": { "
@@ -1446,6 +1561,7 @@ TEST_F(ClassifyTest, relayOverrideAndClientClass) {
         "    }"
         " }, "
         " {  \"pools\": [ { \"pool\": \"2001:db8:2::/64\" } ],"
+        "    \"id\": 2, "
         "    \"subnet\": \"2001:db8:2::/48\", "
         "    \"relay\": { "
         "        \"ip-address\": \"2001:db8:3::1\""
@@ -1587,6 +1703,7 @@ TEST_F(ClassifyTest, member) {
         "    \"type\": \"boolean\" }],"
         "\"subnet6\": [ "
         "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"id\": 1, "
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\" } ],"
         "\"client-classes\": [ "
