@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -80,8 +80,7 @@ public:
         // generateClientId assigns DUID to duid_.
         generateClientId();
         lease_.reset(new Lease6(Lease::TYPE_NA, IOAddress("2001:db8:1::1"),
-                                duid_, 1234, 501, 502,
-                                1, HWAddrPtr(), 0));
+                                duid_, 1234, 501, 502, 1, HWAddrPtr()));
         // Config DDNS to be enabled, all controls off
         enableD2();
     }
@@ -261,7 +260,6 @@ public:
         pkt->addOption(opt_ia);
     }
 
-
     /// @brief Adds IA option to the message.
     ///
     /// Added option holds status code.
@@ -385,7 +383,7 @@ public:
             // Context flags are normally set during lease allocation. Since that
             // hasn't occurred we'll set them here to match the expected values.
             // Call createNameChangeRequests
-            ctx.fwd_dns_update_ =  exp_fwd.value_;
+            ctx.fwd_dns_update_ = exp_fwd.value_;
             ctx.rev_dns_update_ = exp_rev.value_;
             ASSERT_NO_THROW(srv_->createNameChangeRequests(answer, ctx));
             if (exp_fwd.value_ || exp_rev.value_) {
@@ -423,13 +421,14 @@ public:
             "{ \"interfaces-config\": { \n"
             "  \"interfaces\": [ \"eth0\" ] \n"
             "}, \n"
-            "\"valid-lifetime\": 4000,  \n"
+            "\"valid-lifetime\": 4000, \n"
             "\"preferred-lifetime\": 3000, \n"
-            "\"rebind-timer\": 2000,  \n"
-            "\"renew-timer\": 1000,  \n"
-            "\"subnet6\": [ {  \n"
+            "\"rebind-timer\": 2000, \n"
+            "\"renew-timer\": 1000, \n"
+            "\"subnet6\": [ { \n"
+            "    \"id\": 1, \n"
             "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], \n"
-            "    \"subnet\": \"2001:db8:1::/48\",  \n"
+            "    \"subnet\": \"2001:db8:1::/48\", \n"
             "    \"interface\": \"eth0\" \n"
             " } ], \n"
             "\"dhcp-ddns\": { \n"
@@ -440,7 +439,7 @@ public:
 
         // Create the configuration and configure the server
         char config_buf[1024];
-        sprintf(config_buf, config_template, mode);
+        snprintf(config_buf, 1024, config_template, mode);
         configure(config_buf, *srv_);
 
         // Build our client packet
@@ -484,7 +483,6 @@ public:
         }
     }
 
-
     /// @brief Tests that the client's message holding an FQDN is processed
     /// and that lease is acquired.
     ///
@@ -523,18 +521,18 @@ public:
 
         ASSERT_FALSE(drop);
         if (msg_type == DHCPV6_SOLICIT) {
-          ASSERT_NO_THROW(reply = srv_->processSolicit(ctx));
+            ASSERT_NO_THROW(reply = srv_->processSolicit(ctx));
 
         } else if (msg_type == DHCPV6_REQUEST) {
-          ASSERT_NO_THROW(reply = srv_->processRequest(ctx));
+            ASSERT_NO_THROW(reply = srv_->processRequest(ctx));
 
         } else if (msg_type == DHCPV6_RENEW) {
-          ASSERT_NO_THROW(reply = srv_->processRenew(ctx));
+            ASSERT_NO_THROW(reply = srv_->processRenew(ctx));
 
         } else if (msg_type == DHCPV6_RELEASE) {
             // For Release no lease will be acquired so we have to leave
             // function here.
-          ASSERT_NO_THROW(reply = srv_->processRelease(ctx));
+            ASSERT_NO_THROW(reply = srv_->processRelease(ctx));
             return;
         } else {
             // We are not interested in testing other message types.
@@ -600,6 +598,8 @@ public:
     /// @param fqdn The expected string value of the FQDN, if blank the
     /// check is skipped
     /// @param exp_use_cr expected value of NCR::conflict_resolution_
+    /// @param ddns_ttl_percent expected value of ddns_ttl_percent used for
+    /// the NCR
     void verifyNameChangeRequest(const isc::dhcp_ddns::NameChangeType type,
                                  const bool reverse, const bool forward,
                                  const std::string& addr,
@@ -607,7 +607,9 @@ public:
                                  const uint64_t expires,
                                  const uint16_t valid_lft,
                                  const std::string& fqdn = "",
-                                 const bool exp_use_cr = true) {
+                                 const bool exp_use_cr = true,
+                                 util::Optional<double> exp_ddns_ttl_percent
+                                 = util::Optional<double>()) {
         NameChangeRequestPtr ncr;
         ASSERT_NO_THROW(ncr = d2_mgr_.peekAt(0));
         ASSERT_TRUE(ncr);
@@ -620,7 +622,7 @@ public:
             EXPECT_EQ(dhcid, ncr->getDhcid().toStr());
         }
 
-        uint32_t ttl = calculateDdnsTtl(valid_lft);
+        uint32_t ttl = calculateDdnsTtl(valid_lft, exp_ddns_ttl_percent);
         if (expires != 0) {
             EXPECT_EQ(expires + ttl, ncr->getLeaseExpiresOn());
         }
@@ -663,7 +665,7 @@ public:
 
         const PoolCollection& pool_col = subnet_->getPools(type);
         ASSERT_EQ(pool_idx + 1, pool_col.size());
-        PoolPtr pool  = (subnet_->getPools(type)).at(pool_idx);
+        PoolPtr pool = (subnet_->getPools(type)).at(pool_idx);
         ASSERT_TRUE(pool);
         pool_ = boost::dynamic_pointer_cast<Pool6>(pool);
         ASSERT_TRUE(pool);
@@ -756,7 +758,6 @@ TEST_F(FqdnDhcpv6SrvTest, createNameChangeRequestsNoAnswer) {
     ctx.fwd_dns_update_ = ctx.rev_dns_update_ = true;
     EXPECT_THROW(srv_->createNameChangeRequests(answer, ctx),
                  isc::Unexpected);
-
 }
 
 // Test that exception is thrown if supplied answer from the server
@@ -772,7 +773,6 @@ TEST_F(FqdnDhcpv6SrvTest, createNameChangeRequestsNoDUID) {
     ctx.subnet_ = subnet_;
     ctx.fwd_dns_update_ = ctx.rev_dns_update_ = true;
     EXPECT_THROW(srv_->createNameChangeRequests(answer, ctx), isc::Unexpected);
-
 }
 
 // Test no NameChangeRequests if Client FQDN is not added to the server's
@@ -944,7 +944,6 @@ TEST_F(FqdnDhcpv6SrvTest, noRemovalsWhenDisabled) {
     ASSERT_NO_THROW(queueNCR(CHG_REMOVE, lease_));
 }
 
-
 // Test creation of the NameChangeRequest to remove reverse mapping for the
 // given lease.
 TEST_F(FqdnDhcpv6SrvTest, createRemovalNameChangeRequestRev) {
@@ -961,7 +960,6 @@ TEST_F(FqdnDhcpv6SrvTest, createRemovalNameChangeRequestRev) {
                             "000201415AA33D1187D148275136FA30300478"
                             "FAAAA3EBD29826B5C907B2C9268A6F52",
                             lease_->cltt_, lease_->valid_lft_);
-
 }
 
 // Test that NameChangeRequest to remove DNS records is not generated when
@@ -973,7 +971,6 @@ TEST_F(FqdnDhcpv6SrvTest, createRemovalNameChangeRequestNoUpdate) {
     ASSERT_NO_THROW(queueNCR(CHG_REMOVE, lease_));
 
     ASSERT_EQ(0, d2_mgr_.getQueueSize());
-
 }
 
 // Test that NameChangeRequest is not generated if the hostname hasn't been
@@ -987,7 +984,6 @@ TEST_F(FqdnDhcpv6SrvTest, createRemovalNameChangeRequestNoHostname) {
     ASSERT_NO_THROW(queueNCR(CHG_REMOVE, lease_));
 
     ASSERT_EQ(0, d2_mgr_.getQueueSize());
-
 }
 
 // Test that NameChangeRequest is not generated if the invalid hostname has
@@ -1001,7 +997,6 @@ TEST_F(FqdnDhcpv6SrvTest, createRemovalNameChangeRequestWrongHostname) {
     ASSERT_NO_THROW(queueNCR(CHG_REMOVE, lease_));
 
     ASSERT_EQ(0, d2_mgr_.getQueueSize());
-
 }
 
 // Test that Advertise message generated in a response to the Solicit will
@@ -1038,7 +1033,6 @@ TEST_F(FqdnDhcpv6SrvTest, processTwoRequestsDiffFqdn) {
                             "FAAAA3EBD29826B5C907B2C9268A6F52",
                             0, 4000);
 
-
     // Client may send another request message with a new domain-name. In this
     // case the same lease will be returned. The existing DNS entry needs to
     // be replaced with a new one. Server should determine that the different
@@ -1059,7 +1053,6 @@ TEST_F(FqdnDhcpv6SrvTest, processTwoRequestsDiffFqdn) {
                             "000201D422AA463306223D269B6CB7AFE7AAD265FC"
                             "EA97F93623019B2E0D14E5323D5A",
                             0, lease_->valid_lft_);
-
 }
 
 // Test that client may send two requests, each carrying FQDN option with
@@ -1085,7 +1078,6 @@ TEST_F(FqdnDhcpv6SrvTest, processTwoRequestsSameFqdn) {
                             "FAAAA3EBD29826B5C907B2C9268A6F52",
                             0, 4000);
 
-
     // Client may send another request message with a same domain-name. In this
     // case the same lease will be returned. The existing DNS entry should be
     // left alone, so we expect no NameChangeRequests queued..
@@ -1120,10 +1112,8 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestSolicit) {
     testProcessMessage(DHCPV6_SOLICIT, "otherhost.example.com",
                        "otherhost.example.com.");
     ASSERT_EQ(0, d2_mgr_.getQueueSize());
-
 }
 
-
 // Test that client may send Request followed by the Renew, both holding
 // FQDN options, but each option holding different domain-name. The Renew
 // should result in generation of the two NameChangeRequests, one to remove
@@ -1169,7 +1159,6 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenewDiffFqdn) {
                             "000201D422AA463306223D269B6CB7AFE7AAD265FC"
                             "EA97F93623019B2E0D14E5323D5A",
                             0, lease_->valid_lft_);
-
 }
 
 // Test that client may send Request followed by the Renew, both holding
@@ -1236,7 +1225,6 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenewFqdnFlags) {
     // Queue should be empty.
     ASSERT_EQ(0, d2_mgr_.getQueueSize());
 
-
     // Renew with both N and S flags = 0. This tells the server to update
     // both directions, which should change forward flag to true. This should
     // generate a reverse only remove and a dual add.
@@ -1276,8 +1264,10 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenewFqdnFlags) {
                             lease_->cltt_, lease_->valid_lft_);
 }
 
-
 TEST_F(FqdnDhcpv6SrvTest, processRequestRelease) {
+    CfgMgr::instance().getCurrentCfg()->getCfgExpiration()->setFlushReclaimedTimerWaitTime(0);
+    CfgMgr::instance().getCurrentCfg()->getCfgExpiration()->setHoldReclaimedTime(0);
+
     // Create a Request message with FQDN option and generate server's
     // response using processRequest function. This will result in the
     // creation of a new lease and the appropriate NameChangeRequest
@@ -1311,6 +1301,33 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRelease) {
                             lease_->cltt_, lease_->valid_lft_);
 }
 
+TEST_F(FqdnDhcpv6SrvTest, processRequestReleaseNoDelete) {
+    // Create a Request message with FQDN option and generate server's
+    // response using processRequest function. This will result in the
+    // creation of a new lease and the appropriate NameChangeRequest
+    // to add both reverse and forward mapping to DNS.
+    testProcessMessage(DHCPV6_REQUEST, "myhost.example.com",
+                       "myhost.example.com.");
+
+    // The lease should have been recorded in the database.
+    lease_ = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
+                                                   IOAddress("2001:db8:1:1::dead:beef"));
+    ASSERT_TRUE(lease_);
+
+    ASSERT_EQ(1, d2_mgr_.getQueueSize());
+    verifyNameChangeRequest(isc::dhcp_ddns::CHG_ADD, true, true,
+                            "2001:db8:1:1::dead:beef",
+                            "000201415AA33D1187D148275136FA30300478"
+                            "FAAAA3EBD29826B5C907B2C9268A6F52",
+                            0, lease_->valid_lft_);
+
+    // Client may send Release message. In this case the lease should be
+    // expired and no NameChangeRequest to remove DNS entries is generated.
+    testProcessMessage(DHCPV6_RELEASE, "otherhost.example.com",
+                       "otherhost.example.com.");
+    ASSERT_EQ(0, d2_mgr_.getQueueSize());
+}
+
 // Checks that the server include DHCPv6 Client FQDN option in its
 // response even when client doesn't request this option using ORO.
 TEST_F(FqdnDhcpv6SrvTest, processRequestWithoutFqdn) {
@@ -1341,7 +1358,6 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestEmptyFqdn) {
                             "000201C905E54BE12DE6AF92ADE72752B9F362"
                             "13B5A8BC9D217548CD739B4CF31AFB1B",
                             0, 4000);
-
 }
 
 // Checks that when the server reuses expired lease, the NameChangeRequest
@@ -1354,8 +1370,8 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestReuseExpiredLease) {
     // exactly one address. This address will be handed out to the
     // client, will get expired and then be reused.
     CfgMgr::instance().clear();
-    subnet_ = Subnet6Ptr(new Subnet6(IOAddress("2001:db8:1:1::"), 56, 1, 2,
-                                     3, 4));
+    subnet_ = Subnet6::create(IOAddress("2001:db8:1:1::"),
+                              56, 1, 2, 3, 4, SubnetID(10));
     subnet_->setIface("eth0");
     subnet_->setDdnsSendUpdates(true);
 
@@ -1419,7 +1435,6 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestReuseExpiredLease) {
                             "2001:db8:1:1::dead:beef",
                             "000201415AA33D1187D148275136FA30300478"
                             "FAAAA3EBD29826B5C907B2C9268A6F52", 0, 4);
-
 }
 
 TEST_F(FqdnDhcpv6SrvTest, processClientDelegation) {
@@ -1448,6 +1463,7 @@ TEST_F(FqdnDhcpv6SrvTest, hostnameReservationSuffix) {
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ "
         " { "
+        "    \"id\": 1, \n"
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"pools\": [ { \"pool\": \"2001:db8:1:1::/64\" } ],"
         "    \"interface\" : \"eth0\" , "
@@ -1502,6 +1518,7 @@ TEST_F(FqdnDhcpv6SrvTest, hostnameReservationNoSuffix) {
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ "
         " { "
+        "    \"id\": 1, \n"
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"pools\": [ { \"pool\": \"2001:db8:1:1::/64\" } ],"
         "    \"interface\" : \"eth0\" , "
@@ -1536,7 +1553,6 @@ TEST_F(FqdnDhcpv6SrvTest, hostnameReservationNoSuffix) {
                             "000201E2EB74FB53A5778E74AFD43870ECA5"
                             "4150B1F52B0CFED434802DA1259D6D3CA4",
                             0, 4000, "alice.example.com.");
-
 }
 
 TEST_F(FqdnDhcpv6SrvTest, hostnameReservationDdnsDisabled) {
@@ -1552,6 +1568,7 @@ TEST_F(FqdnDhcpv6SrvTest, hostnameReservationDdnsDisabled) {
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ "
         " { "
+        "    \"id\": 1, \n"
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"pools\": [ { \"pool\": \"2001:db8:1:1::/64\" } ],"
         "    \"interface\" : \"eth0\" , "
@@ -1607,7 +1624,6 @@ TEST_F(FqdnDhcpv6SrvTest, replaceClientNameModeTest) {
                               CLIENT_NAME_PRESENT, NAME_NOT_REPLACED);
 }
 
-
 // Verifies that setting hostname-char-set sanitizes FQDN option
 // values received from clients.
 TEST_F(FqdnDhcpv6SrvTest, sanitizeFqdn) {
@@ -1640,7 +1656,6 @@ TEST_F(FqdnDhcpv6SrvTest, sanitizeFqdn) {
              std::string("m\000yhost.exa\000mple.com", 20),
              Option6ClientFqdn::FULL, Option6ClientFqdn::FLAG_S,
              "mxyhost.exaxmple.com.", false);
-
 }
 
 // Verifies that scoped ddns-parameter handling.
@@ -1656,11 +1671,13 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsScopeTest) {
         "\"renew-timer\": 1000,\n"
         "\"ddns-send-updates\": false,\n"
         "\"subnet6\": [ {\n"
+        "    \"id\": 1, \n"
         "    \"subnet\": \"2001:db8:1::/48\",\n"
         "    \"pools\": [ { \"pool\": \"2001:db8:1::1 - 2001:db8:1::10\" } ],\n"
         "    \"interface\": \"eth0\"\n"
         " },\n"
         " {\n"
+        "    \"id\": 2, \n"
         "    \"subnet\": \"2001:db8:2::/48\",\n"
         "    \"pools\": [ { \"pool\": \"2001:db8:2::1 - 2001:db8:2::10\" } ],\n"
         "    \"interface\": \"eth1\",\n"
@@ -1727,7 +1744,6 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsScopeTest) {
     ASSERT_EQ(1, CfgMgr::instance().getD2ClientMgr().getQueueSize());
     verifyNameChangeRequest(isc::dhcp_ddns::CHG_ADD, true, true, "2001:db8:2::1",
                             "", 0, 4000);
-
 }
 
 // Verifies that the DDNS parameters used for a lease in subnet in
@@ -1754,12 +1770,14 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsSharedNetworkTest) {
             "\"name\": \"frog\", \n"
             "\"interface\": \"eth0\", \n"
             "\"subnet6\": [ { \n"
+                "\"id\": 1, \n"
                 "\"subnet\": \"2001:db8:1::/64\", \n"
                 "\"pools\": [ { \"pool\": \"2001:db8:1::1 - 2001:db8:1::1\" } ], \n"
                 "\"interface\": \"eth0\", \n"
                 "\"ddns-qualifying-suffix\": \"one.example.com.\" \n"
             " }, \n"
             " { \n"
+                "\"id\": 2, \n"
                 "\"subnet\": \"2001:db8:2::/64\", \n"
                 "\"pools\": [ { \"pool\": \"2001:db8:2::1 - 2001:db8:2::1\" } ], \n"
                 "\"interface\": \"eth0\", \n"
@@ -1804,7 +1822,7 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsSharedNetworkTest) {
 
     // Make sure the lease hostname and fqdn flags are correct.
     Lease6Ptr lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
-                                                   IOAddress("2001:db8:1::1"));
+                                                            IOAddress("2001:db8:1::1"));
     ASSERT_TRUE(lease);
     EXPECT_EQ("client1.one.example.com.", lease->hostname_);
     EXPECT_TRUE(lease->fqdn_fwd_);
@@ -1908,6 +1926,7 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsSharedNetworkTest2) {
             "\"name\": \"frog\", \n"
             "\"interface\": \"eth0\", \n"
             "\"subnet6\": [ { \n"
+                "\"id\": 1, \n"
                 "\"subnet\": \"2001:db8:1::/64\", \n"
                 "\"pools\": [ { \"pool\": \"2001:db8:1::1 - 2001:db8:1::1\" } ], \n"
                 "\"interface\": \"eth0\", \n"
@@ -1915,6 +1934,7 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsSharedNetworkTest2) {
                 "\"ddns-send-updates\": true \n"
             " }, \n"
             " { \n"
+                "\"id\": 2, \n"
                 "\"subnet\": \"2001:db8:2::/64\", \n"
                 "\"pools\": [ { \"pool\": \"2001:db8:2::1 - 2001:db8:2::1\" } ], \n"
                 "\"interface\": \"eth0\", \n"
@@ -1959,7 +1979,7 @@ TEST_F(FqdnDhcpv6SrvTest, ddnsSharedNetworkTest2) {
 
     // Make sure the lease hostname and fdqn flags are correct.
     Lease6Ptr lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
-                                                   IOAddress("2001:db8:1::1"));
+                                                            IOAddress("2001:db8:1::1"));
     ASSERT_TRUE(lease);
     EXPECT_EQ("client1.one.example.com.", lease->hostname_);
     EXPECT_TRUE(lease->fqdn_fwd_);
@@ -2055,7 +2075,7 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenew) {
         SCOPED_TRACE(scenario.description_); {
             // Make sure the lease does not exist.
             ASSERT_FALSE(LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
-                                          IOAddress("2001:db8:1:1::dead:beef")));
+                                                               IOAddress("2001:db8:1:1::dead:beef")));
             // Set and verify DDNS params flags
             subnet_->setDdnsSendUpdates(scenario.send_updates_);
             subnet_->setDdnsUpdateOnRenew(scenario.update_on_renew_);
@@ -2068,7 +2088,7 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenew) {
 
             // The lease should have been recorded in the database.
             Lease6Ptr old_lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
-                                                     IOAddress("2001:db8:1:1::dead:beef"));
+                                                                        IOAddress("2001:db8:1:1::dead:beef"));
             ASSERT_TRUE(old_lease);
 
             if (!scenario.send_updates_ || scenario.old_fqdn_.empty()) {
@@ -2088,7 +2108,7 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenew) {
 
             // The lease should have been recorded in the database.
             Lease6Ptr new_lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
-                                                   IOAddress("2001:db8:1:1::dead:beef"));
+                                                                        IOAddress("2001:db8:1:1::dead:beef"));
             ASSERT_TRUE(new_lease);
 
             // Verify queue count is correct.
@@ -2120,4 +2140,38 @@ TEST_F(FqdnDhcpv6SrvTest, processRequestRenew) {
     }
 }
 
+// Verify that when specified ddns-ttl-percent is used to calculate
+// the lease length in an NCR.
+TEST_F(FqdnDhcpv6SrvTest, ddnsTtlPercent) {
+    // Create Reply message with Client Id and Server id.
+    Pkt6Ptr answer = generateMessageWithIds(DHCPV6_REPLY);
+
+    // Create an IA.
+    addIA(1234, IOAddress("2001:db8:1::1"), answer);
+
+    // Use domain name in upper case. It should be converted to lower-case
+    // before DHCID is calculated. So, we should get the same result as if
+    // we typed domain name in lower-case.
+    Option6ClientFqdnPtr fqdn = createClientFqdn(Option6ClientFqdn::FLAG_S,
+                                                 "MYHOST.EXAMPLE.COM",
+                                                 Option6ClientFqdn::FULL);
+    answer->addOption(fqdn);
+
+    // Create NameChangeRequest for the first allocated address.
+    AllocEngine::ClientContext6 ctx;
+    subnet_->setDdnsUseConflictResolution(false);
+    subnet_->setDdnsTtlPercent(Optional<double>(0.10));
+    ctx.subnet_ = subnet_;
+    ctx.fwd_dns_update_ = ctx.rev_dns_update_ = true;
+    ASSERT_NO_THROW(srv_->createNameChangeRequests(answer, ctx));
+    ASSERT_EQ(1, d2_mgr_.getQueueSize());
+
+    // Verify that NameChangeRequest is correct.
+    verifyNameChangeRequest(isc::dhcp_ddns::CHG_ADD, true, true,
+                            "2001:db8:1::1",
+                            "000201415AA33D1187D148275136FA30300478"
+                            "FAAAA3EBD29826B5C907B2C9268A6F52",
+                            0, 500, "", false, subnet_->getDdnsTtlPercent());
+}
+
 } // end of anonymous namespace
