@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -217,10 +217,10 @@ public:
     void runTimersWithTimeout(const IOServicePtr& io_service, const long timeout_ms,
                               std::function<bool()> cond = std::function<bool()>()) {
         IntervalTimer timer(*io_service);
-        bool stopped = false;
+        std::atomic<bool> stopped(false);
         timer.setup([&io_service, &stopped]() {
-            io_service->stop();
             stopped = true;
+            io_service->stop();
         }, timeout_ms, IntervalTimer::ONE_SHOT);
 
         // Run as long as the timeout hasn't occurred and the interrupting
@@ -384,15 +384,17 @@ TEST_F(JSONFileBackendTest, jsonFile) {
         "\"rebind-timer\": 2000, "
         "\"renew-timer\": 1000, "
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " },"
         " {"
+        "    \"id\": 2, "
         "    \"pools\": [ { \"pool\": \"2001:db8:2::/80\" } ],"
         "    \"subnet\": \"2001:db8:2::/64\", "
-        "    \"id\": 0"
         " },"
         " {"
+        "    \"id\": 3, "
         "    \"pools\": [ { \"pool\": \"2001:db8:3::/80\" } ],"
         "    \"subnet\": \"2001:db8:3::/64\" "
         " } ],"
@@ -470,6 +472,7 @@ TEST_F(JSONFileBackendTest, hashComments) {
         "\"renew-timer\": 1000, \n"
         "# comments in the middle should be ignored, too\n"
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " } ],"
@@ -522,6 +525,7 @@ TEST_F(JSONFileBackendTest, cppLineComments) {
         "\"renew-timer\": 1000, \n"
         "// comments in the middle should be ignored, too\n"
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " } ],"
@@ -574,6 +578,7 @@ TEST_F(JSONFileBackendTest, cBlockComments) {
         "\"renew-timer\": 1000, \n"
         "/* comments in the middle should be ignored, too*/\n"
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " } ],"
@@ -627,6 +632,7 @@ TEST_F(JSONFileBackendTest, include) {
         "}";
     string include = "\n"
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " } ]\n";
@@ -674,6 +680,7 @@ TEST_F(JSONFileBackendTest, recursiveInclude) {
         "\"rebind-timer\": 2000, "
         "\"renew-timer\": 1000, \n"
         "\"subnet6\": [ { "
+        "    \"id\": 1, "
         "    \"pools\": [ { \"pool\": \"2001:db8:1::/80\" } ],"
         "    \"subnet\": \"2001:db8:1::/64\" "
         " } ],"
