@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,12 +9,18 @@
 #include <dhcp6/parser_context.h>
 #include <dhcpsrv/parsers/simple_parser6.h>
 #include <testutils/io_utils.h>
+#include <testutils/log_utils.h>
+#include <testutils/test_to_element.h>
 #include <testutils/user_context_utils.h>
 #include <testutils/gtest_utils.h>
+
 #include <gtest/gtest.h>
+
 #include <fstream>
 #include <set>
 
+#include <boost/algorithm/string.hpp>
+
 using namespace isc::data;
 using namespace std;
 using namespace isc::test;
@@ -32,7 +38,11 @@ namespace test {
 void compareJSON(ConstElementPtr a, ConstElementPtr b) {
     ASSERT_TRUE(a);
     ASSERT_TRUE(b);
-    EXPECT_EQ(a->str(), b->str());
+    EXPECT_EQ(a->str(), b->str())
+#ifdef HAVE_CREATE_UNIFIED_DIFF
+        << "\nDiff:\n" << generateDiff(prettyPrint(a), prettyPrint(b)) << "\n"
+#endif
+    ;
 }
 
 /// @brief Tests if the input string can be parsed with specific parser
@@ -48,8 +58,9 @@ void compareJSON(ConstElementPtr a, ConstElementPtr b) {
 /// @param compare whether to compare the output with legacy JSON parser
 void testParser(const std::string& txt, Parser6Context::ParserType parser_type,
     bool compare = true) {
-    ConstElementPtr test_json;
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
 
+    ConstElementPtr test_json;
     ASSERT_NO_THROW({
             try {
                 Parser6Context ctx;
@@ -129,6 +140,7 @@ TEST(ParserTest, keywordDhcp6) {
                   "\"rebind-timer\": 2000, \n"
                   "\"renew-timer\": 1000, \n"
                   "\"subnet6\": [ { "
+                  "    \"id\": 1, "
                   "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                   "    \"subnet\": \"2001:db8:1::/48\", "
                   "    \"interface\": \"test\" } ],\n"
@@ -149,6 +161,7 @@ TEST(ParserTest, bashComments) {
                 "# and here\n"
                 "\"renew-timer\": 1000, \n"
                 "\"subnet6\": [ { "
+                "    \"id\": 1, "
                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                 "    \"subnet\": \"2001:db8:1::/48\", "
                 "    \"interface\": \"eth0\""
@@ -166,6 +179,7 @@ TEST(ParserTest, cppComments) {
                 "\"rebind-timer\": 2000, // everything after // is ignored\n"
                 "\"renew-timer\": 1000, // this will be ignored, too\n"
                 "\"subnet6\": [ { "
+                "    \"id\": 1, "
                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                 "    \"subnet\": \"2001:db8:1::/48\", "
                 "    \"interface\": \"eth0\""
@@ -183,6 +197,7 @@ TEST(ParserTest, bashCommentsInline) {
                 "\"rebind-timer\": 2000, # everything after # is ignored\n"
                 "\"renew-timer\": 1000, # this will be ignored, too\n"
                 "\"subnet6\": [ { "
+                "    \"id\": 1, "
                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                 "    \"subnet\": \"2001:db8:1::/48\", "
                 "    \"interface\": \"eth0\""
@@ -201,6 +216,7 @@ TEST(ParserTest, multilineComments) {
                 "\"rebind-timer\": 2000,\n"
                 "\"renew-timer\": 1000, \n"
                 "\"subnet6\": [ { "
+                "    \"id\": 1, "
                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                 "    \"subnet\": \"2001:db8:1::/48\", "
                 "    \"interface\": \"eth0\""
@@ -219,6 +235,7 @@ TEST(ParserTest, embbededComments) {
                 "\"rebind-timer\": 2000,\n"
                 "\"renew-timer\": 1000, \n"
                 "\"subnet6\": [ { "
+                "    \"id\": 1, "
                 "    \"user-context\": { \"comment\": \"indirect\" },"
                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                 "    \"subnet\": \"2001:db8:1::/48\", "
@@ -276,13 +293,16 @@ TEST(ParserTest, file) {
     configs.push_back("all-keys.json");
     configs.push_back("all-options.json");
     configs.push_back("backends.json");
-    configs.push_back("cassandra.json");
     configs.push_back("classify.json");
     configs.push_back("classify2.json");
     configs.push_back("comments.json");
+    configs.push_back("config-backend.json");
     configs.push_back("dhcpv4-over-dhcpv6.json");
+    configs.push_back("dnr.json");
     configs.push_back("duid.json");
     configs.push_back("global-reservations.json");
+    configs.push_back("ha-hot-standby-server1-with-tls.json");
+    configs.push_back("ha-hot-standby-server2.json");
     configs.push_back("hooks.json");
     configs.push_back("iPXE.json");
     configs.push_back("leases-expiration.json");
@@ -295,6 +315,7 @@ TEST(ParserTest, file) {
     configs.push_back("simple.json");
     configs.push_back("softwire46.json");
     configs.push_back("stateless.json");
+    configs.push_back("tee-times.json");
     configs.push_back("with-ddns.json");
 
     for (int i = 0; i<configs.size(); i++) {
@@ -320,8 +341,9 @@ TEST(ParserTest, globalParameters) {
 /// @param msg expected content of the exception
 void testError(const std::string& txt,
                Parser6Context::ParserType parser_type,
-               const std::string& msg)
-{
+               const std::string& msg) {
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
+
     try {
         Parser6Context ctx;
         ConstElementPtr parsed = ctx.parseString(txt, parser_type);
@@ -564,10 +586,6 @@ TEST(ParserTest, errors) {
               Parser6Context::PARSER_JSON,
               "<string>:1.3: syntax error, unexpected \",\", "
               "expecting }");
-    testError("{ \"foo\":true, }\n",
-              Parser6Context::PARSER_JSON,
-              "<string>:1.15: syntax error, unexpected }, "
-              "expecting constant string");
 
     // bad type
     testError("{ \"Dhcp6\":{\n"
@@ -635,7 +653,7 @@ TEST(ParserTest, errors) {
               "  \"Dhcp6\":{\n"
               "  \"comment\": \"second\" }}\n",
               Parser6Context::PARSER_DHCP6,
-              "<string>:2.23: syntax error, unexpected \",\", expecting }");
+              "<string>:3.3-9: syntax error, unexpected Dhcp6, expecting \",\" or }");
 
     // duplicate of not string entries
     testError("{ \"Dhcp6\":{\n"
@@ -752,7 +770,8 @@ TEST(ParserTest, mapEntries) {
     loadFile(sample_dir + "reservations.json", sample_json);
     loadFile(sample_dir + "all-keys-netconf.json", sample_json);
     KeywordSet sample_keys = {
-        "hosts-database"
+        "hosts-database",
+        "reservation-mode"
     };
     // Recursively extract keywords.
     static void (*extract)(ConstElementPtr, KeywordSet&) =
@@ -863,6 +882,99 @@ TEST(ParserTest, duplicateMapEntries) {
     cout << "checked " << cnt << " duplicated map entries\n";
 }
 
+/// @brief Test fixture for trailing commas.
+class TrailingCommasTest : public isc::dhcp::test::LogContentTest {
+public:
+    /// @brief Add a log entry.
+    ///
+    /// @param loc Location of the trailing comma.
+    void addLog(const string& loc) {
+        string log = "DHCP6_CONFIG_SYNTAX_WARNING configuration syntax ";
+        log += "warning: " + loc;
+        log += ": Extraneous comma. ";
+        log += "A piece of configuration may have been omitted.";
+        addString(log);
+    }
+};
+
+// Test that trailing commas are allowed.
+TEST_F(TrailingCommasTest, tests) {
+    string txt(R"({
+  "Dhcp6": {
+    "control-socket": {
+      "socket-name": "/tmp/kea-dhcp6-ctrl.sock",
+      "socket-type": "unix",
+    },
+    "hooks-libraries": [
+      {
+        "library": "/usr/local/lib/kea/hooks/libdhcp_dummy.so",
+      },
+    ],
+    "interfaces-config": {
+      "interfaces": [
+        "eth0",
+      ],
+    },
+    "lease-database": {
+      "name": "/tmp/kea-dhcp6.csv",
+      "persist": true,
+      "type": "memfile",
+    },
+    "loggers": [
+      {
+        "debuglevel": 99,
+        "name": "kea-dhcp6",
+        "output_options": [
+          {
+            "output": "stdout",
+          },
+        ],
+        "severity": "DEBUG",
+      },
+    ],
+    "multi-threading": {
+      "enable-multi-threading": false,
+      "packet-queue-size": 0,
+      "thread-pool-size": 0
+    },
+    "subnet6": [
+      {
+        "pools": [
+          {
+            "pool": "2001:db8:1::/64",
+          },
+        ],
+        "subnet": "2001:db8:1::/64",
+        "id": 1,
+      },
+    ],
+  },
+})");
+    testParser(txt, Parser6Context::PARSER_DHCP6, false);
+
+    addLog("<string>:5.28");
+    addLog("<string>:9.63");
+    addLog("<string>:10.8");
+    addLog("<string>:14.15");
+    addLog("<string>:15.8");
+    addLog("<string>:20.24");
+    addLog("<string>:28.31");
+    addLog("<string>:29.12");
+    addLog("<string>:31.28");
+    addLog("<string>:32.8");
+    addLog("<string>:43.38");
+    addLog("<string>:44.12");
+    addLog("<string>:47.16");
+    addLog("<string>:48.8");
+    addLog("<string>:49.6");
+    addLog("<string>:50.4");
+    EXPECT_TRUE(checkFile());
+
+    // Test with many consecutive commas.
+    boost::replace_all(txt, ",", ",,,,");
+    testParser(txt, Parser6Context::PARSER_DHCP6, false);
 }
-}
-}
+
+}  // namespace test
+}  // namespace dhcp
+}  // namespace isc
