@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -19,7 +19,7 @@
 #include <dhcpsrv/lease_file_loader.h>
 #include <log/logger_manager.h>
 #include <log/logger_name.h>
-#include <cfgrpt/config_report.h>
+#include <process/cfgrpt/config_report.h>
 
 #include <iostream>
 #include <sstream>
@@ -40,9 +40,6 @@ const uint32_t MAX_LEASE_ERRORS = 100;
 namespace isc {
 namespace lfc {
 
-// Refer to config_report so it will be embedded in the binary
-const char* const* lfc_config_report = isc::detail::config_report;
-
 /// @brief Defines the application name, it may be used to locate
 /// configuration data and appears in log statements.
 const char* LFCController::lfc_app_name_ = "DhcpLFC";
@@ -139,7 +136,7 @@ LFCController::launch(int argc, char* argv[], const bool test_mode) {
     try {
         pid_file.deleteFile();
     } catch (const PIDFileError& pid_ex) {
-          LOG_FATAL(lfc_logger, LFC_FAIL_PID_DEL).arg(pid_ex.what());
+        LOG_FATAL(lfc_logger, LFC_FAIL_PID_DEL).arg(pid_ex.what());
     }
 
     LOG_INFO(lfc_logger, LFC_TERMINATE);
@@ -325,8 +322,18 @@ LFCController::getVersion(const bool extended) const{
 
     version_stream << VERSION;
     if (extended) {
-        version_stream << std::endl << EXTENDED_VERSION << std::endl
-        << "database: " << isc::dhcp::Memfile_LeaseMgr::getDBVersion();
+        std::string db_version;
+        if (protocol_version_ == 4) {
+            db_version = Memfile_LeaseMgr::getDBVersion(Memfile_LeaseMgr::V4);
+        } else if (protocol_version_ == 6) {
+            db_version = Memfile_LeaseMgr::getDBVersion(Memfile_LeaseMgr::V6);
+        }
+        if (!db_version.empty()) {
+            db_version = "database: " + db_version;
+        }
+        version_stream << std::endl
+                       << EXTENDED_VERSION << std::endl
+                       << db_version;
     }
 
     return (version_stream.str());
@@ -404,8 +411,7 @@ LFCController::startLogger(const bool test_mode) const {
     // else use our defaults
     if (test_mode) {
         initLogger();
-    }
-    else {
+    } else {
         OutputOption option;
         LoggerManager manager;
 
