@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -41,6 +41,9 @@ public:
     /// @brief A vector holding MAC addresses.
     typedef std::vector<std::vector<uint8_t> > MacAddrsVector;
 
+    /// @brief Maximum allowed level of encapsulation of added relay options.
+    const static uint8_t RELAY_OPTIONS_MAX_ENCAPSULATION = 1;
+
     /// \brief A class encapsulating the type of lease being requested from the
     /// server.
     ///
@@ -119,7 +122,7 @@ public:
         DORA_SARR
     };
 
-    /// \brief Reset to defaults
+    /// \brief Reset to defaults.
     ///
     /// Reset data members to default values. This is specifically
     /// useful when unit tests are performed using different
@@ -173,6 +176,16 @@ public:
     /// \return delay between two consecutive performance reports.
     int getReportDelay() const { return report_delay_; }
 
+    /// \brief Returns clean report mode.
+    ///
+    /// \return true if cleaner report is enabled.
+    int getCleanReport() const { return clean_report_; }
+
+    /// \brief Returns clean report separator.
+    ///
+    /// \return returns string which is used as separator for report..
+    std::string getCleanReportSeparator() const { return clean_report_separator_; }
+
     /// \brief Returns number of simulated clients.
     ///
     /// \return number of simulated clients.
@@ -193,6 +206,11 @@ public:
     /// \return all base values specified.
     std::vector<std::string> getBase() const { return base_; }
 
+    /// \brief Returns address uniqueness value.
+    ///
+    /// \return address uniqueness specified value.
+    bool getAddrUnique() const { return addr_unique_; }
+
     /// \brief Returns maximum number of exchanges.
     ///
     /// \return number of exchange requests before test is aborted.
@@ -203,7 +221,19 @@ public:
     /// \return test period before it is aborted.
     int getPeriod() const { return period_; }
 
-    /// \brief Returns drop time
+    /// \brief Returns time to wait for elapsed time increase.
+    ///
+    /// \return how long perfdhcp will wait before start sending
+    /// messages with increased elapsed time.
+    int getWaitForElapsedTime() const { return wait_for_elapsed_time_; }
+
+    /// \brief Returns increased elapsed time.
+    ///
+    /// \return how long perfdhcp will send messages with increased
+    /// elapsed time.
+    int getIncreaseElapsedTime() const { return increased_elapsed_time_; }
+
+    /// \brief Returns drop time.
     ///
     /// The method returns maximum time elapsed from
     /// sending the packet before it is assumed dropped.
@@ -347,9 +377,19 @@ public:
 
     /// @brief Returns extra options to be inserted.
     ///
-    /// @return container with options
+    /// @return container with options.
     const isc::dhcp::OptionCollection& getExtraOpts() const { return extra_opts_; }
 
+    /// @brief Returns relay options to be inserted at given level of encapsulation.
+    ///
+    /// @param encapsulation_level level of encapsulation, by default 1
+    ///
+    /// @throws isc::OutOfRange When trying to access relay options at encapsulation
+    /// level that doesn't exist.
+    ///
+    /// @return container with options.
+    const isc::dhcp::OptionCollection& getRelayOpts(uint8_t encapsulation_level = 1) const;
+
     /// \brief Check if single-threaded mode is enabled.
     ///
     /// \return true if single-threaded mode is enabled.
@@ -365,6 +405,25 @@ public:
     /// \return server name.
     std::string getServerName() const { return server_name_; }
 
+    /// \brief Returns file location with set of relay addresses.
+    ///
+    /// \return relay addresses list file location.
+    std::string getRelayAddrListFile() const { return relay_addr_list_file_; }
+
+    /// \brief Returns list of relay addresses.
+    ///
+    /// \return list of relay addresses.
+    std::vector<std::string> getRelayAddrList() const { return relay_addr_list_; }
+
+    /// \brief Returns random relay address.
+    ///
+    /// \return single string containing relay address.
+    std::string getRandRelayAddr() { return relay_addr_list_[rand() % relay_addr_list_.size()]; }
+
+    /// \brief Check if multi subnet mode is enabled.
+    ///
+    /// \return true if multi subnet mode is enabled.
+    bool checkMultiSubnet() { return multi_subnet_; }
 
     /// \brief Find if diagnostic flag has been set.
     ///
@@ -383,7 +442,7 @@ public:
     /// \brief Print usage.
     ///
     /// Prints perfdhcp usage.
-    void usage() const;
+    static void usage();
 
     /// \brief Print program version.
     ///
@@ -519,17 +578,26 @@ private:
     /// mac_list_ vector.
     bool decodeMacString(const std::string& line);
 
+    /// \brief Opens the text file containing list of addresses (one per line).
+    void loadRelayAddr();
+
+    /// \brief Checks if loaded relay addresses from text file are correct,
+    /// adds them to relay_addr_list_.
+    ///
+    /// \return true if address is incorrect.
+    bool validateIP(const std::string& line);
+
     /// IP protocol version to be used, expected values are:
-    /// 4 for IPv4 and 6 for IPv6, default value 0 means "not set"
+    /// 4 for IPv4 and 6 for IPv6, default value 0 means "not set".
     uint8_t ipversion_;
 
-    /// Packet exchange mode (e.g. DORA/SARR)
+    /// Packet exchange mode (e.g. DORA/SARR).
     ExchangeMode exchange_mode_;
 
     /// Lease Type to be obtained: address only, IPv6 prefix only.
     LeaseType lease_type_;
 
-    /// Rate in exchange per second
+    /// Rate in exchange per second.
     unsigned int rate_;
 
     /// A rate at which DHCPv6 Renew messages are sent.
@@ -538,9 +606,15 @@ private:
     /// A rate at which DHCPv6 Release messages are sent.
     unsigned int release_rate_;
 
-    /// Delay between generation of two consecutive performance reports
+    /// Delay between generation of two consecutive performance reports.
     int report_delay_;
 
+    /// Enable cleaner, easy to parse, output of performance reports.
+    bool clean_report_;
+
+    /// If clean report is enabled separator for output can be configured.
+    std::string clean_report_separator_;
+
     /// Number of simulated clients (aka randomization range).
     uint32_t clients_num_;
 
@@ -549,29 +623,40 @@ private:
     std::vector<uint8_t> mac_template_;
 
     /// DUID template used to generate unique DUIDs for
-    /// simulated clients
+    /// simulated clients.
     std::vector<uint8_t> duid_template_;
 
+    /// Check address uniqueness.
+    bool addr_unique_;
+
     /// Collection of base values specified with -b<value>
-    /// options. Supported "bases" are mac=<mac> and duid=<duid>
+    /// options. Supported "bases" are mac=<mac> and duid=<duid>.
     std::vector<std::string> base_;
 
-    /// Number of microseconds by which you should delay the exit
+    /// Number of microseconds by which you should delay the exit.
     int exit_wait_time_;
 
     /// Number of 2 or 4-way exchanges to perform.
     std::vector<int> num_request_;
 
-    /// Test period in seconds
+    /// Test period in seconds.
     int period_;
 
+    // for how long perfdhcp will wait before start sending
+    // messages with increased elapsed time.
+    int wait_for_elapsed_time_;
+
+    // Amount of time after which perfdhcp will send messages with
+    // elapsed time increased.
+    int increased_elapsed_time_;
+
     /// Indicates number of -d<value> parameters specified by user.
     /// If this value goes above 2, command line parsing fails.
     uint8_t drop_time_set_;
 
     /// Time to elapse before request is lost. The first value of
     /// two-element vector refers to DO/SA exchanges,
-    /// second value refers to RA/RR. Default values are { 1, 1 }
+    /// second value refers to RA/RR. Default values are { 1, 1 }.
     std::vector<double> drop_time_;
 
     /// Maximum number of drops request before aborting test.
@@ -589,7 +674,7 @@ private:
     std::string localname_;
 
     /// Indicates that specified value with -l<value> is
-    /// rather interface (not address)
+    /// rather interface (not address).
     bool is_interface_;
 
     /// Number of preload packets. Preload packets are used to
@@ -597,10 +682,10 @@ private:
     /// measurements.
     int preload_;
 
-    /// Local port number (host endian)
+    /// Local port number (host endian).
     int local_port_;
 
-    /// Remote port number (host endian)
+    /// Remote port number (host endian).
     int remote_port_;
 
     /// Randomization seed.
@@ -633,10 +718,19 @@ private:
     /// List of MAC addresses loaded from a file.
     std::vector<std::vector<uint8_t> > mac_list_;
 
+    /// Location of a file containing a list of subnet addresses, one per line.
+    std::string relay_addr_list_file_;
+
+    /// List of validated subnet addresses.
+    std::vector<std::string> relay_addr_list_;
+
+    /// Flag to indicate multiple subnets testing.
+    bool multi_subnet_;
+
     /// Offset of transaction id in template files. First vector
     /// element points to offset for DISCOVER/SOLICIT messages,
     /// second element points to transaction id offset for
-    /// REQUEST messages
+    /// REQUEST messages.
     std::vector<int> xid_offset_;
 
     /// Random value offset in templates. Random value offset
@@ -650,7 +744,7 @@ private:
     /// Offset of server id option in template packet.
     int sid_offset_;
 
-    /// Offset of requested ip data in template packet
+    /// Offset of requested ip data in template packet.
     int rip_offset_;
 
     /// String representing diagnostic selectors specified
@@ -670,6 +764,9 @@ private:
     /// @brief Extra options to be sent in each packet.
     isc::dhcp::OptionCollection extra_opts_;
 
+    /// @brief Map of relay options to be sent per encapsulation level.
+    std::map<uint8_t, isc::dhcp::OptionCollection> relay_opts_;
+
     /// @brief Option to switch modes between single-threaded and multi-threaded.
     bool single_thread_mode_;
 
