@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,16 +13,16 @@
 #include <perfdhcp/receiver.h>
 #include <perfdhcp/command_options.h>
 #include <perfdhcp/perf_socket.h>
+#include <perfdhcp/random_number_generator.h>
 
 #include <dhcp/iface_mgr.h>
+#include <dhcp/dhcp4.h>
 #include <dhcp/dhcp6.h>
 #include <dhcp/pkt4.h>
 #include <dhcp/pkt6.h>
-#include <util/random/random_number_generator.h>
 
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
-#include <boost/function.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 
 #include <string>
@@ -128,7 +128,7 @@ public:
     /// @brief Delay the exit by a fixed given time to catch up to all exchanges
     ///     that were already started.
     /// @return true if need to wait, false = ok to exit now
-    bool waitToExit() const;
+    bool waitToExit();
 
     /// @brief Checks if all expected packets were already received
     bool haveAllPacketsBeenReceived() const;
@@ -221,11 +221,16 @@ public:
     /// has been reached.
     void cleanCachedPackets();
 
+    /// \brief Get interrupted flag.
     bool interrupted() const { return interrupted_; }
 
+    /// \brief Get stats manager.
     StatsMgr& getStatsMgr() { return stats_mgr_; };
 
+    /// \brief Start receiver.
     void start() { receiver_.start(); }
+
+    /// \brief Stop receiver.
     void stop() { receiver_.stop(); }
 
     /// \brief Run wrapped command.
@@ -233,7 +238,10 @@ public:
     /// \param do_stop execute wrapped command with "stop" argument.
     void runWrapped(bool do_stop = false) const;
 
+    /// \brief Get received server id flag.
     bool serverIdReceived() const { return first_packet_serverid_.size() > 0; }
+
+    /// \brief Get received server id.
     std::string getServerId() const { return vector2Hex(first_packet_serverid_); }
 
     /// \brief Send number of packets to initiate new exchanges.
@@ -261,10 +269,13 @@ public:
 
     /// \brief Send number of DHCPREQUEST (renew) messages to a server.
     ///
+    /// \param msg_type A type of the messages to be sent (DHCPREQUEST or
+    /// DHCPRELEASE).
     /// \param msg_num A number of messages to be sent.
     ///
     /// \return A number of messages actually sent.
-    uint64_t sendMultipleRequests(const uint64_t msg_num);
+    uint64_t sendMultipleMessages4(const uint32_t msg_type,
+                                   const uint64_t msg_num);
 
     /// \brief Send number of DHCPv6 Renew or Release messages to the server.
     ///
@@ -300,21 +311,54 @@ public:
     /// in packet templates and their contents.
     void printTemplates() const;
 
+    /// \brief Get set of unique replied addresses.
+    std::set<std::string>& getAllUniqueAddrReply() {
+        return unique_reply_address_;
+    }
+
+    /// \brief Get set of unique advertised addresses.
+    std::set<std::string>& getAllUniqueAddrAdvert() {
+        return unique_address_;
+    }
+
+    /// \brief Convert binary value to hex string.
+    ///
+    /// \todo Consider moving this function to src/lib/util.
+    ///
+    /// \param b byte to convert.
+    /// \return hex string.
+    static std::string byte2Hex(const uint8_t b);
+
+    /// \brief Convert vector in hexadecimal string.
+    ///
+    /// \todo Consider moving this function to src/lib/util.
+    ///
+    /// \param vec vector to be converted.
+    /// \param separator separator.
+    static std::string vector2Hex(const std::vector<uint8_t>& vec,
+                                  const std::string& separator = "");
+
+    /// \brief Initialized at first exit condition with the time perfdhcp
+    ///  should exit
+    boost::posix_time::ptime exit_time_;
+
     // We would really like following methods and members to be private but
     // they have to be accessible for unit-testing. Another, possibly better,
     // solution is to make this class friend of test class but this is not
     // what's followed in other classes.
 protected:
     /// Generate uniformly distributed integers in range of [min, max]
-    isc::util::random::UniformRandomIntegerGenerator number_generator_;
+    UniformRandomIntegerGenerator number_generator_;
 
     /// \brief Creates DHCPREQUEST from a DHCPACK message.
     ///
+    /// @param msg_type the message type to be created (DHCPREQUEST or DHCPRELEASE)
     /// \param ack An instance of the DHCPACK message to be used to
     /// create a new message.
     ///
     /// \return Pointer to the created message.
-    dhcp::Pkt4Ptr createRequestFromAck(const dhcp::Pkt4Ptr& ack);
+    dhcp::Pkt4Ptr createMessageFromAck(const uint16_t msg_type,
+                                       const dhcp::Pkt4Ptr& ack);
 
     /// \brief Creates DHCPv6 message from the Reply packet.
     ///
@@ -536,6 +580,116 @@ protected:
     /// \throw isc::Unexpected if unexpected error occurred.
     void processReceivedPacket4(const dhcp::Pkt4Ptr& pkt4);
 
+    /// \brief Process IA in received DHCPv6 packet.
+    ///
+    /// Process IA in received message to check if it contain proper
+    /// address and/or prefix
+    ///
+    /// \param [in] pkt6 object representing DHCPv6 packet received.
+    /// \return true if the message include correct IA, false otherwise.
+    bool validateIA(const dhcp::Pkt6Ptr& pkt6);
+
+    /// \brief Process received v6 addresses uniqueness.
+    ///
+    /// Generate list of addresses and check for uniqueness.
+    ///
+    /// \param pkt6 object representing received DHCPv6 packet
+    /// \param xchg_type ExchangeType enum value.
+    void address6Uniqueness(const dhcp::Pkt6Ptr& pkt6, ExchangeType xchg_type);
+
+    /// \brief Process received v4 addresses uniqueness.
+    ///
+    /// Generate list of addresses and check for uniqueness.
+    ///
+    /// \param pkt4 object representing received DHCPv4 packet
+    /// \param xchg_type ExchangeType enum value.
+    void address4Uniqueness(const dhcp::Pkt4Ptr& pkt4, ExchangeType xchg_type);
+
+    /// \brief add unique address to already assigned list.
+    ///
+    /// Add address and/or prefix to unique set if it's not already there,
+    /// otherwise increment the number of non unique addresses.
+    ///
+    /// \param current set of addresses that should be added to unique list
+    /// \param xchg_type ExchangeType enum value.
+    void addUniqeAddr(const std::set<std::string>& current, ExchangeType xchg_type) {
+        switch(xchg_type) {
+            case ExchangeType::SA: {
+                for (auto current_it = current.begin();
+                     current_it != current.end(); ++current_it) {
+                    // addresses should be unique cross packets
+                    auto ret = unique_address_.emplace(*current_it);
+                    if (!ret.second) {
+                        stats_mgr_.updateNonUniqueAddrNum(ExchangeType::SA);
+                    }
+                }
+                break;
+            }
+            case ExchangeType::RR: {
+                for (auto current_it = current.begin();
+                     current_it != current.end(); ++current_it) {
+                    // addresses should be unique cross packets
+                    auto ret = unique_reply_address_.emplace(*current_it);
+                    if (!ret.second) {
+                        stats_mgr_.updateNonUniqueAddrNum(ExchangeType::RR);
+                    }
+                }
+                break;
+            }
+            case ExchangeType::RLA:
+            case ExchangeType::RL: {
+                removeUniqueAddr(current);
+                break;
+            }
+            case ExchangeType::DO: {
+                for (auto current_it = current.begin();
+                     current_it != current.end(); ++current_it) {
+                    // addresses should be unique cross packets
+                    auto ret = unique_address_.emplace(*current_it);
+                    if (!ret.second) {
+                        stats_mgr_.updateNonUniqueAddrNum(ExchangeType::DO);
+                    }
+                }
+                break;
+            }
+            case ExchangeType::RA: {
+                for (auto current_it = current.begin();
+                     current_it != current.end(); ++current_it) {
+                    // addresses should be unique cross packets
+                    auto ret = unique_reply_address_.emplace(*current_it);
+                    if (!ret.second) {
+                        stats_mgr_.updateNonUniqueAddrNum(ExchangeType::RA);
+                    }
+                }
+                break;
+            }
+            case ExchangeType::RNA:
+            case ExchangeType::RN:
+            default:
+                break;
+        }
+    }
+
+    /// \brief remove unique address from list.
+    ///
+    /// If address is released we should remove it from both
+    /// advertised (offered) and assigned sets.
+    ///
+    /// \param addr holding value of unique address.
+    void removeUniqueAddr(const std::set<std::string>& addr) {
+        for (auto addr_it = addr.begin(); addr_it != addr.end(); ++addr_it) {
+            auto it = unique_address_.find(*addr_it);
+            if (it != unique_address_.end()) {
+                unique_address_.erase(it);
+            }
+
+            auto it2 = unique_reply_address_.find(*addr_it);
+            if (it2 != unique_reply_address_.end()) {
+                unique_reply_address_.erase(it2);
+            }
+        }
+    }
+
     /// \brief Process received DHCPv6 packet.
     ///
     /// Method performs processing of the received DHCPv6 packet,
@@ -548,7 +702,7 @@ protected:
     /// \throw isc::Unexpected if unexpected error occurred.
     void processReceivedPacket6(const dhcp::Pkt6Ptr& pkt6);
 
-    /// \brief Register option factory functions for DHCPv4
+    /// \brief Register option factory functions for DHCPv4.
     ///
     /// Method registers option factory functions for DHCPv4.
     /// These functions are called to create instances of DHCPv4
@@ -557,7 +711,7 @@ protected:
     /// Use \ref registerOptionFactories instead.
     void registerOptionFactories4() const;
 
-    /// \brief Register option factory functions for DHCPv6
+    /// \brief Register option factory functions for DHCPv6.
     ///
     /// Method registers option factory functions for DHCPv6.
     /// These functions are called to create instances of DHCPv6
@@ -572,7 +726,6 @@ protected:
     /// depending in which mode test is currently running.
     void registerOptionFactories() const;
 
-
     /// \brief Resets internal state of the object.
     ///
     /// Method resets internal state of the object. It has to be
@@ -648,8 +801,11 @@ protected:
 
     /// \brief Send DHCPv4 renew (DHCPREQUEST).
     ///
+    /// \param msg_type A type of the message to be sent (DHCPREQUEST or
+    /// DHCPRELEASE).
+    ///
     /// \return true if the message has been sent, false otherwise.
-    bool sendRequestFromAck();
+    bool sendMessageFromAck(const uint16_t msg_type);
 
     /// \brief Send DHCPv6 Renew or Release message.
     ///
@@ -789,7 +945,7 @@ protected:
     /// and that would create dependency between test_control.h and
     /// command_options.h.
     ///
-    /// @param pkt4 options will be added here
+    /// @param pkt4 options will be added here.
     void addExtraOpts(const dhcp::Pkt4Ptr& pkt4);
 
     /// @brief Inserts extra options specified by user.
@@ -799,7 +955,7 @@ protected:
     /// and that would create dependency between test_control.h and
     /// command_options.h.
     ///
-    /// @param pkt6 options will be added here
+    /// @param pkt6 options will be added here.
     void addExtraOpts(const dhcp::Pkt6Ptr& pkt6);
 
     /// \brief Copies IA_NA or IA_PD option from one packet to another.
@@ -820,14 +976,6 @@ protected:
     /// is NULL.
     void copyIaOptions(const dhcp::Pkt6Ptr& pkt_from, dhcp::Pkt6Ptr& pkt_to);
 
-    /// \brief Convert binary value to hex string.
-    ///
-    /// \todo Consider moving this function to src/lib/util.
-    ///
-    /// \param b byte to convert.
-    /// \return hex string.
-    std::string byte2Hex(const uint8_t b) const;
-
     /// \brief Calculate elapsed time between two packets.
     ///
     /// This function calculates the time elapsed between two packets. If
@@ -871,21 +1019,12 @@ protected:
     /// \return transaction id offset in packet.
     int getTransactionIdOffset(const int arg_idx) const;
 
-    /// \brief Convert vector in hexadecimal string.
-    ///
-    /// \todo Consider moving this function to src/lib/util.
-    ///
-    /// \param vec vector to be converted.
-    /// \param separator separator.
-    std::string vector2Hex(const std::vector<uint8_t>& vec,
-                           const std::string& separator = "") const;
-
     /// \brief Handle child signal.
     ///
     /// Function handles child signal by waiting for
     /// the process to complete.
     ///
-    /// \param sig signal (ignored)
+    /// \param sig signal (ignored).
     static void handleChild(int sig);
 
     /// \brief Handle interrupt signal.
@@ -893,7 +1032,7 @@ protected:
     /// Function sets flag indicating that program has been
     /// interrupted.
     ///
-    /// \param sig signal (ignored)
+    /// \param sig signal (ignored).
     static void handleInterrupt(int sig);
 
     /// \brief Print main diagnostics data.
@@ -901,7 +1040,7 @@ protected:
     /// Method prints main diagnostics data.
     void printDiagnostics() const;
 
-    /// \brief Print template information
+    /// \brief Print template information.
     ///
     /// \param packet_type packet type.
     void printTemplate(const uint8_t packet_type) const;
@@ -919,32 +1058,55 @@ protected:
     /// spaces or hexadecimal digits.
     void readPacketTemplate(const std::string& file_name);
 
+    /// \brief Keep addresses and prefixes from advertise msg for uniqueness checks.
+    std::set<std::string> unique_address_;
+
+    /// \brief Keep addresses and prefixes from reply msg for uniqueness checks.
+    std::set<std::string> unique_reply_address_;
+
+    /// \brief Socket used for DHCP traffic.
     BasePerfSocket &socket_;
+
+    /// \brief Receiver used to receive DHCP traffic.
     Receiver receiver_;
 
-    boost::posix_time::ptime last_report_; ///< Last intermediate report time.
+    /// \brief Last intermediate report time.
+    boost::posix_time::ptime last_report_;
 
-    StatsMgr stats_mgr_;  ///< Statistics Manager.
+    /// \brief Statistics Manager.
+    StatsMgr stats_mgr_;
 
-    PacketStorage<dhcp::Pkt4> ack_storage_; ///< A storage for DHCPACK messages.
-    PacketStorage<dhcp::Pkt6> reply_storage_; ///< A storage for reply messages.
+    /// \brief Storage for DHCPACK messages.
+    PacketStorage<dhcp::Pkt4> ack_storage_;
 
-    NumberGeneratorPtr transid_gen_; ///< Transaction id generator.
-    NumberGeneratorPtr macaddr_gen_; ///< Numbers generator for MAC address.
+    /// \brief Storage for reply messages.
+    PacketStorage<dhcp::Pkt6> reply_storage_;
 
-    /// Buffer holding server id received in first packet
+    /// \brief Transaction id generator.
+    NumberGeneratorPtr transid_gen_;
+
+    /// \brief Numbers generator for MAC address.
+    NumberGeneratorPtr macaddr_gen_;
+
+    /// \brief Buffer holding server id received in first packet
     dhcp::OptionBuffer first_packet_serverid_;
 
-    /// Packet template buffers.
+    /// \brief Packet template buffers.
     TemplateBufferCollection template_buffers_;
 
     /// First packets send. They are used at the end of the test
     /// to print packet templates when diagnostics flag T is specified.
+
+    /// \brief Template for v4.
     std::map<uint8_t, dhcp::Pkt4Ptr> template_packets_v4_;
+
+    /// \brief Template for v6.
     std::map<uint8_t, dhcp::Pkt6Ptr> template_packets_v6_;
 
-    static bool interrupted_;  ///< Is program interrupted.
+    /// \brief Program interrupted flag.
+    static bool interrupted_;
 
+    /// \brief Command options.
     CommandOptions& options_;
 };
 
