@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,25 +26,26 @@ const SimpleDefaults HA_CONFIG_LB_DEFAULTS = {
 
 /// @brief Default values for HA configuration.
 const SimpleDefaults HA_CONFIG_DEFAULTS = {
-    { "delayed-updates-limit",   Element::integer, "0" },
-    { "heartbeat-delay",         Element::integer, "10000" },
-    { "max-ack-delay",           Element::integer, "10000" },
-    { "max-response-delay",      Element::integer, "60000" },
-    { "max-unacked-clients",     Element::integer, "10" },
-    { "require-client-certs",    Element::boolean, "true" },
-    { "restrict-commands",       Element::boolean, "false" },
-    { "send-lease-updates",      Element::boolean, "true" },
-    { "sync-leases",             Element::boolean, "true" },
-    { "sync-timeout",            Element::integer, "60000" },
-    { "sync-page-limit",         Element::integer, "10000" },
-    { "wait-backup-ack",         Element::boolean, "false" }
+    { "delayed-updates-limit",      Element::integer, "0" },
+    { "heartbeat-delay",            Element::integer, "10000" },
+    { "max-ack-delay",              Element::integer, "10000" },
+    { "max-response-delay",         Element::integer, "60000" },
+    { "max-unacked-clients",        Element::integer, "10" },
+    { "max-rejected-lease-updates", Element::integer, "10" },
+    { "require-client-certs",       Element::boolean, "true" },
+    { "restrict-commands",          Element::boolean, "false" },
+    { "send-lease-updates",         Element::boolean, "true" },
+    { "sync-leases",                Element::boolean, "true" },
+    { "sync-timeout",               Element::integer, "60000" },
+    { "sync-page-limit",            Element::integer, "10000" },
+    { "wait-backup-ack",            Element::boolean, "false" }
 };
 
 /// @brief Default values for HA multi-threading configuration.
 const SimpleDefaults HA_CONFIG_MT_DEFAULTS = {
-    { "enable-multi-threading",    Element::boolean, "false" },
+    { "enable-multi-threading",    Element::boolean, "true" },
     { "http-client-threads",       Element::integer, "0" },
-    { "http-dedicated-listener",   Element::boolean, "false" },
+    { "http-dedicated-listener",   Element::boolean, "true" },
     { "http-listener-threads",     Element::integer, "0" }
 };
 
@@ -183,6 +184,10 @@ HAConfigParser::parseInternal(const HAConfigPtr& config_storage,
     uint32_t max_unacked_clients = getAndValidateInteger<uint32_t>(c, "max-unacked-clients");
     config_storage->setMaxUnackedClients(max_unacked_clients);
 
+    // Get 'max-rejected-lease-updates'.
+    uint32_t max_rejected_lease_updates = getAndValidateInteger<uint32_t>(c, "max-rejected-lease-updates");
+    config_storage->setMaxRejectedLeaseUpdates(max_rejected_lease_updates);
+
     // Get 'wait-backup-ack'.
     config_storage->setWaitBackupAck(getBoolean(c, "wait-backup-ack"));
 
@@ -406,5 +411,5 @@ HAConfigParser::logConfigStatus(const HAConfigPtr& config_storage) const {
     }
 }
 
-} // end of namespace ha
-} // end of namespace isc
+}  // namespace ha
+}  // namespace isc
