@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -32,10 +32,8 @@ namespace {
 /// configuration.
 class HAConfigTest : public HATest {
 public:
-
     /// @brief Constructor.
-    HAConfigTest()
-        : HATest() {
+    HAConfigTest() : HATest(), hardware_threads_(MultiThreadingMgr::detectThreadCount()) {
     }
 
     /// @brief Verifies if an exception is thrown if provided HA
@@ -59,6 +57,9 @@ public:
                 " exception type";
         }
     }
+
+    /// @brief number of threads the system reports as supported
+    uint32_t hardware_threads_;
 };
 
 // Verifies that load balancing configuration is parsed correctly.
@@ -77,6 +78,7 @@ TEST_F(HAConfigTest, configureLoadBalancing) {
         "        \"max-response-delay\": 11,"
         "        \"max-ack-delay\": 5,"
         "        \"max-unacked-clients\": 20,"
+        "        \"max-rejected-lease-updates\": 9,"
         "        \"wait-backup-ack\": false,"
         "        \"peers\": ["
         "            {"
@@ -134,6 +136,7 @@ TEST_F(HAConfigTest, configureLoadBalancing) {
     EXPECT_EQ(11, impl->getConfig()->getMaxResponseDelay());
     EXPECT_EQ(5, impl->getConfig()->getMaxAckDelay());
     EXPECT_EQ(20, impl->getConfig()->getMaxUnackedClients());
+    EXPECT_EQ(9, impl->getConfig()->getMaxRejectedLeaseUpdates());
     EXPECT_FALSE(impl->getConfig()->amWaitingBackupAck());
 
     HAConfig::PeerConfigPtr cfg = impl->getConfig()->getThisServerConfig();
@@ -207,11 +210,12 @@ TEST_F(HAConfigTest, configureLoadBalancing) {
     ASSERT_TRUE(state_cfg);
     EXPECT_EQ(STATE_PAUSE_ONCE, state_cfg->getPausing());
 
-    // Verify multi-threading default values.
-    EXPECT_FALSE(impl->getConfig()->getEnableMultiThreading());
-    EXPECT_FALSE(impl->getConfig()->getHttpDedicatedListener());
-    EXPECT_EQ(0, impl->getConfig()->getHttpListenerThreads());
-    EXPECT_EQ(0, impl->getConfig()->getHttpClientThreads());
+    // Verify multi-threading default values. Default is 0 for the listener and client threads, but
+    // after MT is applied, HAImpl resolves them to the auto-detected values.
+    EXPECT_TRUE(impl->getConfig()->getEnableMultiThreading());
+    EXPECT_TRUE(impl->getConfig()->getHttpDedicatedListener());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpListenerThreads());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpClientThreads());
 }
 
 // Verifies that hot standby configuration is parsed correctly.
@@ -258,6 +262,7 @@ TEST_F(HAConfigTest, configureHotStandby) {
     EXPECT_EQ(10000, impl->getConfig()->getHeartbeatDelay());
     EXPECT_EQ(10000, impl->getConfig()->getMaxAckDelay());
     EXPECT_EQ(10, impl->getConfig()->getMaxUnackedClients());
+    EXPECT_EQ(10, impl->getConfig()->getMaxRejectedLeaseUpdates());
     EXPECT_FALSE(impl->getConfig()->amWaitingBackupAck());
 
     HAConfig::PeerConfigPtr cfg = impl->getConfig()->getThisServerConfig();
@@ -321,11 +326,12 @@ TEST_F(HAConfigTest, configureHotStandby) {
     ASSERT_TRUE(state_cfg);
     EXPECT_EQ(STATE_PAUSE_NEVER, state_cfg->getPausing());
 
-    // Verify multi-threading default values.
-    EXPECT_FALSE(impl->getConfig()->getEnableMultiThreading());
-    EXPECT_FALSE(impl->getConfig()->getHttpDedicatedListener());
-    EXPECT_EQ(0, impl->getConfig()->getHttpListenerThreads());
-    EXPECT_EQ(0, impl->getConfig()->getHttpClientThreads());
+    // Verify multi-threading default values. Default is 0 for the listener and client threads, but
+    // after MT is applied, HAImpl resolves them to the auto-detected values.
+    EXPECT_TRUE(impl->getConfig()->getEnableMultiThreading());
+    EXPECT_TRUE(impl->getConfig()->getHttpDedicatedListener());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpListenerThreads());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpClientThreads());
 }
 
 // Verifies that passive-backup configuration is parsed correctly.
@@ -388,11 +394,12 @@ TEST_F(HAConfigTest, configurePassiveBackup) {
     ASSERT_TRUE(cfg->getBasicAuth());
     EXPECT_EQ("a2VhdGVzdDpLZWFUZXN0", cfg->getBasicAuth()->getCredential());
 
-    // Verify multi-threading default values.
-    EXPECT_FALSE(impl->getConfig()->getEnableMultiThreading());
-    EXPECT_FALSE(impl->getConfig()->getHttpDedicatedListener());
-    EXPECT_EQ(0, impl->getConfig()->getHttpListenerThreads());
-    EXPECT_EQ(0, impl->getConfig()->getHttpClientThreads());
+    // Verify multi-threading default values. Default is 0 for the listener and client threads, but
+    // after MT is applied, HAImpl resolves them to the auto-detected values.
+    EXPECT_TRUE(impl->getConfig()->getEnableMultiThreading());
+    EXPECT_TRUE(impl->getConfig()->getHttpDedicatedListener());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpListenerThreads());
+    EXPECT_EQ(hardware_threads_, impl->getConfig()->getHttpClientThreads());
 }
 
 // This server name must not be empty.
@@ -1727,15 +1734,12 @@ TEST_F(HAConfigTest, multiThreadingPermutations) {
     bool ha_mt = true;
     bool listener = true;
 
-    // Number of threads the system reports as supported.
-    uint32_t sys_threads = MultiThreadingMgr::detectThreadCount();
-
     std::vector<Scenario> scenarios {
         {
-            "1 no ha+mt/default",
+            "1 ha+mt by default",
             "",
             dhcp_mt, 4,
-            !ha_mt, !listener, 0, 0
+            ha_mt, listener, 4, 4
         },
         {
             "2 dhcp mt enabled, ha mt disabled",
@@ -1781,7 +1785,7 @@ TEST_F(HAConfigTest, multiThreadingPermutations) {
             // reported value.
             makeHAMtJson(ha_mt, listener, 0, 0),
             dhcp_mt, 0,
-            (sys_threads > 0), listener, sys_threads, sys_threads
+            (hardware_threads_ > 0), listener, hardware_threads_, hardware_threads_
         }
     };
 
@@ -1872,4 +1876,4 @@ TEST_F(HAConfigTest, ipv6Url) {
     EXPECT_EQ(impl->getConfig()->getThisServerConfig()->getUrl().toText(), "http://[2001:db8::1]:8080/");
 }
 
-} // end of anonymous namespace
+}  // namespace
