@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -17,6 +17,7 @@
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/network_state.h>
 #include <hooks/hooks_manager.h>
+#include <testutils/gtest_utils.h>
 #include <boost/pointer_cast.hpp>
 #include <gtest/gtest.h>
 #include <string>
@@ -31,6 +32,29 @@ using namespace isc::hooks;
 
 namespace {
 
+/// @brief Structure that holds registered hook indexes.
+///
+/// This allows us to park packets.
+struct TestHooks {
+    /// @brief Index of leases4_committed callout.
+    int hook_index_leases4_committed_;
+
+    /// @brief Index of leases6_committed callout.
+    int hook_index_leases6_committed_;
+
+    /// @brief Constructor
+    ///
+    /// The constructor registers hook points for callout tests.
+    TestHooks() {
+        hook_index_leases4_committed_ =
+            HooksManager::registerHook("leases4_committed");
+        hook_index_leases6_committed_ =
+            HooksManager::registerHook("leases6_committed");
+    }
+};
+
+TestHooks test_hooks;
+
 /// @brief Derivation of the @c HAImpl which provides access to protected
 /// methods and members.
 class TestHAImpl : public HAImpl {
@@ -183,8 +207,8 @@ TEST_F(HAImplTest, buffer4Receive) {
     };
 
     // Provide DHCP message type option, truncated vendor option and domain name.
-    // Parsing this message should be successful but domain name following the
-    // truncated vendor option should be skipped.
+    // Parsing this message should be successful but vendor option should be
+    // skipped.
     std::vector<uint8_t> options = {
         53, 1, 1, // Message type = DHCPDISCOVER
         125, 6, // vendor options
@@ -211,13 +235,15 @@ TEST_F(HAImplTest, buffer4Receive) {
     // The client class should be assigned to the message to indicate that the
     // server1 should process this message.
     ASSERT_EQ(1, query4->getClasses().size());
-    EXPECT_EQ("HA_server1", *(query4->getClasses().cbegin()));
+    EXPECT_TRUE(query4->inClass("HA_server1"));
 
     // Check that the message has been parsed. The DHCP message type should
     // be set in this case.
     EXPECT_EQ(DHCPDISCOVER, static_cast<int>(query4->getType()));
-    // Domain name should be skipped because the vendor option was truncated.
-    EXPECT_FALSE(query4->getOption(DHO_DOMAIN_NAME));
+    // Vendor option should be skipped because it was truncated.
+    EXPECT_FALSE(query4->getOption(DHO_VIVSO_SUBOPTIONS));
+    // Domain name should not be skipped because the vendor option was truncated.
+    EXPECT_TRUE(query4->getOption(DHO_DOMAIN_NAME));
 }
 
 // Tests for buffer6_receive callout implementation.
@@ -299,7 +325,7 @@ TEST_F(HAImplTest, buffer6Receive) {
     // The client class should be assigned to the message to indicate that the
     // server1 should process this message.
     ASSERT_EQ(1, query6->getClasses().size());
-    EXPECT_EQ("HA_server1", *(query6->getClasses().cbegin()));
+    EXPECT_TRUE(query6->inClass("HA_server1"));
 
     // Check that the message has been parsed. The DHCP message type should
     // be set in this case.
@@ -319,11 +345,19 @@ TEST_F(HAImplTest, leases4Committed) {
     ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
                                          HAServerType::DHCPv4));
 
+    // Make sure we wait for the acks from the backup server to be able to
+    // test the case of sending lease updates even though the service is
+    // in the state in which the lease updates are normally not sent.
+    ha_impl.config_->setWaitBackupAck(true);
+
     // Create callout handle to be used for passing arguments to the
     // callout.
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
     ASSERT_TRUE(callout_handle);
 
+    // Set the hook index so we can park packets.
+    callout_handle->setCurrentHook(test_hooks.hook_index_leases4_committed_);
+
     // query4
     Pkt4Ptr query4 = createMessage4(DHCPREQUEST, 1, 0, 0);
     callout_handle->setArgument("query4", query4);
@@ -339,12 +373,15 @@ TEST_F(HAImplTest, leases4Committed) {
     // Set initial status.
     callout_handle->setStatus(CalloutHandle::NEXT_STEP_CONTINUE);
 
+    // Park the packet.
+    HooksManager::park("leases4_committed", query4, []{});
+
     // There are no leases so the callout should return.
     ASSERT_NO_THROW(ha_impl.leases4Committed(*callout_handle));
 
     // No updates are generated so the default status should not be modified.
     EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
-    EXPECT_FALSE(callout_handle->getParkingLotHandlePtr()->drop(query4));
+    EXPECT_TRUE(callout_handle->getParkingLotHandlePtr()->drop(query4));
 
     // Create a lease and pass it to the callout, but temporarily disable lease
     // updates.
@@ -357,16 +394,24 @@ TEST_F(HAImplTest, leases4Committed) {
 
     ha_impl.config_->setSendLeaseUpdates(false);
 
+    // Park the packet.
+    HooksManager::park("leases4_committed", query4, []{});
+
     // Run the callout again.
     ASSERT_NO_THROW(ha_impl.leases4Committed(*callout_handle));
 
     // No updates are generated so the default status should not be modified.
     EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
-    EXPECT_FALSE(callout_handle->getParkingLotHandlePtr()->drop(query4));
+    EXPECT_TRUE(callout_handle->getParkingLotHandlePtr()->drop(query4));
 
     // Enable updates and retry.
     ha_impl.config_->setSendLeaseUpdates(true);
     callout_handle->setArgument("leases4", leases4);
+
+    // Park the packet.
+    HooksManager::park("leases4_committed", query4, []{});
+
+    // Run the callout again.
     ASSERT_NO_THROW(ha_impl.leases4Committed(*callout_handle));
 
     // This time the lease update should be generated and the status should
@@ -386,11 +431,19 @@ TEST_F(HAImplTest, leases6Committed) {
     ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
                                          HAServerType::DHCPv6));
 
+    // Make sure we wait for the acks from the backup server to be able to
+    // test the case of sending lease updates even though the service is
+    // in the state in which the lease updates are normally not sent.
+    ha_impl.config_->setWaitBackupAck(true);
+
     // Create callout handle to be used for passing arguments to the
     // callout.
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
     ASSERT_TRUE(callout_handle);
 
+    // Set the hook index so we can park packets.
+    callout_handle->setCurrentHook(test_hooks.hook_index_leases6_committed_);
+
     // query6
     Pkt6Ptr query6 = createMessage6(DHCPV6_REQUEST, 1, 0);
     callout_handle->setArgument("query6", query6);
@@ -406,12 +459,15 @@ TEST_F(HAImplTest, leases6Committed) {
     // Set initial status.
     callout_handle->setStatus(CalloutHandle::NEXT_STEP_CONTINUE);
 
+    // Park the packet.
+    HooksManager::park("leases6_committed", query6, []{});
+
     // There are no leases so the callout should return.
     ASSERT_NO_THROW(ha_impl.leases6Committed(*callout_handle));
 
     // No updates are generated so the default status should not be modified.
     EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
-    EXPECT_FALSE(callout_handle->getParkingLotHandlePtr()->drop(query6));
+    EXPECT_TRUE(callout_handle->getParkingLotHandlePtr()->drop(query6));
 
     // Create a lease and pass it to the callout, but temporarily disable lease
     // updates.
@@ -423,16 +479,24 @@ TEST_F(HAImplTest, leases6Committed) {
 
     ha_impl.config_->setSendLeaseUpdates(false);
 
+    // Park the packet.
+    HooksManager::park("leases6_committed", query6, []{});
+
     // Run the callout again.
     ASSERT_NO_THROW(ha_impl.leases6Committed(*callout_handle));
 
     // No updates are generated so the default status should not be modified.
     EXPECT_EQ(CalloutHandle::NEXT_STEP_CONTINUE, callout_handle->getStatus());
-    EXPECT_FALSE(callout_handle->getParkingLotHandlePtr()->drop(query6));
+    EXPECT_TRUE(callout_handle->getParkingLotHandlePtr()->drop(query6));
 
     // Enable updates and retry.
     ha_impl.config_->setSendLeaseUpdates(true);
     callout_handle->setArgument("leases6", leases6);
+
+    // Park the packet.
+    HooksManager::park("leases6_committed", query6, []{});
+
+    // Run the callout again.
     ASSERT_NO_THROW(ha_impl.leases6Committed(*callout_handle));
 
     // This time the lease update should be generated and the status should
@@ -640,4 +704,111 @@ TEST_F(HAImplTest, statusGetBackupServer) {
     EXPECT_TRUE(isEquivalent(got, Element::fromJSON(expected)));
 }
 
+// Tests status-get command processed handler for primary server being in the
+// passive-backup state.
+TEST_F(HAImplTest, statusGetPassiveBackup) {
+    TestHAImpl ha_impl;
+    ASSERT_NO_THROW(ha_impl.configure(createValidPassiveBackupJsonConfiguration()));
+
+    // Starting the service is required prior to running any callouts.
+    NetworkStatePtr network_state(new NetworkState(NetworkState::DHCPv4));
+    ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
+                                         HAServerType::DHCPv4));
+
+    std::string name = "status-get";
+    ConstElementPtr response =
+        Element::fromJSON("{ \"arguments\": { \"pid\": 1 }, \"result\": 0 }");
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+
+    callout_handle->setArgument("name", name);
+    callout_handle->setArgument("response", response);
+
+    ASSERT_NO_THROW(ha_impl.commandProcessed(*callout_handle));
+
+    ConstElementPtr got;
+    callout_handle->getArgument("response", got);
+    ASSERT_TRUE(got);
+
+    std::string expected =
+        "{"
+        "    \"arguments\": {"
+        "        \"high-availability\": ["
+        "            {"
+        "                \"ha-mode\": \"passive-backup\","
+        "                \"ha-servers\": {"
+        "                    \"local\": {"
+        "                        \"role\": \"primary\","
+        "                        \"scopes\": [ \"server1\" ],"
+        "                        \"state\": \"passive-backup\""
+        "                    }"
+        "                }"
+        "            }"
+        "        ],"
+        "        \"pid\": 1"
+        "    },"
+        "    \"result\": 0"
+        "}";
+    EXPECT_TRUE(isEquivalent(got, Element::fromJSON(expected)));
+}
+
+// Test ha-maintenance-notify command handler.
+TEST_F(HAImplTest, maintenanceNotify) {
+    HAImpl ha_impl;
+    ASSERT_NO_THROW(ha_impl.configure(createValidJsonConfiguration()));
+
+    // Starting the service is required prior to running any callouts.
+    NetworkStatePtr network_state(new NetworkState(NetworkState::DHCPv4));
+    ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
+                                         HAServerType::DHCPv4));
+
+    ConstElementPtr command = Element::fromJSON(
+        "{"
+        "    \"command\": \"ha-maintenance-notify\","
+        "    \"arguments\": {"
+        "        \"cancel\": false"
+        "    }"
+         "}"
+    );
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+    callout_handle->setArgument("command", command);
+
+    ASSERT_NO_THROW(ha_impl.maintenanceNotifyHandler(*callout_handle));
+
+    ConstElementPtr response;
+    callout_handle->getArgument("response", response);
+    ASSERT_TRUE(response);
+
+    checkAnswer(response, CONTROL_RESULT_SUCCESS, "Server is in-maintenance state.");
+}
+
+// Test ha-reset command handler.
+TEST_F(HAImplTest, haReset) {
+    HAImpl ha_impl;
+    ASSERT_NO_THROW(ha_impl.configure(createValidJsonConfiguration()));
+
+    // Starting the service is required prior to running any callouts.
+    NetworkStatePtr network_state(new NetworkState(NetworkState::DHCPv4));
+    ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
+                                         HAServerType::DHCPv4));
+
+    ConstElementPtr command = Element::fromJSON(
+        "{"
+        "    \"command\": \"ha-reset\""
+        "}"
+    );
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+    callout_handle->setArgument("command", command);
+
+    ASSERT_NO_THROW(ha_impl.haResetHandler(*callout_handle));
+
+    ConstElementPtr response;
+    callout_handle->getArgument("response", response);
+    ASSERT_TRUE(response);
+
+    checkAnswer(response, CONTROL_RESULT_SUCCESS, "HA state machine already in WAITING state.");
+}
+
 }
