@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,7 +11,6 @@
 #include <ha_test.h>
 #include <ha_config.h>
 #include <ha_service.h>
-#include <http/http_thread_pool.h>
 
 #include <util/multi_threading_mgr.h>
 #include <testutils/gtest_utils.h>
@@ -410,10 +409,10 @@ TEST_F(HAMtServiceTest, multiThreadingConfigStartup) {
 
     std::vector<Scenario> scenarios {
         {
-            "1 no ha+mt/default",
+            "1 ha+mt by default",
             "",
             dhcp_mt, 4,
-            !ha_mt, !listener, 0, 0
+            ha_mt, listener, 4, 4
         },
         {
             "2 dhcp mt enabled, ha mt disabled",
@@ -559,4 +558,4 @@ TEST_F(HAMtServiceTest, multiThreadingConfigStartup) {
     }
 }
 
-} // end of anonymous namespace
+}  // namespace
