@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -155,6 +155,10 @@ HATest::createValidJsonConfiguration(const HAConfig::HAMode& ha_mode) const {
         "         \"max-response-delay\": 1000,"
         "         \"max-ack-delay\": 10000,"
         "         \"max-unacked-clients\": 10,"
+        "         \"max-rejected-clients\": 10,"
+        "         \"multi-threading\": {"
+        "             \"enable-multi-threading\": false"
+        "         },"
         "         \"wait-backup-ack\": false,"
         "         \"peers\": ["
         "             {"
@@ -191,6 +195,9 @@ HATest::createValidPassiveBackupJsonConfiguration() const {
         "     {"
         "         \"this-server-name\": \"server1\","
         "         \"mode\": \"passive-backup\","
+        "         \"multi-threading\": {"
+        "             \"enable-multi-threading\": false"
+        "         },"
         "         \"wait-backup-ack\": false,"
         "         \"peers\": ["
         "             {"
@@ -366,6 +373,6 @@ HATest::makeHAMtJson(bool enable_multi_threading, bool http_dedicated_listener,
     return (ss.str());
 }
 
-} // end of namespace isc::ha::test
-} // end of namespace isc::ha
-} // end of namespace isc
+}  // namespace test
+}  // namespace ha
+}  // namespace isc
