@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -16,6 +16,7 @@
 #include <dhcpsrv/resource_handler.h>
 #include <cc/command_interpreter.h>
 #include <cc/data.h>
+#include <process/daemon.h>
 #include <stats/stats_mgr.h>
 #include <testutils/user_context_utils.h>
 #include <testutils/multi_threading_utils.h>
@@ -49,6 +50,16 @@ public:
         unloadLibs();
     }
 
+    /// @brief Set family.
+    void setFamily(uint16_t family) {
+        isc::dhcp::CfgMgr::instance().setFamily(family);
+        if (family == AF_INET) {
+            isc::process::Daemon::setProcName("kea-dhcp4");
+        } else {
+            isc::process::Daemon::setProcName("kea-dhcp6");
+        }
+    }
+
     /// @brief Adds library/parameters to list of libraries to be loaded
     void addLib(const std::string& lib, isc::data::ConstElementPtr params) {
         libraries_.push_back(make_pair(lib, params));
@@ -300,7 +311,7 @@ public:
                                       const isc::dhcp::SubnetID& subnet_id,
                                       const uint8_t hw_address_pattern,
                                       const uint8_t client_id_pattern,
-                                      bool declined = false) {
+                                      bool declined = false, uint32_t pool_id = 0) {
         isc::dhcp::Lease4Ptr lease(new isc::dhcp::Lease4());
 
         lease->addr_ = isc::asiolink::IOAddress(ip_address);
@@ -320,6 +331,7 @@ public:
         lease->fqdn_fwd_ = false;
         lease->fqdn_rev_ = true;
         lease->hostname_ = "myhost.example.com.";
+        lease->pool_id_ = pool_id;
 
         return (lease);
     }
@@ -340,7 +352,7 @@ public:
     isc::dhcp::Lease6Ptr createLease6(const std::string& ip_address,
                                       const isc::dhcp::SubnetID& subnet_id,
                                       const uint8_t duid_pattern,
-                                      bool declined = false) {
+                                      bool declined = false, uint32_t pool_id = 0) {
         isc::dhcp::Lease6Ptr lease(new isc::dhcp::Lease6());
 
         lease->addr_ = isc::asiolink::IOAddress(ip_address);
@@ -361,6 +373,7 @@ public:
         lease->fqdn_fwd_ = false;
         lease->fqdn_rev_ = true;
         lease->hostname_ = "myhost.example.com.";
+        lease->pool_id_ = pool_id;
 
         return (lease);
     }
@@ -378,7 +391,12 @@ public:
     void initLeaseMgr(bool v6, bool insert_lease, bool declined = false) {
         isc::dhcp::LeaseMgrFactory::destroy();
         std::ostringstream s;
-        s << "type=memfile persist=false " << (v6 ? "universe=6" : "universe=4");
+        s << "type=memfile persist=false ";
+        if (v6) {
+            s << "universe=6 extended-info-tables=true";
+        } else {
+            s << "universe=4";
+        }
         isc::dhcp::LeaseMgrFactory::create(s.str());
 
         lmptr_ = &(isc::dhcp::LeaseMgrFactory::instance());
@@ -404,7 +422,7 @@ public:
         if (insert_lease) {
             if (v6) {
                 lmptr_->addLease(createLease6("2001:db8:1::1", 66, 0x42, declined));
-                lmptr_->addLease(createLease6("2001:db8:1::2", 66, 0x56, declined));
+                lmptr_->addLease(createLease6("2001:db8:1::2", 66, 0x56, declined, 5));
                 lmptr_->addLease(createLease6("2001:db8:2::1", 99, 0x42, declined));
                 lmptr_->addLease(createLease6("2001:db8:2::2", 99, 0x56, declined));
                 if (declined) {
@@ -430,7 +448,7 @@ public:
                     int64_t(2));
             } else {
                 lmptr_->addLease(createLease4("192.0.2.1", 44, 0x08, 0x42, declined));
-                lmptr_->addLease(createLease4("192.0.2.2", 44, 0x09, 0x56, declined));
+                lmptr_->addLease(createLease4("192.0.2.2", 44, 0x09, 0x56, declined, 5));
                 lmptr_->addLease(createLease4("192.0.3.1", 88, 0x08, 0x42, declined));
                 lmptr_->addLease(createLease4("192.0.3.2", 88, 0x09, 0x56, declined));
                 if (declined) {
