@@ -113,7 +113,7 @@ public:
                                     const db::DbCallback db_reconnect_callback);
 
     /// @brief Destructor.
-    virtual ~MySqlConfigBackendImpl() {};
+    virtual ~MySqlConfigBackendImpl();
 
     /// @brief Creates MySQL binding from an @c Optional of integer type.
     ///
@@ -127,7 +127,7 @@ public:
         if (value.unspecified()) {
             return (db::MySqlBinding::createNull());
         }
-        return (db::MySqlBinding::createInteger(value));
+        return (db::MySqlBinding::createInteger(value.get()));
     }
 
     /// @brief Creates MySQL binding from a @c Triplet.
@@ -562,6 +562,7 @@ public:
     /// - formatted_value,
     /// - space,
     /// - persistent,
+    /// - cancelled,
     /// - dhcp4_subnet_id/dhcp6_subnet_id,
     /// - scope_id,
     /// - user_context,
