@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -31,6 +31,7 @@ using namespace isc::asiolink;
 using namespace isc::hooks;
 using namespace isc::stats;
 using namespace isc::util;
+using namespace isc::log;
 using namespace std;
 
 namespace isc {
@@ -77,6 +78,8 @@ public:
                 os << "[subnets " << first_subnet_id_
                    << " through " << last_subnet_id_ << "]";
                 break;
+            default:
+                os << "unsupported";
             }
 
             return (os.str());
@@ -204,13 +207,21 @@ public:
     void addValueRow6(ElementPtr value_rows, const SubnetID &subnet_id,
                       int64_t assigned, int64_t declined, int64_t assigned_pds);
 
-    /// @brief Fetches a single statistic for a subnet from StatsMgr
+    /// @brief Fetches a single integer statistic for a subnet from StatsMgr.
     ///
     /// Uses the given id and name to query the StatsMgr for the desired value.
     ///
-    /// @param subnet_id id of the desired subnet
-    /// @param name name of the desired statistic
+    /// @param subnet_id the subnet ID for the desired statistic
+    /// @param name the name of the desired statistic
     int64_t getSubnetStat(const SubnetID& subnet_id, const std::string& name);
+
+    /// @brief Fetches a single bigint statistic for a subnet from StatsMgr.
+    ///
+    /// Uses the given id and name to query the StatsMgr for the desired value.
+    ///
+    /// @param subnet_id the subnet ID for the desired statistic
+    /// @param name the name of the desired statistic
+    int128_t getBigSubnetStat(const SubnetID& subnet_id, const std::string& name);
 };
 
 int
@@ -387,11 +398,17 @@ LeaseStatCmdsImpl::makeResultSet4(const ElementPtr& result_wrapper,
     // First we need to determine the range of configured subnets
     // which meet the selection criteria.  If the range contains
     // no subnets we punt.
+    // Iterate over the selected range of configured subnets generating
+    // a result-set row for each one.  If a subnet has data in the query
+    // content use it, otherwise, it gets a row with totals only.  This
+    // way we send back a row for every selected subnet.
     const Subnet4Collection* subnets =
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
+
+    // Set the bounds on the selected subnet range
     const auto& idx = subnets->get<SubnetSubnetIdIndexTag>();
 
-    // Init to ALL so we can use auto ;)
+    // Init to ALL so we can use auto
     auto lower = idx.begin();
     auto upper = idx.end();
     switch (params.select_mode_) {
@@ -420,7 +437,7 @@ LeaseStatCmdsImpl::makeResultSet4(const ElementPtr& result_wrapper,
             << params.last_subnet_id_ << " includes no known subnets");
     }
 
-    // Now, that we have a valid range, run the Lease query.
+    // Now we can run the stats query.
     LeaseStatsQueryPtr query;
     switch (params.select_mode_) {
     case LeaseStatsQuery::ALL_SUBNETS:
@@ -435,9 +452,12 @@ LeaseStatCmdsImpl::makeResultSet4(const ElementPtr& result_wrapper,
                 .startSubnetRangeLeaseStatsQuery4(params.first_subnet_id_,
                                                   params.last_subnet_id_);
         break;
+    default:
+        return (0);
     }
 
-    // Create the empty result-set.
+    // Create the result-set map.
+    // labels could be class statics?
     std::vector<std::string>column_labels = { "subnet-id", "total-addresses",
                                               "cumulative-assigned-addresses",
                                               "assigned-addresses",
@@ -449,14 +469,23 @@ LeaseStatCmdsImpl::makeResultSet4(const ElementPtr& result_wrapper,
     bool query_eof = !(query->getNextRow(query_row));
 
     // Now we iterate over the selected range, building rows accordingly.
+    bool orphaned_stats = false;
     for (auto cur_subnet = lower; cur_subnet != upper; ++cur_subnet) {
         SubnetID cur_id = (*cur_subnet)->getID();
 
-        // Add total only rows for subnets that occur before,
-        // in-between, or after the subnets in the query content
-        if ((cur_id < query_row.subnet_id_) ||
-            (cur_id > query_row.subnet_id_) ||
-            (query_eof)) {
+        // Skip any unexpected result set rows.  These occur when
+        // subnets no longer exist but either their leases (memfile)
+        // or their leaseX-stat rows (db lease backends) still do.
+        while ((cur_id > query_row.subnet_id_) && (!query_eof)) {
+            orphaned_stats = true;
+            query_eof = !(query->getNextRow(query_row));
+        }
+
+        // Add total only rows for subnets that occur before
+        // or after the subnets in the query content. These are
+        // subnets which exist but for which there is not yet any
+        // lease data.
+        if ((cur_id < query_row.subnet_id_) || (query_eof)) {
             // Generate a totals only row
             addValueRow4(value_rows, cur_id, 0, 0);
             continue;
@@ -477,21 +506,29 @@ LeaseStatCmdsImpl::makeResultSet4(const ElementPtr& result_wrapper,
                 declined = query_row.state_count_;
             }
 
+            // Get next query row
             query_eof = !(query->getNextRow(query_row));
         }
-
         // Add the row for the current subnet
         if (add_row) {
             addValueRow4(value_rows, cur_id, assigned, declined);
         }
     }
 
+    // If there are any orphaned statistics log it.
+    if (!(query_eof) || orphaned_stats) {
+        LOG_DEBUG(stat_cmds_logger, DBGLVL_TRACE_BASIC, STAT_CMDS_LEASE4_ORPHANED_STATS);
+    }
+
     return (value_rows->size());
 }
 
 uint64_t
 LeaseStatCmdsImpl::makeResultSet6(const ElementPtr& result_wrapper,
                                   const Parameters& params) {
+    // First we need to determine the range of configured subnets
+    // which meet the selection criteria.  If the range contains
+    // no subnets we punt.
     // Iterate over the selected range of configured subnets generating
     // a result-set row for each one.  If a subnet has data in the query
     // content use it, otherwise, it gets a row with totals only.  This
@@ -502,12 +539,12 @@ LeaseStatCmdsImpl::makeResultSet6(const ElementPtr& result_wrapper,
     // Set the bounds on the selected subnet range
     const auto& idx = subnets->get<SubnetSubnetIdIndexTag>();
 
-    // Init to all so we can use auto ;)
+    // Init to ALL so we can use auto
     auto lower = idx.begin();
     auto upper = idx.end();
     switch (params.select_mode_) {
     case LeaseStatsQuery::SINGLE_SUBNET:
-        lower = idx.lower_bound(params.first_subnet_id_);
+        lower = idx.find(params.first_subnet_id_);
         // If it's an unknown subnet, punt.
         if (lower == idx.end()) {
             isc_throw(NotFound, "subnet-id: "
@@ -524,23 +561,13 @@ LeaseStatCmdsImpl::makeResultSet6(const ElementPtr& result_wrapper,
         break;
     }
 
-    // If it's empty range, punt.
+    // If it's an empty range, punt.
     if (lower == upper) {
         isc_throw(NotFound, "selected ID range: "
             << params.first_subnet_id_ << " through "
             << params.last_subnet_id_ << " includes no known subnets");
     }
 
-    // Create the result-set map.
-    // labels could be class statics?
-    std::vector<std::string>column_labels = { "subnet-id", "total-nas",
-                                              "cumulative-assigned-nas",
-                                              "assigned-nas",
-                                              "declined-nas", "total-pds",
-                                              "cumulative-assigned-pds",
-                                              "assigned-pds" };
-    ElementPtr value_rows = createResultSet(result_wrapper, column_labels);
-
     // Now we can run the stats query.
     LeaseStatsQueryPtr query;
     switch (params.select_mode_) {
@@ -556,33 +583,55 @@ LeaseStatCmdsImpl::makeResultSet6(const ElementPtr& result_wrapper,
                 .startSubnetRangeLeaseStatsQuery6(params.first_subnet_id_,
                                                   params.last_subnet_id_);
         break;
+    default:
+        return (0);
     }
 
+    // Create the result-set map.
+    // labels could be class statics?
+    std::vector<std::string>column_labels = { "subnet-id", "total-nas",
+                                              "cumulative-assigned-nas",
+                                              "assigned-nas",
+                                              "declined-addresses", "total-pds",
+                                              "cumulative-assigned-pds",
+                                              "assigned-pds" };
+    ElementPtr value_rows = createResultSet(result_wrapper, column_labels);
+
     // Get the first query row
     LeaseStatsRow query_row;
     bool query_eof = !(query->getNextRow(query_row));
 
+    // Now we iterate over the selected range, building rows accordingly.
+    bool orphaned_stats = false;
     for (auto cur_subnet = lower; cur_subnet != upper; ++cur_subnet) {
         SubnetID cur_id = (*cur_subnet)->getID();
 
-        // Add total only rows for subnets that occur before,
-        // in-between, or after subnets in the query content
-        if ((cur_id < query_row.subnet_id_) ||
-            (cur_id > query_row.subnet_id_) ||
-            (query_eof)) {
+        // Skip any unexpected result set rows.  These occur when
+        // subnets no longer exist but either their leases (memfile)
+        // or their leaseX-stat rows (db lease backends) still do.
+        while ((cur_id > query_row.subnet_id_) && (!query_eof)) {
+            orphaned_stats = true;
+            query_eof = !(query->getNextRow(query_row));
+        }
+
+        // Add total only rows for subnets that occur before
+        // or after the subnets in the query content. These are
+        // subnets which exist but for which there is not yet any
+        // lease data.
+        if ((cur_id < query_row.subnet_id_) || (query_eof)) {
             // Generate a totals only row
             addValueRow6(value_rows, cur_id, 0, 0, 0);
             continue;
         }
 
-        // Current subnet matches query row, so iterate over
-        // its query rows and accumulate them into a result-set row.
+        // Current subnet matches query row, so iterate over its
+        // query rows (one per state) and accumulate them
+        // into a result-set row.
         int64_t assigned = 0;
         int64_t declined = 0;
         int64_t assigned_pds = 0;
         bool add_row = false;
         while (!query_eof && (query_row.subnet_id_ == cur_id)) {
-
             if (query_row.lease_state_ == Lease::STATE_DEFAULT) {
                 add_row = true;
                 if (query_row.lease_type_ == Lease::TYPE_NA) {
@@ -598,12 +647,17 @@ LeaseStatCmdsImpl::makeResultSet6(const ElementPtr& result_wrapper,
             // Get next query row
             query_eof = !(query->getNextRow(query_row));
         }
-
+        // Add the row for the current subnet
         if (add_row) {
             addValueRow6(value_rows, cur_id, assigned, declined, assigned_pds);
         }
     }
 
+    // If there are any orphaned statistics log it.
+    if (!(query_eof) || orphaned_stats) {
+        LOG_DEBUG(stat_cmds_logger, DBGLVL_TRACE_BASIC, STAT_CMDS_LEASE6_ORPHANED_STATS);
+    }
+
     return (value_rows->size());
 }
 
@@ -636,7 +690,7 @@ LeaseStatCmdsImpl::createResultSet(const ElementPtr &result_wrapper,
 
 void
 LeaseStatCmdsImpl::addValueRow4(ElementPtr value_rows, const SubnetID &subnet_id,
-                          int64_t assigned, int64_t declined) {
+                                int64_t assigned, int64_t declined) {
     ElementPtr row = Element::createList();
     row->add(Element::create(static_cast<int64_t>(subnet_id)));
     row->add(Element::create(getSubnetStat(subnet_id, "total-addresses")));
@@ -648,14 +702,14 @@ LeaseStatCmdsImpl::addValueRow4(ElementPtr value_rows, const SubnetID &subnet_id
 
 void
 LeaseStatCmdsImpl::addValueRow6(ElementPtr value_rows, const SubnetID &subnet_id,
-                           int64_t assigned, int64_t declined, int64_t assigned_pds) {
+                                int64_t assigned, int64_t declined, int64_t assigned_pds) {
     ElementPtr row = Element::createList();
     row->add(Element::create(static_cast<int64_t>(subnet_id)));
-    row->add(Element::create(getSubnetStat(subnet_id, "total-nas")));
+    row->add(Element::create(getBigSubnetStat(subnet_id, "total-nas")));
     row->add(Element::create(getSubnetStat(subnet_id, "cumulative-assigned-nas")));
     row->add(Element::create(assigned));
     row->add(Element::create(declined));
-    row->add(Element::create(getSubnetStat(subnet_id, "total-pds")));
+    row->add(Element::create(getBigSubnetStat(subnet_id, "total-pds")));
     row->add(Element::create(getSubnetStat(subnet_id, "cumulative-assigned-pds")));
     row->add(Element::create(assigned_pds));
     value_rows->add(row);
@@ -672,13 +726,23 @@ LeaseStatCmdsImpl::getSubnetStat(const SubnetID& subnet_id, const std::string& n
     return (0);
 }
 
+int128_t
+LeaseStatCmdsImpl::getBigSubnetStat(const SubnetID& subnet_id, const std::string& name) {
+    ObservationPtr stat = StatsMgr::instance().
+                          getObservation(StatsMgr::generateName("subnet", subnet_id, name));
+    if (stat) {
+        return (stat->getBigInteger().first);
+    }
+
+    return (0);
+}
+
 // Using a critical section to avoid any changes in parallel.
 
 int
 StatCmds::statLease4GetHandler(CalloutHandle& handle) {
     try {
         LeaseStatCmdsImpl impl;
-        MultiThreadingCriticalSection cs;
         return (impl.statLease4GetHandler(handle));
     } catch (const std::exception& ex) {
 
@@ -692,7 +756,6 @@ int
 StatCmds::statLease6GetHandler(CalloutHandle& handle) {
     try {
         LeaseStatCmdsImpl impl;
-        MultiThreadingCriticalSection cs;
         return (impl.statLease6GetHandler(handle));
     } catch (const std::exception& ex) {
 
