@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,11 +10,12 @@
 #include <exceptions/exceptions.h>
 #include <hooks/hooks_manager.h>
 #include <config/command_mgr.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/lease_mgr.h>
 #include <dhcpsrv/lease_mgr_factory.h>
-#include <dhcpsrv/cfgmgr.h>
 #include <cc/command_interpreter.h>
 #include <cc/data.h>
+#include <process/daemon.h>
 #include <stats/stats_mgr.h>
 #include <testutils/gtest_utils.h>
 
@@ -24,11 +25,12 @@
 
 using namespace std;
 using namespace isc;
-using namespace isc::hooks;
+using namespace isc::asiolink;
 using namespace isc::config;
 using namespace isc::data;
 using namespace isc::dhcp;
-using namespace isc::asiolink;
+using namespace isc::hooks;
+using namespace isc::process;
 using namespace isc::stats;
 using namespace boost::posix_time;
 
@@ -50,6 +52,16 @@ public:
         unloadLibs();
     }
 
+    /// @brief Set family.
+    void setFamily(uint16_t family = AF_INET) {
+        CfgMgr::instance().setFamily(family);
+        if (family == AF_INET) {
+            Daemon::setProcName("kea-dhcp4");
+        } else {
+            Daemon::setProcName("kea-dhcp6");
+        }
+    }
+
     /// @brief Adds library/parameters to list of libraries to be loaded
     void addLib(const std::string& lib, ConstElementPtr params) {
         libraries_.push_back(make_pair(lib, params));
@@ -604,11 +616,13 @@ TEST_F(StatCmdsTest, commands) {
 
     vector<string> cmds = { "stat-lease4-get",
                             "stat-lease6-get" };
+    setFamily();
     testCommands(cmds);
 }
 
 // Check that the library can be loaded and unloaded multiple times.
 TEST_F(StatCmdsTest, multipleLoads) {
+    setFamily();
     testMultipleLoads();
 }
 
@@ -621,6 +635,8 @@ struct TestScenario {
 
 // Verifies detection of invalid v4 input parameters.
 TEST_F(StatCmdsTest, StatLease4GetBadParams) {
+    setFamily(AF_INET);
+
     // Initialize lease manager
     initLeaseMgr4();
 
@@ -787,6 +803,7 @@ TEST_F(StatCmdsTest, StatLease4GetBadParams) {
 // Verifies result content for valid v4 statistic commands.
 // These test scenarios are all valid, and not expected to throw.
 TEST_F(StatCmdsTest, statLease4GetValid) {
+    setFamily(AF_INET);
 
     // Initialize lease manager.
     initLeaseMgr4();
@@ -986,6 +1003,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
 // Verifies result content for valid v4 statistic commands that
 // result in no matching subnets.
 TEST_F(StatCmdsTest, statLease4GetSubnetsNotFound) {
+    setFamily(AF_INET);
 
     // Initialize lease manager.
     initLeaseMgr4();
@@ -1046,6 +1064,8 @@ TEST_F(StatCmdsTest, statLease4GetSubnetsNotFound) {
 
 // Verifies detection of invalid v6 input parameters.
 TEST_F(StatCmdsTest, StatLease6GetBadParams) {
+    setFamily(AF_INET6);
+
     // Initialize lease manager
     initLeaseMgr6();
 
@@ -1212,6 +1232,7 @@ TEST_F(StatCmdsTest, StatLease6GetBadParams) {
 // Verifies result content for valid v6 statistic commands.
 // These test scenarios are all valid, and not expected to throw.
 TEST_F(StatCmdsTest, statLease6GetValid) {
+    setFamily(AF_INET6);
 
     // Initialize lease manager
     initLeaseMgr6();
@@ -1232,7 +1253,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1256,7 +1277,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1283,7 +1304,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1309,7 +1330,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1337,7 +1358,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1365,7 +1386,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1393,7 +1414,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1418,6 +1439,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
 // Verifies result content for valid v6 statistic commands that
 // result in no matching subnets.
 TEST_F(StatCmdsTest, statLease6GetSubnetsNotFound) {
+    setFamily(AF_INET6);
 
     // Initialize lease manager
     initLeaseMgr6();
@@ -1478,6 +1500,7 @@ TEST_F(StatCmdsTest, statLease6GetSubnetsNotFound) {
 // Verifies that statistics for v4 subnets which no longer
 // exist are dropped from the result sets.
 TEST_F(StatCmdsTest, statLease4OrphanedStats) {
+    setFamily(AF_INET);
 
     // Initialize lease manager.
     initLeaseMgr4();
@@ -1529,6 +1552,7 @@ TEST_F(StatCmdsTest, statLease4OrphanedStats) {
 // Verifies that statistics for v6 subnets which no longer
 // exist are dropped from the result sets.
 TEST_F(StatCmdsTest, statLease6OrphanedStats) {
+    setFamily(AF_INET6);
 
     // Initialize lease manager.
     initLeaseMgr6();
@@ -1557,7 +1581,7 @@ TEST_F(StatCmdsTest, statLease6OrphanedStats) {
         "   \"columns\": [\n"
         "        \"subnet-id\", \"total-nas\",\n"
         "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
-        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"declined-addresses\", \"total-pds\",\n"
         "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
         "   ],\n"
         "   \"rows\": [\n"
@@ -1579,4 +1603,3 @@ TEST_F(StatCmdsTest, statLease6OrphanedStats) {
 }
 
 } // end of anonymous namespace
-
