@@ -1,10 +1,11 @@
-// Copyright (C) 2011-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <asiolink/asio_wrapper.h>
 #include <asiolink/io_service.h>
 
@@ -16,23 +17,6 @@
 namespace isc {
 namespace asiolink {
 
-namespace {
-// A trivial wrapper for boost::function.  SunStudio doesn't seem to be capable
-// of handling a boost::function object if directly passed to
-// io_service::post().
-class CallbackWrapper {
-public:
-    CallbackWrapper(const boost::function<void()>& callback) :
-        callback_(callback)
-    {}
-    void operator()() {
-        callback_();
-    }
-private:
-    boost::function<void()> callback_;
-};
-}
-
 class IOServiceImpl {
 private:
     IOServiceImpl(const IOService& source);
@@ -41,8 +25,9 @@ public:
     /// \brief The constructor
     IOServiceImpl() :
         io_service_(),
-        work_(new boost::asio::io_service::work(io_service_))
-    {};
+        work_(new boost::asio::io_service::work(io_service_)) {
+    };
+
     /// \brief The destructor.
     ~IOServiceImpl() {};
     //@}
@@ -75,7 +60,21 @@ public:
     /// \brief Stop the underlying event loop.
     ///
     /// This will return the control to the caller of the \c run() method.
-    void stop() { io_service_.stop();} ;
+    void stop() {
+        io_service_.stop();
+    }
+
+    /// \brief Indicates if the IOService has been stopped.
+    ///
+    /// \return true if the IOService has been stopped, false otherwise.
+    bool stopped() const {
+        return (io_service_.stopped());
+    }
+
+    /// \brief Restarts the IOService in preparation for a subsequent \c run() invocation.
+    void restart() {
+        io_service_.reset();
+    }
 
     /// \brief Removes IO service work object to let it finish running
     /// when all handlers have been invoked.
@@ -89,22 +88,26 @@ public:
     /// that share the same \c io_service with the authoritative server.
     /// It will eventually be removed once the wrapper interface is
     /// generalized.
-    boost::asio::io_service& get_io_service() { return io_service_; };
-    void post(const boost::function<void ()>& callback) {
-        const CallbackWrapper wrapper(callback);
-        io_service_.post(wrapper);
+    boost::asio::io_service& get_io_service() {
+        return (io_service_);
     }
+
+    /// \brief Post a callback on the IO service
+    ///
+    /// \param callback The callback to be run on the IO service.
+    void post(const std::function<void ()>& callback) {
+        io_service_.post(callback);
+    }
+
 private:
     boost::asio::io_service io_service_;
     boost::shared_ptr<boost::asio::io_service::work> work_;
 };
 
-IOService::IOService() {
-    io_impl_ = new IOServiceImpl();
+IOService::IOService() : io_impl_(new IOServiceImpl()) {
 }
 
 IOService::~IOService() {
-    delete io_impl_;
 }
 
 void
@@ -127,6 +130,16 @@ IOService::stop() {
     io_impl_->stop();
 }
 
+bool
+IOService::stopped() const {
+    return (io_impl_->stopped());
+}
+
+void
+IOService::restart() {
+    io_impl_->restart();
+}
+
 void
 IOService::stopWork() {
     io_impl_->stopWork();
@@ -138,7 +151,7 @@ IOService::get_io_service() {
 }
 
 void
-IOService::post(const boost::function<void ()>& callback) {
+IOService::post(const std::function<void ()>& callback) {
     return (io_impl_->post(callback));
 }
 
