@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,7 @@
 
 #include <asiolink/addr_utilities.h>
 #include <exceptions/exceptions.h>
+#include <util/bigints.h>
 
 #include <gtest/gtest.h>
 
@@ -17,8 +18,8 @@
 #include <stdlib.h>
 
 using namespace std;
-using namespace isc::dhcp;
 using namespace isc::asiolink;
+using namespace isc::util;
 
 namespace {
 
@@ -97,7 +98,7 @@ TEST(AddrUtilitiesTest, lastAddrInPrefix6) {
 
     IOAddress addr2("2001::");
 
-    // These are tricker, though, as they are done in 1 bit increments
+    // These are trickier, though, as they are done in 1 bit increments
 
     // the last address in 2001::/127 pool should be 2001::1
     EXPECT_EQ("2001::1", lastAddrInPrefix(addr2, 127).toText());
@@ -135,7 +136,7 @@ TEST(AddrUtilitiesTest, firstAddrInPrefix6) {
 
     IOAddress addr2("2001::ffff");
 
-    // These are tricker, though, as they are done in 1 bit increments
+    // These are trickier, though, as they are done in 1 bit increments
 
     // the first address in 2001::/127 pool should be 2001::1
     EXPECT_EQ("2001::fffe", firstAddrInPrefix(addr2, 127).toText());
@@ -267,6 +268,9 @@ TEST(AddrUtilitiesTest, prefixLengthFromRange4) {
     // Fail if a network boundary is crossed
     EXPECT_EQ(-1, plfr(IOAddress("10.0.0.255"), IOAddress("10.0.1.1")));
 
+    // Fail if first is not at the begin
+    EXPECT_EQ(-1, plfr(IOAddress("10.0.0.2"), IOAddress("10.0.0.5")));
+
     // The upper bound cannot be smaller than the lower bound
     EXPECT_THROW(plfr(IOAddress("192.0.2.5"), IOAddress("192.0.2.4")),
                  isc::BadValue);
@@ -324,6 +328,13 @@ TEST(AddrUtilitiesTest, prefixLengthFromRange6) {
     EXPECT_EQ(-1, plfr(IOAddress("2001:db8::ffff"),
                        IOAddress("2001:db8::1:1")));
 
+    // Fail if first is not at the begin
+    EXPECT_EQ(-1, plfr(IOAddress("2001:db8::2"), IOAddress("2001:db8::5")));
+    EXPECT_EQ(-1, plfr(IOAddress("2001:db8::2:0"),
+                       IOAddress("2001:db8::5:ffff")));
+    EXPECT_EQ(-1, plfr(IOAddress("2001:db8::2:ff00:0"),
+                       IOAddress("2001:db8::3:00ff:ffff")));
+
     // The upper bound cannot be smaller than the lower bound
     EXPECT_THROW(plfr(IOAddress("fe80::5"), IOAddress("fe80::4")),
                  isc::BadValue);
@@ -349,13 +360,29 @@ TEST(AddrUtilitiesTest, prefixesInRange) {
     EXPECT_EQ(uint64_t(9223372036854775808ull), prefixesInRange(64, 127));
 
     // How many /128 prefixes are in /64 pool?
-    EXPECT_EQ(std::numeric_limits<uint64_t>::max(),
-              prefixesInRange(64, 128));
+    EXPECT_EQ(uint128_t(1) << 64, prefixesInRange(64, 128));
 
     // Let's go overboard again. How many IPv6 addresses are there?
-    EXPECT_EQ(std::numeric_limits<uint64_t>::max(),
-              prefixesInRange(0, 128));
+    EXPECT_EQ(uint128_t(1) << 127, prefixesInRange(1, 128));
 
+    // Let's go overboard again. How many IPv6 addresses are there?
+    EXPECT_EQ(uint128_t(-1), prefixesInRange(0, 128));
 }
 
-}; // end of anonymous namespace
+// Checks the function which finds an IPv4 address from input address and offset.
+TEST(AddrUtilitiesTest, offsetIPv4Address) {
+    EXPECT_EQ("10.1.2.46", offsetAddress(IOAddress("10.1.1.45"), 257).toText());
+    EXPECT_EQ("10.1.7.9", offsetAddress(IOAddress("10.1.1.45"), 1500).toText());
+    // Using very large offset. The maximum IPv4 address should be returned.
+    EXPECT_EQ("255.255.255.255", offsetAddress(IOAddress("255.255.254.254"), 0xFFFFFFFFFFFFFFFA).toText());
+}
+
+// Checks the function which finds an IPv6 address from input address and offset.
+TEST(AddrUtilitiesTest, offsetIPv6Address) {
+    EXPECT_EQ("2001:db8:1::4", offsetAddress(IOAddress("2001:db8:1::4"), 0).toText());
+    EXPECT_EQ("2001:db8:1::10:3", offsetAddress(IOAddress("2001:db8:1::4"), 0xFFFFF).toText());
+    EXPECT_EQ("2001:db8:2::", offsetAddress(IOAddress("2001:db8:1:FFFF::1"), 0xFFFFFFFFFFFFFFFF).toText());
+    EXPECT_EQ("3000::1c", offsetAddress(IOAddress("3000::15"), 7).toText());
+}
+
+} // end of anonymous namespace
