@@ -1,18 +1,17 @@
-// Copyright (C) 2011-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
-
-#include <string>
+#include <asiolink/asio_wrapper.h>
+#include <asiolink/io_address.h>
+#include <asiolink/udp_endpoint.h>
 
 #include <gtest/gtest.h>
 
-#include <boost/asio.hpp>
-#include <asiolink/io_address.h>
-#include <asiolink/udp_endpoint.h>
+#include <string>
 
 using namespace isc::asiolink;
 using namespace std;
