@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -295,8 +295,7 @@ Connection::receiveHandler(const boost::system::error_code& ec,
             std::stringstream os;
             if (feed_.getProcessedText().empty()) {
                os << "no input data to discard";
-            }
-            else {
+            } else {
                os << "discarding partial command of "
                   << feed_.getProcessedText().size() << " bytes";
             }
