@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,11 +26,11 @@ namespace isc {
 namespace dhcp {
 
 Option4AddrLst::Option4AddrLst(uint8_t type)
-    :Option(V4, type) {
+    : Option(V4, type) {
 }
 
 Option4AddrLst::Option4AddrLst(uint8_t type, const AddressContainer& addrs)
-    :Option(V4, type) {
+    : Option(V4, type) {
     setAddresses(addrs);
     // don't set addrs_ directly. setAddresses() will do additional checks.
 }
@@ -38,7 +38,7 @@ Option4AddrLst::Option4AddrLst(uint8_t type, const AddressContainer& addrs)
 
 Option4AddrLst::Option4AddrLst(uint8_t type, OptionBufferConstIter first,
                                OptionBufferConstIter last)
-    :Option(V4, type) {
+    : Option(V4, type) {
     if ( (distance(first, last) % V4ADDRESS_LEN) ) {
         isc_throw(OutOfRange, "DHCPv4 Option4AddrLst " << type_
                   << " has invalid length=" << distance(first, last)
@@ -53,7 +53,7 @@ Option4AddrLst::Option4AddrLst(uint8_t type, OptionBufferConstIter first,
 }
 
 Option4AddrLst::Option4AddrLst(uint8_t type, const IOAddress& addr)
-    :Option(V4, type) {
+    : Option(V4, type) {
     setAddress(addr);
 }
 
@@ -63,14 +63,10 @@ Option4AddrLst::clone() const {
 }
 
 void
-Option4AddrLst::pack(isc::util::OutputBuffer& buf) const {
-
-    if (addrs_.size() * V4ADDRESS_LEN > 255) {
+Option4AddrLst::pack(isc::util::OutputBuffer& buf, bool check) const {
+    if (check && addrs_.size() * V4ADDRESS_LEN > 255) {
         isc_throw(OutOfRange, "DHCPv4 Option4AddrLst " << type_ << " is too big."
                   << "At most 255 bytes are supported.");
-        /// TODO Larger options can be stored as separate instances
-        /// of DHCPv4 options. Clients MUST concatenate them.
-        /// Fortunately, there are no such large options used today.
     }
 
     buf.writeUint8(type_);
@@ -94,7 +90,6 @@ void Option4AddrLst::setAddress(const isc::asiolink::IOAddress& addr) {
 }
 
 void Option4AddrLst::setAddresses(const AddressContainer& addrs) {
-
     // Do not copy it as a whole. addAddress() does sanity checks.
     // i.e. throw if someone tries to set IPv6 address.
     addrs_.clear();
@@ -104,7 +99,6 @@ void Option4AddrLst::setAddresses(const AddressContainer& addrs) {
     }
 }
 
-
 void Option4AddrLst::addAddress(const isc::asiolink::IOAddress& addr) {
     if (!addr.isV4()) {
         isc_throw(BadValue, "Can't store non-IPv4 address in "
@@ -114,7 +108,6 @@ void Option4AddrLst::addAddress(const isc::asiolink::IOAddress& addr) {
 }
 
 uint16_t Option4AddrLst::len() const {
-
     // Returns length of the complete option (option header + data length)
     return (getHeaderLen() + addrs_.size() * V4ADDRESS_LEN);
 }
