@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -348,7 +348,7 @@ Option4ClientFqdn::Option4ClientFqdn(OptionBufferConstIter first,
 }
 
 Option4ClientFqdn::~Option4ClientFqdn() {
-    delete(impl_);
+    delete (impl_);
 }
 
 Option4ClientFqdn::Option4ClientFqdn(const Option4ClientFqdn& source)
@@ -484,9 +484,9 @@ Option4ClientFqdn::getDomainNameType() const {
 }
 
 void
-Option4ClientFqdn::pack(isc::util::OutputBuffer& buf) const {
+Option4ClientFqdn::pack(isc::util::OutputBuffer& buf, bool check) const {
     // Header = option code and length.
-    packHeader(buf);
+    packHeader(buf, check);
     // Flags field.
     buf.writeUint8(impl_->flags_);
     // RCODE1 and RCODE2
