@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -44,7 +44,7 @@ Option6PDExclude::Option6PDExclude(const isc::asiolink::IOAddress& delegated_pre
                   << static_cast<int>(excluded_prefix_length_));
     }
 
-    // Excluded prefix must be longer than or equal to the delegated prefix.
+    // Excluded prefix must be longer than the delegated prefix length.
     if (excluded_prefix_length_ <= delegated_prefix_length) {
         isc_throw(BadValue, "length of the excluded prefix "
                   << excluded_prefix << "/"
@@ -126,7 +126,7 @@ Option6PDExclude::clone() const {
 }
 
 void
-Option6PDExclude::pack(isc::util::OutputBuffer& buf) const {
+Option6PDExclude::pack(isc::util::OutputBuffer& buf, bool) const {
     // Make sure that the subnet identifier is valid. It should never
     // be empty.
     if ((excluded_prefix_length_ == 0) || subnet_id_.empty()) {
