@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -70,7 +70,8 @@ OptionVendorClass::unpack(OptionBufferConstIter begin,
     size_t offset = 0;
     while (offset < std::distance(begin, end)) {
         // Parse a tuple.
-        OpaqueDataTuple tuple(getLengthFieldType(), begin + offset, end);
+        OpaqueDataTuple tuple(OptionDataTypeUtil::getTupleLenFieldType(getUniverse()),
+                              begin + offset, end);
         addTuple(tuple);
         // The tuple has been parsed correctly which implies that it is safe to
         // advance the offset by its total length.
@@ -79,6 +80,16 @@ OptionVendorClass::unpack(OptionBufferConstIter begin,
         // tuple. Let's read it, unless we have already reached the end of
         // buffer.
         if ((getUniverse() == V4) && (begin + offset != end)) {
+            // Get the other enterprise id.
+            uint32_t other_id = isc::util::readUint32(&(*(begin + offset)),
+                                                      distance(begin + offset,
+                                                               end));
+            // Throw if there are two different enterprise ids.
+            if (other_id != vendor_id_) {
+                isc_throw(isc::BadValue, "V-I Vendor Class option with two "
+                          "different enterprise ids: " << vendor_id_
+                          << " and " << other_id);
+            }
             // Advance the offset by the size of enterprise id.
             offset += sizeof(vendor_id_);
             // If the offset already ran over the buffer length or there is
@@ -95,7 +106,7 @@ OptionVendorClass::unpack(OptionBufferConstIter begin,
 
 void
 OptionVendorClass::addTuple(const OpaqueDataTuple& tuple) {
-    if (tuple.getLengthFieldType() != getLengthFieldType()) {
+    if (tuple.getLengthFieldType() != OptionDataTypeUtil::getTupleLenFieldType(getUniverse())) {
         isc_throw(isc::BadValue, "attempted to add opaque data tuple having"
                   " invalid size of the length field "
                   << tuple.getDataFieldSize() << " to Vendor Class option");
@@ -112,7 +123,7 @@ OptionVendorClass::setTuple(const size_t at, const OpaqueDataTuple& tuple) {
                   " vendor option at position " << at << " which is out of"
                   " range");
 
-    } else if (tuple.getLengthFieldType() != getLengthFieldType()) {
+    } else if (tuple.getLengthFieldType() != OptionDataTypeUtil::getTupleLenFieldType(getUniverse())) {
         isc_throw(isc::BadValue, "attempted to set opaque data tuple having"
                   " invalid size of the length field "
                   << tuple.getDataFieldSize() << " to Vendor Class option");
