@@ -17,7 +17,7 @@ namespace dhcp {
 Pkt::Pkt(uint32_t transid, const isc::asiolink::IOAddress& local_addr,
          const isc::asiolink::IOAddress& remote_addr, uint16_t local_port,
          uint16_t remote_port)
-    : transid_(transid), iface_(""), ifindex_(-1), local_addr_(local_addr),
+    : transid_(transid), iface_(""), ifindex_(UNSET_IFINDEX), local_addr_(local_addr),
       remote_addr_(remote_addr), local_port_(local_port),
       remote_port_(remote_port), buffer_out_(0), copy_retrieved_options_(false) {
 }
@@ -25,7 +25,7 @@ Pkt::Pkt(uint32_t transid, const isc::asiolink::IOAddress& local_addr,
 Pkt::Pkt(const uint8_t* buf, uint32_t len, const isc::asiolink::IOAddress& local_addr,
          const isc::asiolink::IOAddress& remote_addr, uint16_t local_port,
          uint16_t remote_port)
-    : transid_(0), iface_(""), ifindex_(-1), local_addr_(local_addr),
+    : transid_(0), iface_(""), ifindex_(UNSET_IFINDEX), local_addr_(local_addr),
       remote_addr_(remote_addr), local_port_(local_port),
       remote_port_(remote_port), buffer_out_(0), copy_retrieved_options_(false) {
     if (len != 0) {
@@ -73,6 +73,33 @@ Pkt::getOption(const uint16_t type) {
     return (OptionPtr()); // NULL
 }
 
+OptionCollection
+Pkt::getNonCopiedOptions(const uint16_t opt_type) const {
+    std::pair<OptionCollection::const_iterator,
+              OptionCollection::const_iterator> range = options_.equal_range(opt_type);
+    return (OptionCollection(range.first, range.second));
+}
+
+OptionCollection
+Pkt::getOptions(const uint16_t opt_type) {
+    OptionCollection options_copy;
+
+    std::pair<OptionCollection::iterator,
+              OptionCollection::iterator> range = options_.equal_range(opt_type);
+    // If options should be copied on retrieval, we should now iterate over
+    // matching options, copy them and replace the original ones with new
+    // instances.
+    if (copy_retrieved_options_) {
+        for (OptionCollection::iterator opt_it = range.first;
+             opt_it != range.second; ++opt_it) {
+            OptionPtr option_copy = opt_it->second->clone();
+            opt_it->second = option_copy;
+        }
+    }
+    // Finally, return updated options. This can also be empty in some cases.
+    return (OptionCollection(range.first, range.second));
+}
+
 bool
 Pkt::delOption(uint16_t type) {
     const auto& x = options_.find(type);
@@ -85,19 +112,28 @@ Pkt::delOption(uint16_t type) {
 }
 
 bool
-Pkt::inClass(const std::string& client_class) {
+Pkt::inClass(const ClientClass& client_class) {
     return (classes_.contains(client_class));
 }
 
 void
-Pkt::addClass(const std::string& client_class, bool required) {
-    // Always have ALL first.
-    if (classes_.empty()) {
-        classes_.insert("ALL");
-    }
+Pkt::addClass(const ClientClass& client_class, bool required) {
     ClientClasses& classes = !required ? classes_ : required_classes_;
     if (!classes.contains(client_class)) {
         classes.insert(client_class);
+        static_cast<void>(subclasses_.push_back(SubClassRelation(client_class, client_class)));
+    }
+}
+
+void
+Pkt::addSubClass(const ClientClass& class_def, const ClientClass& subclass) {
+    if (!classes_.contains(class_def)) {
+        classes_.insert(class_def);
+        static_cast<void>(subclasses_.push_back(SubClassRelation(class_def, subclass)));
+    }
+    if (!classes_.contains(subclass)) {
+        classes_.insert(subclass);
+        static_cast<void>(subclasses_.push_back(SubClassRelation(subclass, subclass)));
     }
 }
 
