@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -182,7 +182,7 @@ PktFilterInet6::receive(const SocketInfo& socket_info) {
     struct in6_addr to_addr;
     memset(&to_addr, 0, sizeof(to_addr));
 
-    int ifindex = -1;
+    unsigned int ifindex = UNSET_IFINDEX;
     if (result >= 0) {
         struct in6_pktinfo* pktinfo = NULL;
 
@@ -319,8 +319,7 @@ PktFilterInet6::send(const Iface&, uint16_t sockfd, const Pkt6Ptr& pkt) {
     // may be set using CMSG_SPACE (which includes padding) or
     // using CMSG_LEN. Both forms appear to work fine on Linux, FreeBSD,
     // NetBSD, but OpenBSD appears to have a bug, discussed here:
-    // http://www.archivum.info/mailing.openbsd.bugs/2009-02/00017/
-    // kernel-6080-msg_controllen-of-IPV6_PKTINFO.html
+    // https://marc.info/?l=openbsd-bugs&m=123485913417684&w=2
     // which causes sendmsg to return EINVAL if the CMSG_LEN is
     // used to set the msg_controllen value.
     m.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
