@@ -71,9 +71,11 @@ public:
         : Pkt6(buf, len, proto) {
     }
 
-    using Pkt6::getNonCopiedOptions;
+    using Pkt::getNonCopiedOptions;
     using Pkt6::getNonCopiedRelayOption;
+    using Pkt6::getNonCopiedRelayOptions;
     using Pkt6::getNonCopiedAnyRelayOption;
+    using Pkt6::getNonCopiedAllRelayOptions;
 };
 
 typedef boost::shared_ptr<NakedPkt6> NakedPkt6Ptr;
@@ -485,9 +487,9 @@ TEST_F(Pkt6Test, addGetDelOptions) {
 }
 
 // Check that multiple options of the same type may be retrieved by using
-// Pkt6::getOptions or Pkt6::getNonCopiedOptions. In the former case, also
-// check that retrieved options are copied when Pkt6::setCopyRetrievedOptions
-// is enabled.
+// getOptions or getNonCopiedOptions. In the former case, also check
+// that retrieved options are copied when setCopyRetrievedOptions is
+// enabled.
 TEST_F(Pkt6Test, getOptions) {
     NakedPkt6 pkt(DHCPV6_SOLICIT, 1234);
     OptionPtr opt1(new Option(Option::V6, 1));
@@ -633,6 +635,14 @@ TEST_F(Pkt6Test, getName) {
             EXPECT_STREQ("LEASEQUERY", Pkt6::getName(type));
             break;
 
+        case DHCPV6_LEASEQUERY_DATA:
+            EXPECT_STREQ("LEASEQUERY_DATA", Pkt6::getName(type));
+            break;
+
+        case DHCPV6_LEASEQUERY_DONE:
+            EXPECT_STREQ("LEASEQUERY_DONE", Pkt6::getName(type));
+            break;
+
         case DHCPV6_LEASEQUERY_REPLY:
             EXPECT_STREQ("LEASEQUERY_REPLY", Pkt6::getName(type));
             break;
@@ -700,6 +710,7 @@ TEST_F(Pkt6Test, relayUnpack) {
     EXPECT_EQ(2, msg->relay_info_[0].options_.size());
 
     // There should be interface-id option
+    EXPECT_EQ(1, msg->getRelayOptions(D6O_INTERFACE_ID, 0).size());
     ASSERT_TRUE(opt = msg->getRelayOption(D6O_INTERFACE_ID, 0));
     OptionBuffer data = opt->getData();
     EXPECT_EQ(32, opt->len()); // 28 bytes of data + 4 bytes header
@@ -708,6 +719,7 @@ TEST_F(Pkt6Test, relayUnpack) {
     EXPECT_TRUE(0 == memcmp("ISAM144|299|ipv6|nt:vp:1:110", &data[0], 28));
 
     // Get the remote-id option
+    EXPECT_EQ(1, msg->getRelayOptions(D6O_REMOTE_ID, 0).size());
     ASSERT_TRUE(opt = msg->getRelayOption(D6O_REMOTE_ID, 0));
     EXPECT_EQ(22, opt->len()); // 18 bytes of data + 4 bytes header
     boost::shared_ptr<OptionCustom> custom = boost::dynamic_pointer_cast<OptionCustom>(opt);
@@ -725,6 +737,7 @@ TEST_F(Pkt6Test, relayUnpack) {
     // Part 2: Check options inserted by the second relay
 
     // Get the interface-id from the second relay
+    EXPECT_EQ(1, msg->getRelayOptions(D6O_INTERFACE_ID, 1).size());
     ASSERT_TRUE(opt = msg->getRelayOption(D6O_INTERFACE_ID, 1));
     data = opt->getData();
     EXPECT_EQ(25, opt->len()); // 21 bytes + 4 bytes header
@@ -732,6 +745,7 @@ TEST_F(Pkt6Test, relayUnpack) {
     EXPECT_TRUE(0 == memcmp("ISAM144 eth 1/1/05/01", &data[0], 21));
 
     // Get the remote-id option
+    EXPECT_EQ(1, msg->getRelayOptions(D6O_REMOTE_ID, 1).size());
     ASSERT_TRUE(opt = msg->getRelayOption(D6O_REMOTE_ID, 1));
     EXPECT_EQ(8, opt->len());
     custom = boost::dynamic_pointer_cast<OptionCustom>(opt);
@@ -742,6 +756,7 @@ TEST_F(Pkt6Test, relayUnpack) {
 
     // Let's check if there is no leak between options stored in
     // the SOLICIT message and the relay.
+    EXPECT_TRUE(msg->getRelayOptions(D6O_IA_NA, 1).empty());
     EXPECT_FALSE(opt = msg->getRelayOption(D6O_IA_NA, 1));
 
 
@@ -853,6 +868,7 @@ TEST_F(Pkt6Test, relayPack) {
 
     // There should be exactly one option
     EXPECT_EQ(1, clone->relay_info_[0].options_.size());
+    EXPECT_EQ(1, clone->getRelayOptions(200, 0).size());
     OptionPtr opt = clone->getRelayOption(200, 0);
     EXPECT_TRUE(opt);
     EXPECT_EQ(opt->getType() , optRelay1->getType());
@@ -896,6 +912,50 @@ TEST_F(Pkt6Test, getRelayOption) {
     EXPECT_TRUE(opt_iface_id == opt_iface_id_returned);
 }
 
+TEST_F(Pkt6Test, getRelayOptions) {
+    NakedPkt6Ptr msg(boost::dynamic_pointer_cast<NakedPkt6>(capture2()));
+    ASSERT_TRUE(msg);
+
+    ASSERT_NO_THROW(msg->unpack());
+    ASSERT_EQ(2, msg->relay_info_.size());
+
+    OptionCollection opts_iface_id =
+        msg->getNonCopiedRelayOptions(D6O_INTERFACE_ID, 0);
+    ASSERT_EQ(1, opts_iface_id.size());
+
+    OptionPtr opt_iface_id = msg->getNonCopiedRelayOption(D6O_INTERFACE_ID, 0);
+    ASSERT_TRUE(opt_iface_id);
+
+    OptionCollection opts_iface_id_returned =
+        msg->getRelayOptions(D6O_INTERFACE_ID, 0);
+    ASSERT_EQ(1, opts_iface_id_returned.size());
+
+    OptionPtr opt_iface_id_returned = msg->getRelayOption(D6O_INTERFACE_ID, 0);
+    ASSERT_TRUE(opt_iface_id_returned);
+
+    EXPECT_TRUE(opt_iface_id == opt_iface_id_returned);
+    EXPECT_TRUE(opts_iface_id == opts_iface_id_returned);
+    EXPECT_TRUE(opts_iface_id.begin()->second == opt_iface_id);
+    EXPECT_TRUE(opts_iface_id_returned.begin()->second == opt_iface_id_returned);
+
+    msg->setCopyRetrievedOptions(true);
+
+    opts_iface_id_returned = msg->getRelayOptions(D6O_INTERFACE_ID, 0);
+    ASSERT_EQ(1, opts_iface_id_returned.size());
+    opt_iface_id_returned = msg->getRelayOption(D6O_INTERFACE_ID, 0);
+    EXPECT_FALSE(opt_iface_id == opt_iface_id_returned);
+    EXPECT_FALSE(opts_iface_id.begin()->second == opt_iface_id_returned);
+    EXPECT_FALSE(opts_iface_id_returned.begin()->second == opt_iface_id);
+    EXPECT_FALSE(opts_iface_id_returned.begin()->second == opt_iface_id_returned);
+
+    opt_iface_id = msg->getNonCopiedRelayOption(D6O_INTERFACE_ID, 0);
+    EXPECT_TRUE(opt_iface_id == opt_iface_id_returned);
+
+    opts_iface_id_returned = msg->getNonCopiedRelayOptions(D6O_INTERFACE_ID, 0);
+    opts_iface_id = msg->getNonCopiedRelayOptions(D6O_INTERFACE_ID, 0);
+    EXPECT_TRUE(opts_iface_id == opts_iface_id_returned);
+}
+
 // This test verifies that options added by relays to the message can be
 // accessed and retrieved properly
 TEST_F(Pkt6Test, getAnyRelayOption) {
@@ -950,6 +1010,10 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_FALSE(opt);
     opt = msg->getAnyRelayOption(300, Pkt6::RELAY_GET_LAST);
     EXPECT_FALSE(opt);
+    EXPECT_TRUE(msg->getAllRelayOptions(300, Pkt6::RELAY_SEARCH_FROM_CLIENT).empty());
+    EXPECT_TRUE(msg->getAllRelayOptions(300, Pkt6::RELAY_SEARCH_FROM_SERVER).empty());
+    EXPECT_TRUE(msg->getAllRelayOptions(300, Pkt6::RELAY_GET_FIRST).empty());
+    EXPECT_TRUE(msg->getAllRelayOptions(300, Pkt6::RELAY_GET_LAST).empty());
 
     // Option 200 is added in every relay.
 
@@ -960,30 +1024,81 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_TRUE(opt->equals(relay3_opt1));
     EXPECT_TRUE(opt == relay3_opt1);
 
-    // We want to ge that one inserted by relay1 (first match, starting from
+    // Check collections.
+    OptionCollection opts0 =
+        msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    EXPECT_EQ(3, opts0.size());
+    vector<OptionPtr> lopts0;
+    for (auto it : opts0) {
+        lopts0.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts0.size());
+    EXPECT_TRUE(lopts0[0] == opt);
+    EXPECT_TRUE(lopts0[0] == relay3_opt1);
+    EXPECT_TRUE(lopts0[1] == relay2_opt4);
+    EXPECT_TRUE(lopts0[2] == relay1_opt1);
+    OptionCollection opts =
+        msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    EXPECT_TRUE(opts == opts0);
+
+    // We want to get that one inserted by relay1 (first match, starting from
     // closest to the server.
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay1_opt1));
     EXPECT_TRUE(opt == relay1_opt1);
 
+    // Check collections.
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_EQ(3, opts.size());
+    vector<OptionPtr> lopts;
+    for (auto it : opts) {
+        lopts.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts.size());
+    EXPECT_TRUE(lopts[0] == opt);
+    EXPECT_TRUE(lopts[0] == relay1_opt1);
+    EXPECT_TRUE(lopts[1] == relay2_opt4);
+    EXPECT_TRUE(lopts[2] == relay3_opt1);
+    EXPECT_TRUE(opts == msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER));
+
+    // Check reverse order.
+    vector<OptionPtr> ropts;
+    for (auto it = opts.rbegin(); it != opts.rend(); ++it) {
+        ropts.push_back(it->second);
+    }
+    EXPECT_TRUE(lopts0 == ropts);
+
     // We just want option from the first relay (closest to the client)
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_GET_FIRST);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay3_opt1));
     EXPECT_TRUE(opt == relay3_opt1);
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_GET_FIRST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opt == opts.begin()->second);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_GET_FIRST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opts.begin()->second == relay3_opt1);
 
     // We just want option from the last relay (closest to the server)
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_GET_LAST);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay1_opt1));
     EXPECT_TRUE(opt == relay1_opt1);
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_GET_LAST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opt == opts.begin()->second);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_GET_LAST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opts.begin()->second == relay1_opt1);
 
     // Enable copying options when they are retrieved and redo the tests
     // but expect that options are still equal but different pointers
     // are returned.
     msg->setCopyRetrievedOptions(true);
 
+    // From client.
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay3_opt1));
@@ -995,6 +1110,35 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     ASSERT_TRUE(relay3_opt1);
     EXPECT_TRUE(opt == relay3_opt1);
 
+    // Check collections.
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    lopts0.clear();
+    for (auto it : opts) {
+        lopts0.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts0.size());
+    EXPECT_TRUE(lopts0[0] == opt);
+    EXPECT_TRUE(lopts0[0] == relay3_opt1);
+    EXPECT_TRUE(lopts0[1] == relay2_opt4);
+    EXPECT_TRUE(lopts0[2] == relay1_opt1);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    lopts.clear();
+    for (auto it : opts) {
+        lopts.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts.size());
+    EXPECT_TRUE(relay3_opt1->equals(lopts[0]));
+    EXPECT_FALSE(lopts[0] == lopts0[0]);
+    EXPECT_TRUE(relay2_opt4->equals(lopts[1]));
+    EXPECT_FALSE(lopts[1] == lopts0[1]);
+    EXPECT_TRUE(relay1_opt1->equals(lopts[2]));
+    EXPECT_FALSE(lopts[2] == lopts0[2]);
+    // Get current values for next tests.
+    relay3_opt1 = lopts[0];
+    relay2_opt4 = lopts[1];
+    relay1_opt1 = lopts[2];
+
+    // From server.
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay1_opt1));
@@ -1003,6 +1147,31 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     ASSERT_TRUE(relay1_opt1);
     EXPECT_TRUE(opt == relay1_opt1);
 
+    // Check collections.
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    lopts0.clear();
+    for (auto it : opts) {
+        lopts0.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts0.size());
+    EXPECT_TRUE(lopts0[0] == opt);
+    EXPECT_TRUE(lopts0[0] == relay1_opt1);
+    EXPECT_TRUE(lopts0[1] == relay2_opt4);
+    EXPECT_TRUE(lopts0[2] == relay3_opt1);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    lopts.clear();
+    for (auto it : opts) {
+        lopts.push_back(it.second);
+    }
+    ASSERT_EQ(3, lopts.size());
+    EXPECT_TRUE(relay1_opt1->equals(lopts[0]));
+    EXPECT_FALSE(lopts[0] == lopts0[0]);
+    EXPECT_TRUE(relay2_opt4->equals(lopts[1]));
+    EXPECT_FALSE(lopts[1] == lopts0[1]);
+    EXPECT_TRUE(relay3_opt1->equals(lopts[2]));
+    EXPECT_FALSE(lopts[2] == lopts0[2]);
+
+    // First.
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_GET_FIRST);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay3_opt1));
@@ -1010,7 +1179,16 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     relay3_opt1 = msg->getNonCopiedAnyRelayOption(200, Pkt6::RELAY_GET_FIRST);
     ASSERT_TRUE(relay3_opt1);
     EXPECT_TRUE(opt == relay3_opt1);
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_GET_FIRST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opt == opts.begin()->second);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_GET_FIRST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_FALSE(opts.begin()->second == relay3_opt1);
+    relay3_opt1 = msg->getNonCopiedAnyRelayOption(200, Pkt6::RELAY_GET_FIRST);
+    EXPECT_TRUE(opts.begin()->second == relay3_opt1);
 
+    // Last.
     opt = msg->getAnyRelayOption(200, Pkt6::RELAY_GET_LAST);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay1_opt1));
@@ -1018,6 +1196,14 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     relay1_opt1 = msg->getNonCopiedAnyRelayOption(200, Pkt6::RELAY_GET_LAST);
     ASSERT_TRUE(relay1_opt1);
     EXPECT_TRUE(opt == relay1_opt1);
+    opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_GET_LAST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opt == opts.begin()->second);
+    opts = msg->getAllRelayOptions(200, Pkt6::RELAY_GET_LAST);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_FALSE(opts.begin()->second == relay1_opt1);
+    relay1_opt1 = msg->getNonCopiedAnyRelayOption(200, Pkt6::RELAY_GET_LAST);
+    EXPECT_TRUE(opts.begin()->second == relay1_opt1);
 
     // Disable copying options and continue with other tests.
     msg->setCopyRetrievedOptions(false);
@@ -1027,29 +1213,65 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     opt = msg->getAnyRelayOption(100, Pkt6::RELAY_SEARCH_FROM_SERVER);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay2_opt1));
+    opts = msg->getNonCopiedAllRelayOptions(100, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opts.begin()->second == relay2_opt1);
+    opts = msg->getAllRelayOptions(100, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(relay2_opt1->equals(opts.begin()->second));
 
     opt = msg->getAnyRelayOption(100, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     ASSERT_TRUE(opt);
     EXPECT_TRUE(opt->equals(relay2_opt1));
+    opts = msg->getNonCopiedAllRelayOptions(100, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(opts.begin()->second == relay2_opt1);
+    opts = msg->getAllRelayOptions(100, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_EQ(1, opts.size());
+    EXPECT_TRUE(relay2_opt1->equals(opts.begin()->second));
 
     opt = msg->getAnyRelayOption(100, Pkt6::RELAY_GET_FIRST);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(100, Pkt6::RELAY_GET_FIRST);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(100, Pkt6::RELAY_GET_FIRST);
+    EXPECT_TRUE(opts.empty());
 
     opt = msg->getAnyRelayOption(100, Pkt6::RELAY_GET_LAST);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(100, Pkt6::RELAY_GET_LAST);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(100, Pkt6::RELAY_GET_LAST);
+    EXPECT_TRUE(opts.empty());
 
     // Finally, try to get an option that does not exist
     opt = msg->getAnyRelayOption(500, Pkt6::RELAY_GET_FIRST);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(500, Pkt6::RELAY_GET_FIRST);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(500, Pkt6::RELAY_GET_FIRST);
+    EXPECT_TRUE(opts.empty());
 
     opt = msg->getAnyRelayOption(500, Pkt6::RELAY_GET_LAST);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(500, Pkt6::RELAY_GET_LAST);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(500, Pkt6::RELAY_GET_LAST);
+    EXPECT_TRUE(opts.empty());
 
     opt = msg->getAnyRelayOption(500, Pkt6::RELAY_SEARCH_FROM_SERVER);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(500, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(500, Pkt6::RELAY_SEARCH_FROM_SERVER);
+    EXPECT_TRUE(opts.empty());
 
     opt = msg->getAnyRelayOption(500, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     EXPECT_FALSE(opt);
+    opts = msg->getNonCopiedAllRelayOptions(500, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    EXPECT_TRUE(opts.empty());
+    opts = msg->getAllRelayOptions(500, Pkt6::RELAY_SEARCH_FROM_CLIENT);
+    EXPECT_TRUE(opts.empty());
 }
 
 // Tests whether Pkt6::toText() properly prints out all parameters, including
@@ -1142,6 +1364,39 @@ TEST_F(Pkt6Test, deferredClientClasses) {
     EXPECT_TRUE(pkt.getClasses(true).contains("foo"));
 }
 
+// Tests whether a packet can be assigned to a subclass and later
+// checked if it belongs to a given subclass
+TEST_F(Pkt6Test, templateClasses) {
+    Pkt6 pkt(DHCPV6_ADVERTISE, 1234);
+
+    // Default values (do not belong to any subclass)
+    EXPECT_FALSE(pkt.inClass("SPAWN_template-interface-name_eth0"));
+    EXPECT_FALSE(pkt.inClass("SPAWN_template-interface-id_interface-id0"));
+    EXPECT_TRUE(pkt.getClasses().empty());
+
+    // Add to the first subclass
+    pkt.addSubClass("template-interface-name", "SPAWN_template-interface-name_eth0");
+    EXPECT_TRUE(pkt.inClass("SPAWN_template-interface-name_eth0"));
+    EXPECT_FALSE(pkt.inClass("SPAWN_template-interface-id_interface-id0"));
+    ASSERT_FALSE(pkt.getClasses().empty());
+
+    // Add to a second subclass
+    pkt.addSubClass("template-interface-id", "SPAWN_template-interface-id_interface-id0");
+    EXPECT_TRUE(pkt.inClass("SPAWN_template-interface-name_eth0"));
+    EXPECT_TRUE(pkt.inClass("SPAWN_template-interface-id_interface-id0"));
+
+    // Check that it's ok to add to the same subclass repeatedly
+    EXPECT_NO_THROW(pkt.addSubClass("template-foo", "SPAWN_template-foo_bar"));
+    EXPECT_NO_THROW(pkt.addSubClass("template-foo", "SPAWN_template-foo_bar"));
+    EXPECT_NO_THROW(pkt.addSubClass("template-bar", "SPAWN_template-bar_bar"));
+
+    // Check that the packet belongs to 'SPAWN_template-foo_bar'
+    EXPECT_TRUE(pkt.inClass("SPAWN_template-foo_bar"));
+
+    // Check that the packet belongs to 'SPAWN_template-bar_bar'
+    EXPECT_TRUE(pkt.inClass("SPAWN_template-bar_bar"));
+}
+
 // Tests whether MAC can be obtained and that MAC sources are not
 // confused.
 TEST_F(Pkt6Test, getMAC) {
