@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -19,6 +19,7 @@
 
 using namespace isc::asiolink;
 using namespace isc::data;
+using namespace std;
 
 namespace isc {
 namespace dhcp {
@@ -200,6 +201,58 @@ CfgHosts::getPage6(const SubnetID& subnet_id,
     return (collection);
 }
 
+ConstHostCollection
+CfgHosts::getPage4(size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) const {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal method.
+    ConstHostCollection collection;
+    getPageInternal<ConstHostCollection>(lower_host_id,
+                                         page_size,
+                                         collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getPage4(size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal method.
+    HostCollection collection;
+    getPageInternal<HostCollection>(lower_host_id,
+                                    page_size,
+                                    collection);
+    return (collection);
+}
+
+ConstHostCollection
+CfgHosts::getPage6(size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) const {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal method.
+    ConstHostCollection collection;
+    getPageInternal<ConstHostCollection>(lower_host_id,
+                                         page_size,
+                                         collection);
+    return (collection);
+}
+
+HostCollection
+CfgHosts::getPage6(size_t& /*source_index*/,
+                   uint64_t lower_host_id,
+                   const HostPageSize& page_size) {
+    // Do not issue logging message here because it will be logged by
+    // the getPageInternal method.
+    HostCollection collection;
+    getPageInternal<HostCollection>(lower_host_id,
+                                    page_size,
+                                    collection);
+    return (collection);
+}
+
 ConstHostCollection
 CfgHosts::getAll4(const IOAddress& address) const {
     // Do not issue logging message here because it will be logged by
@@ -433,6 +486,40 @@ CfgHosts::getAllbyHostnameInternal6(const std::string& hostname,
         .arg(storage.size());
 }
 
+template<typename Storage>
+void
+CfgHosts::getPageInternal(uint64_t lower_host_id,
+                          const HostPageSize& page_size,
+                          Storage& storage) const {
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL);
+
+    // Use the host id last index.
+    const HostContainerIndex4& idx = hosts_.get<4>();
+    HostContainerIndex4::const_iterator host = idx.lower_bound(lower_host_id);
+
+    // Exclude the lower bound id when it is not zero.
+    if (lower_host_id &&
+        (host != idx.end()) && ((*host)->getHostId() == lower_host_id)) {
+        ++host;
+    }
+
+    // Return hosts within the page size.
+    for (; host != idx.end(); ++host) {
+        LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                  HOSTS_CFG_GET_ALL_HOST)
+            .arg((*host)->toText());
+        storage.push_back(*host);
+        if (storage.size() >= page_size.page_size_) {
+            break;
+        }
+    }
+
+    // Log how many hosts have been found.
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_COUNT)
+        .arg(storage.size());
+}
+
 template<typename Storage>
 void
 CfgHosts::getPageInternal4(const SubnetID& subnet_id,
@@ -557,16 +644,16 @@ CfgHosts::getAllInternal6(const IOAddress& address, Storage& storage) const {
                   " for a host, specified address was " << address);
     }
     // Search for the Host using the reserved IPv6 address as a key.
-    const HostContainerIndex1& idx = hosts_.get<1>();
-    HostContainerIndex1Range r = idx.equal_range(address);
+    const HostContainer6Index4& idx = hosts6_.get<4>();
+    HostContainer6Index4Range r = idx.equal_range(address);
     // Append each Host object to the storage.
-    for (HostContainerIndex1::iterator host = r.first; host != r.second;
-         ++host) {
+    for (HostContainer6Index4::iterator reservation = r.first; reservation != r.second;
+         ++reservation) {
         LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
                   HOSTS_CFG_GET_ALL_ADDRESS6_HOST)
             .arg(address.toText())
-            .arg((*host)->toText());
-        storage.push_back(*host);
+            .arg(reservation->host_->toText());
+        storage.push_back(reservation->host_);
     }
 
     LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_ADDRESS6_COUNT)
@@ -615,6 +702,30 @@ CfgHosts::get4(const SubnetID& subnet_id, const IOAddress& address) const {
     return (ConstHostPtr());
 }
 
+ConstHostCollection
+CfgHosts::getAll4(const SubnetID& subnet_id,
+                  const asiolink::IOAddress& address) const {
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_GET_ALL_SUBNET_ID_ADDRESS4)
+        .arg(subnet_id).arg(address.toText());
+
+    ConstHostCollection hosts;
+    for (auto host : getAll4(address)) {
+        if (host->getIPv4SubnetID() == subnet_id) {
+            LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE_DETAIL_DATA,
+                      HOSTS_CFG_GET_ALL_SUBNET_ID_ADDRESS4_HOST)
+                .arg(subnet_id)
+                .arg(address.toText())
+                .arg(host->toText());
+            hosts.push_back(host);
+        }
+    }
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_RESULTS, HOSTS_CFG_GET_ALL_SUBNET_ID_ADDRESS4_COUNT)
+        .arg(subnet_id)
+        .arg(address.toText())
+        .arg(hosts.size());
+
+    return (hosts);
+}
 
 ConstHostPtr
 CfgHosts::get6(const SubnetID& subnet_id,
@@ -658,6 +769,14 @@ CfgHosts::get6(const SubnetID& subnet_id,
     return (getHostInternal6<HostPtr, HostCollection>(subnet_id, address));
 }
 
+ConstHostCollection
+CfgHosts::getAll6(const SubnetID& subnet_id,
+                  const asiolink::IOAddress& address) const {
+    ConstHostCollection hosts;
+    getAllInternal6(subnet_id, address, hosts);
+    return (hosts);
+}
+
 template<typename ReturnType, typename Storage>
 ReturnType
 CfgHosts::getHostInternal6(const SubnetID& subnet_id,
@@ -841,7 +960,7 @@ CfgHosts::add(const HostPtr& host) {
                   " is added to the configuration");
     }
 
-    // At least one subnet ID must be used
+    // At least one subnet ID must be used.
     if (host->getIPv4SubnetID() == SUBNET_ID_UNUSED &&
         host->getIPv6SubnetID() == SUBNET_ID_UNUSED) {
         isc_throw(BadValue, "must not use both IPv4 and IPv6 subnet ids of"
@@ -859,31 +978,6 @@ CfgHosts::add4(const HostPtr& host) {
     HWAddrPtr hwaddr = host->getHWAddress();
     DuidPtr duid = host->getDuid();
 
-    // There should be at least one resource reserved: hostname, IPv4
-    // address, siaddr, sname, file or IPv6 address or prefix.
-    /// @todo: this check should be done in add(), not in add4()
-    if (host->getHostname().empty() &&
-        (host->getIPv4Reservation().isV4Zero()) &&
-        !host->hasIPv6Reservation() &&
-        host->getNextServer().isV4Zero() &&
-        host->getServerHostname().empty() &&
-        host->getBootFileName().empty() &&
-        host->getCfgOption4()->empty() &&
-        host->getCfgOption6()->empty() &&
-        host->getClientClasses4().empty() &&
-        host->getClientClasses6().empty()) {
-        std::ostringstream s;
-        if (hwaddr) {
-            s << "for DUID: " << hwaddr->toText();
-        } else if (duid) {
-            s << "for HW address: " << duid->toText();
-        }
-        isc_throw(BadValue, "specified reservation " << s.str()
-                  << " must include at least one resource, i.e. "
-                  "hostname, IPv4 address, IPv6 address/prefix, "
-                  "options");
-    }
-
     // Check for duplicates for the specified IPv4 subnet.
     if (host->getIPv4SubnetID() != SUBNET_ID_UNUSED) {
         if (hwaddr && !hwaddr->hwaddr_.empty() &&
@@ -923,7 +1017,7 @@ CfgHosts::add4(const HostPtr& host) {
     }
 
     // Check if the address is already reserved for the specified IPv4 subnet.
-    if (!host->getIPv4Reservation().isV4Zero() &&
+    if (ip_reservations_unique_ && !host->getIPv4Reservation().isV4Zero() &&
         (host->getIPv4SubnetID() != SUBNET_ID_UNUSED) &&
         get4(host->getIPv4SubnetID(), host->getIPv4Reservation())) {
         isc_throw(ReservedAddress, "failed to add new host using the HW"
@@ -975,25 +1069,53 @@ CfgHosts::add6(const HostPtr& host) {
     for (IPv6ResrvIterator it = reservations.first; it != reservations.second;
          ++it) {
 
-        // If there's an entry for this (subnet-id, address), reject it.
-        if (get6(host->getIPv6SubnetID(), it->second.getPrefix())) {
-            isc_throw(DuplicateHost, "failed to add address reservation for "
-                      << "host using the HW address '"
-                      << (hwaddr ? hwaddr->toText(false) : "(null)")
-                      << " and DUID '" << (duid ? duid->toText() : "(null)")
-                      << "' to the IPv6 subnet id '" << host->getIPv6SubnetID()
-                      << "' for address/prefix " << it->second.getPrefix()
-                      << ": There's already reservation for this address/prefix");
+        if (ip_reservations_unique_) {
+            // If there's an entry for this (subnet-id, address), reject it.
+            if (get6(host->getIPv6SubnetID(), it->second.getPrefix())) {
+                isc_throw(DuplicateHost, "failed to add address reservation for "
+                          << "host using the HW address '"
+                          << (hwaddr ? hwaddr->toText(false) : "(null)")
+                          << " and DUID '" << (duid ? duid->toText() : "(null)")
+                          << "' to the IPv6 subnet id '" << host->getIPv6SubnetID()
+                          << "' for address/prefix " << it->second.getPrefix()
+                          << ": There's already reservation for this address/prefix");
+            }
         }
         hosts6_.insert(HostResrv6Tuple(it->second, host));
     }
 }
 
 bool
-CfgHosts::del(const SubnetID& /*subnet_id*/, const asiolink::IOAddress& /*addr*/) {
-    /// @todo: Implement host removal
-    isc_throw(NotImplemented, "sorry, not implemented");
-    return (false);
+CfgHosts::del(const SubnetID& subnet_id, const asiolink::IOAddress& addr) {
+    size_t erased_hosts = 0;
+    size_t erased_addresses = 0;
+    if (addr.isV4()) {
+        HostContainerIndex4& idx = hosts_.get<4>();
+        // Delete IPv4 reservation and host.
+        for (auto host : getAll4(subnet_id, addr)) {
+            erased_hosts += idx.erase(host->getHostId());
+        }
+        erased_addresses = erased_hosts;
+    } else {
+        HostContainer6Index1& idx6 = hosts6_.get<1>();
+        HostContainerIndex4& idx = hosts_.get<4>();
+        // Delete IPv6 reservations.
+        const auto& range = idx6.equal_range(boost::make_tuple(subnet_id, addr));
+        erased_addresses = boost::distance(range);
+        // Delete hosts.
+        for (auto key = range.first; key != range.second; ++key) {
+            erased_hosts += idx.erase(key->host_->getHostId());
+        }
+        idx6.erase(range.first, range.second);
+    }
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_DEL)
+        .arg(erased_hosts)
+        .arg(erased_addresses)
+        .arg(subnet_id)
+        .arg(addr.toText());
+
+    return (erased_hosts != 0);
 }
 
 size_t
@@ -1009,13 +1131,33 @@ CfgHosts::delAll4(const SubnetID& subnet_id) {
 }
 
 bool
-CfgHosts::del4(const SubnetID& /*subnet_id*/,
-               const Host::IdentifierType& /*identifier_type*/,
-               const uint8_t* /*identifier_begin*/,
-               const size_t /*identifier_len*/) {
-    /// @todo: Implement host removal
-    isc_throw(NotImplemented, "sorry, not implemented");
-    return (false);
+CfgHosts::del4(const SubnetID& subnet_id,
+               const Host::IdentifierType& identifier_type,
+               const uint8_t* identifier_begin,
+               const size_t identifier_len) {
+    HostContainerIndex0& idx = hosts_.get<0>();
+    const auto t = boost::make_tuple(std::vector<uint8_t>(identifier_begin,
+                                                    identifier_begin + identifier_len),
+                                                    identifier_type);
+    const auto& range = idx.equal_range(t);
+    size_t erased = 0;
+    for (auto key = range.first; key != range.second;) {
+        if ((*key)->getIPv4SubnetID() != subnet_id) {
+            ++key;
+            // Skip hosts from other subnets.
+            continue;
+        }
+
+        key = idx.erase(key);
+        erased++;
+    }
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_DEL4)
+        .arg(erased)
+        .arg(subnet_id)
+        .arg(Host::getIdentifierAsText(identifier_type, identifier_begin, identifier_len));
+
+    return (erased != 0);
 }
 
 size_t
@@ -1037,15 +1179,50 @@ CfgHosts::delAll6(const SubnetID& subnet_id) {
 }
 
 bool
-CfgHosts::del6(const SubnetID& /*subnet_id*/,
-               const Host::IdentifierType& /*identifier_type*/,
-               const uint8_t* /*identifier_begin*/,
-               const size_t /*identifier_len*/) {
-    /// @todo: Implement host removal
-    isc_throw(NotImplemented, "sorry, not implemented");
-    return (false);
+CfgHosts::del6(const SubnetID& subnet_id,
+               const Host::IdentifierType& identifier_type,
+               const uint8_t* identifier_begin,
+               const size_t identifier_len) {
+    HostContainerIndex0& idx = hosts_.get<0>();
+    HostContainer6Index3& idx6 = hosts6_.get<3>();
+
+    const auto t = boost::make_tuple(std::vector<uint8_t>(identifier_begin,
+                                                          identifier_begin + identifier_len),
+                                                          identifier_type);
+    const auto& range = idx.equal_range(t);
+    size_t erased_hosts = 0;
+    size_t erased_reservations = 0;
+    for (auto key = range.first; key != range.second;) {
+        if ((*key)->getIPv6SubnetID() != subnet_id) {
+            ++key;
+            // Skip hosts from other subnets.
+            continue;
+        }
+
+        // Delete host.
+        auto host_id = (*key)->getHostId();
+        key = idx.erase(key);
+        erased_hosts++;
+        // Delete reservations.
+        erased_reservations += idx6.erase(host_id);
+    }
+
+    LOG_DEBUG(hosts_logger, HOSTS_DBG_TRACE, HOSTS_CFG_DEL6)
+        .arg(erased_hosts)
+        .arg(erased_reservations)
+        .arg(subnet_id)
+        .arg(Host::getIdentifierAsText(identifier_type, identifier_begin, identifier_len));
+
+    return (erased_hosts != 0);
 }
 
+bool
+CfgHosts::setIPReservationsUnique(const bool unique) {
+    ip_reservations_unique_ = unique;
+    return (true);
+}
+
+
 ElementPtr
 CfgHosts::toElement() const {
     uint16_t family = CfgMgr::instance().getFamily();
