@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -24,16 +24,17 @@ namespace isc {
 namespace dhcp {
 
 OptionDescriptorPtr
-OptionDescriptor::create(const OptionPtr& opt, bool persist,
+OptionDescriptor::create(const OptionPtr& opt, bool persist, bool cancel,
                          const std::string& formatted_value,
                          ConstElementPtr user_context) {
-    return (boost::make_shared<OptionDescriptor>(opt, persist, formatted_value,
+    return (boost::make_shared<OptionDescriptor>(opt, persist, cancel,
+                                                 formatted_value,
                                                  user_context));
 }
 
 OptionDescriptorPtr
-OptionDescriptor::create(bool persist) {
-    return (boost::make_shared<OptionDescriptor>(persist));
+OptionDescriptor::create(bool persist, bool cancel) {
+    return (boost::make_shared<OptionDescriptor>(persist, cancel));
 }
 
 OptionDescriptorPtr
@@ -44,12 +45,14 @@ OptionDescriptor::create(const OptionDescriptor& desc) {
 bool
 OptionDescriptor::equals(const OptionDescriptor& other) const {
     return ((persistent_ == other.persistent_) &&
+            (cancelled_ == other.cancelled_) &&
             (formatted_value_ == other.formatted_value_) &&
             (space_name_ == other.space_name_) &&
             option_->equals(other.option_));
 }
 
-CfgOption::CfgOption() {
+CfgOption::CfgOption()
+    : encapsulated_(false) {
 }
 
 bool
@@ -64,10 +67,12 @@ CfgOption::equals(const CfgOption& other) const {
 }
 
 void
-CfgOption::add(const OptionPtr& option, const bool persistent,
+CfgOption::add(const OptionPtr& option,
+               const bool persistent,
+               const bool cancelled,
                const std::string& option_space,
                const uint64_t id) {
-    OptionDescriptor desc(option, persistent);
+    OptionDescriptor desc(option, persistent, cancelled);
     if (id > 0) {
         desc.setId(id);
     }
@@ -96,7 +101,7 @@ void
 CfgOption::replace(const OptionDescriptor& desc, const std::string& option_space) {
     if (!desc.option_) {
         isc_throw(isc::BadValue, "option being replaced must not be NULL");
-    } 
+    }
 
     // Check for presence of options.
     OptionContainerPtr options = getAll(option_space);
@@ -107,27 +112,25 @@ CfgOption::replace(const OptionDescriptor& desc, const std::string& option_space
 
     // Find the option we want to replace.
     OptionContainerTypeIndex& idx = options->get<1>();
-    OptionContainerTypeIndex::const_iterator od_itr = idx.find(desc.option_->getType());
+    auto const& od_itr = idx.find(desc.option_->getType());
     if (od_itr == idx.end()) {
-        isc_throw(isc::BadValue, "cannot replace option: " 
+        isc_throw(isc::BadValue, "cannot replace option: "
                   << option_space << ":" << desc.option_->getType()
                   << ", it does not exist");
-    } 
+    }
 
     idx.replace(od_itr, desc);
 }
 
-
 std::list<std::string>
 CfgOption::getVendorIdsSpaceNames() const {
     std::list<uint32_t> ids = getVendorIds();
     std::list<std::string> names;
-    for (std::list<uint32_t>::const_iterator id = ids.begin();
-         id != ids.end(); ++id) {
+    for (auto const& id : ids) {
         std::ostringstream s;
         // Vendor space name is constructed as "vendor-XYZ" where XYZ is an
         // uint32_t value, without leading zeros.
-        s << "vendor-" << *id;
+        s << "vendor-" << id;
         names.push_back(s.str());
     }
     return (names);
@@ -146,6 +149,12 @@ CfgOption::merge(CfgOptionDefPtr cfg_def,  CfgOption& other) {
 
     // Next we copy "other" on top of ourself.
     other.copyTo(*this);
+
+    // If we copied new options we may need to populate the
+    // sub-options into the upper level options. The server
+    // expects that the top-level options have suitable
+    // suboptions appended.
+    encapsulate();
 }
 
 void
@@ -156,8 +165,8 @@ CfgOption::createOptions(CfgOptionDefPtr cfg_def) {
     for (auto space : getOptionSpaceNames()) {
         for (auto opt_desc : *(getAll(space))) {
             if (createDescriptorOption(cfg_def, space, opt_desc)) {
-                // Option was recreated, let's replace the descriptor. 
-                replace(opt_desc,space);
+                // Option was recreated, let's replace the descriptor.
+                replace(opt_desc, space);
             }
         }
     }
@@ -193,6 +202,11 @@ CfgOption::createDescriptorOption(CfgOptionDefPtr cfg_def, const std::string& sp
         def = cfg_def->get(space, code);
     }
 
+    // Finish with a last resort option definition.
+    if (!def) {
+        def = LibDHCP::getLastResortOptionDef(space, code);
+    }
+
     std::string& formatted_value = opt_desc.formatted_value_;
     if (!def) {
         if (!formatted_value.empty()) {
@@ -250,6 +264,7 @@ CfgOption::encapsulate() {
     encapsulateInternal(DHCP4_OPTION_SPACE);
     // Append sub-options to the top level "dhcp6" option space.
     encapsulateInternal(DHCP6_OPTION_SPACE);
+    encapsulated_ = true;
 }
 
 void
@@ -258,9 +273,8 @@ CfgOption::encapsulateInternal(const std::string& option_space) {
     OptionContainerPtr options = getAll(option_space);
     // For each option in the option space we will append sub-options
     // from the option spaces they encapsulate.
-    for (OptionContainer::const_iterator opt = options->begin();
-         opt != options->end(); ++opt) {
-        encapsulateInternal(opt->option_);
+    for (auto const& opt : *options) {
+        encapsulateInternal(opt.option_);
     }
 }
 
@@ -270,22 +284,22 @@ CfgOption::encapsulateInternal(const OptionPtr& option) {
     const std::string& encap_space = option->getEncapsulatedSpace();
     // Empty value means that no option space is encapsulated.
     if (!encap_space.empty()) {
+        if (encap_space == DHCP4_OPTION_SPACE || encap_space == DHCP6_OPTION_SPACE) {
+            return;
+        }
         // Retrieve all options from the encapsulated option space.
         OptionContainerPtr encap_options = getAll(encap_space);
-        for (OptionContainer::const_iterator encap_opt =
-                 encap_options->begin(); encap_opt != encap_options->end();
-             ++encap_opt) {
+        for (auto const& encap_opt : *encap_options) {
+            if (option.get() == encap_opt.option_.get()) {
+                // Avoid recursion by not adding options to themselves.
+                continue;
+            }
+
             // Add sub-option if there isn't one added already.
-            if (!option->getOption(encap_opt->option_->getType())) {
-                option->addOption(encap_opt->option_);
-            }
-            // This is a workaround for preventing infinite recursion when
-            // trying to encapsulate options created with default global option
-            // spaces.
-            if (encap_space != DHCP4_OPTION_SPACE &&
-                encap_space != DHCP6_OPTION_SPACE) {
-                encapsulateInternal(encap_opt->option_);
+            if (!option->getOption(encap_opt.option_->getType())) {
+                option->addOption(encap_opt.option_);
             }
+            encapsulateInternal(encap_opt.option_);
         }
     }
 }
@@ -302,30 +316,27 @@ CfgOption::mergeInternal(const OptionSpaceContainer<OptionContainer,
     // For each space in the source container retrieve the actual options and
     // match them with the options held in the destination container under
     // the same space.
-    for (typename std::list<Selector>::const_iterator it = selectors.begin();
-         it != selectors.end(); ++it) {
+    for (auto const& it : selectors) {
         // Get all options in the destination container for the particular
         // option space.
-        OptionContainerPtr dest_all = dest_container.getItems(*it);
-        OptionContainerPtr src_all = src_container.getItems(*it);
+        OptionContainerPtr dest_all = dest_container.getItems(it);
+        OptionContainerPtr src_all = src_container.getItems(it);
         // For each option under this option space check if there is a
         // corresponding option in the destination container. If not,
         // add one.
-        for (OptionContainer::const_iterator src_opt = src_all->begin();
-             src_opt != src_all->end(); ++src_opt) {
+        for (auto const& src_opt : *src_all) {
             const OptionContainerTypeIndex& idx = dest_all->get<1>();
             const OptionContainerTypeRange& range =
-                idx.equal_range(src_opt->option_->getType());
+                idx.equal_range(src_opt.option_->getType());
             // If there is no such option in the destination container,
             // add one.
             if (std::distance(range.first, range.second) == 0) {
-                dest_container.addItem(OptionDescriptor(*src_opt), *it);
+                dest_container.addItem(OptionDescriptor(src_opt), it);
             }
         }
     }
 }
 
-
 OptionContainerPtr
 CfgOption::getAll(const std::string& option_space) const {
     return (options_.getItems(option_space));
@@ -336,6 +347,15 @@ CfgOption::getAll(const uint32_t vendor_id) const {
     return (vendor_options_.getItems(vendor_id));
 }
 
+OptionContainerPtr
+CfgOption::getAllCombined(const std::string& option_space) const {
+    auto vendor_id = LibDHCP::optionSpaceToVendorId(option_space);
+    if (vendor_id > 0) {
+        return (getAll(vendor_id));
+    }
+    return (getAll(option_space));
+}
+
 size_t
 CfgOption::del(const std::string& option_space, const uint16_t option_code) {
     // Check for presence of options.
@@ -433,47 +453,48 @@ CfgOption::toElementWithMetadata(const bool include_metadata) const {
     ElementPtr result = Element::createList();
     // Iterate first on options using space names
     const std::list<std::string>& names = options_.getOptionSpaceNames();
-    for (std::list<std::string>::const_iterator name = names.begin();
-         name != names.end(); ++name) {
-        OptionContainerPtr opts = getAll(*name);
-        for (OptionContainer::const_iterator opt = opts->begin();
-             opt != opts->end(); ++opt) {
+    for (auto const& name : names) {
+        OptionContainerPtr opts = getAll(name);
+        for (auto const& opt : *opts) {
             // Get and fill the map for this option
             ElementPtr map = Element::createMap();
             // Set user context
-            opt->contextToElement(map);
+            opt.contextToElement(map);
             // Set space from parent iterator
-            map->set("space", Element::create(*name));
+            map->set("space", Element::create(name));
             // Set the code
-            uint16_t code = opt->option_->getType();
+            uint16_t code = opt.option_->getType();
             map->set("code", Element::create(code));
             // Set the name (always for standard options else when asked for)
-            OptionDefinitionPtr def = LibDHCP::getOptionDef(*name, code);
+            OptionDefinitionPtr def = LibDHCP::getOptionDef(name, code);
             if (!def) {
-                def = LibDHCP::getRuntimeOptionDef(*name, code);
+                def = LibDHCP::getRuntimeOptionDef(name, code);
             }
             if (!def) {
-                def = LibDHCP::getLastResortOptionDef(*name, code);
+                def = LibDHCP::getLastResortOptionDef(name, code);
             }
             if (def) {
                 map->set("name", Element::create(def->getName()));
             }
             // Set the data item
-            if (!opt->formatted_value_.empty()) {
+            if (!opt.formatted_value_.empty()) {
                 map->set("csv-format", Element::create(true));
-                map->set("data", Element::create(opt->formatted_value_));
+                map->set("data", Element::create(opt.formatted_value_));
             } else {
-                map->set("csv-format", Element::create(false));
-                std::vector<uint8_t> bin = opt->option_->toBinary();
-                std::string repr = util::encode::encodeHex(bin);
-                map->set("data", Element::create(repr));
+                std::vector<uint8_t> bin = opt.option_->toBinary();
+                if (!opt.cancelled_ || !bin.empty()) {
+                    map->set("csv-format", Element::create(false));
+                    std::string repr = util::encode::encodeHex(bin);
+                    map->set("data", Element::create(repr));
+                }
             }
             // Set the persistency flag
-            map->set("always-send", Element::create(opt->persistent_));
-
+            map->set("always-send", Element::create(opt.persistent_));
+            // Set the cancelled flag.
+            map->set("never-send", Element::create(opt.cancelled_));
             // Include metadata if requested.
             if (include_metadata) {
-                map->set("metadata", opt->getMetadata());
+                map->set("metadata", opt.getMetadata());
             }
 
             // Push on the list
@@ -482,26 +503,24 @@ CfgOption::toElementWithMetadata(const bool include_metadata) const {
     }
     // Iterate first on vendor_options using vendor ids
     const std::list<uint32_t>& ids = vendor_options_.getOptionSpaceNames();
-    for (std::list<uint32_t>::const_iterator id = ids.begin();
-         id != ids.end(); ++id) {
-        OptionContainerPtr opts = getAll(*id);
-        for (OptionContainer::const_iterator opt = opts->begin();
-             opt != opts->end(); ++opt) {
+    for (auto const& id : ids) {
+        OptionContainerPtr opts = getAll(id);
+        for (auto const& opt : *opts) {
             // Get and fill the map for this option
             ElementPtr map = Element::createMap();
             // Set user context
-            opt->contextToElement(map);
+            opt.contextToElement(map);
             // Set space from parent iterator
             std::ostringstream oss;
-            oss << "vendor-" << *id;
+            oss << "vendor-" << id;
             map->set("space", Element::create(oss.str()));
             // Set the code
-            uint16_t code = opt->option_->getType();
+            uint16_t code = opt.option_->getType();
             map->set("code", Element::create(code));
             // Set the name
-            Option::Universe universe = opt->option_->getUniverse();
+            Option::Universe universe = opt.option_->getUniverse();
             OptionDefinitionPtr def =
-                LibDHCP::getVendorOptionDef(universe, *id, code);
+                LibDHCP::getVendorOptionDef(universe, id, code);
             if (!def) {
                 // vendor-XXX space is in oss
                 def = LibDHCP::getRuntimeOptionDef(oss.str(), code);
@@ -510,17 +529,21 @@ CfgOption::toElementWithMetadata(const bool include_metadata) const {
                 map->set("name", Element::create(def->getName()));
             }
             // Set the data item
-            if (!opt->formatted_value_.empty()) {
+            if (!opt.formatted_value_.empty()) {
                 map->set("csv-format", Element::create(true));
-                map->set("data", Element::create(opt->formatted_value_));
+                map->set("data", Element::create(opt.formatted_value_));
             } else {
-                map->set("csv-format", Element::create(false));
-                std::vector<uint8_t> bin = opt->option_->toBinary();
-                std::string repr = util::encode::encodeHex(bin);
-                map->set("data", Element::create(repr));
+                std::vector<uint8_t> bin = opt.option_->toBinary();
+                if (!opt.cancelled_ || !bin.empty()) {
+                    map->set("csv-format", Element::create(false));
+                    std::string repr = util::encode::encodeHex(bin);
+                    map->set("data", Element::create(repr));
+                }
             }
             // Set the persistency flag
-            map->set("always-send", Element::create(opt->persistent_));
+            map->set("always-send", Element::create(opt.persistent_));
+            // Set the cancellation flag
+            map->set("never-send", Element::create(opt.cancelled_));
             // Push on the list
             result->add(map);
         }
@@ -528,6 +551,5 @@ CfgOption::toElementWithMetadata(const bool include_metadata) const {
     return (result);
 }
 
-
 }  // namespace dhcp
 }  // namespace isc
