@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #include <config.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcp/option_custom.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/cfg_subnets4.h>
 #include <dhcpsrv/dhcpsrv_log.h>
 #include <dhcpsrv/lease_mgr_factory.h>
@@ -154,7 +155,7 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
         other_subnet->getCfgOption()->createOptions(cfg_def);
 
         // Create the options for pool based on the given definitions.
-        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_V4)) {
+        for (const auto& pool : other_subnet->getPoolsWritable(Lease::TYPE_V4)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
@@ -177,6 +178,8 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
                           << ", network does not exist");
             }
         }
+        // Instantiate the configured allocator and its state.
+        other_subnet->createAllocators();
     }
 }
 
@@ -224,14 +227,21 @@ CfgSubnets4::initSelector(const Pkt4Ptr& query) {
         OptionCustomPtr rai_custom =
             boost::dynamic_pointer_cast<OptionCustom>(rai);
         if (rai_custom) {
-            OptionPtr link_select =
-                rai_custom->getOption(RAI_OPTION_LINK_SELECTION);
-            if (link_select) {
-                OptionBuffer link_select_buf = link_select->getData();
-                if (link_select_buf.size() == sizeof(uint32_t)) {
-                    selector.option_select_ =
-                        IOAddress::fromBytes(AF_INET, &link_select_buf[0]);
-                    return (selector);
+            // If Relay Agent Information Link Selection is ignored in the
+            // configuration, skip returning the related subnet selector here,
+            // and move on to normal subnet selection.
+            bool ignore_link_sel = CfgMgr::instance().getCurrentCfg()->
+                                   getIgnoreRAILinkSelection();
+            if (!ignore_link_sel) {
+                OptionPtr link_select =
+                    rai_custom->getOption(RAI_OPTION_LINK_SELECTION);
+                if (link_select) {
+                    OptionBuffer link_select_buf = link_select->getData();
+                    if (link_select_buf.size() == sizeof(uint32_t)) {
+                        selector.option_select_ =
+                            IOAddress::fromBytes(AF_INET, &link_select_buf[0]);
+                        return (selector);
+                    }
                 }
             }
         }
@@ -487,6 +497,24 @@ CfgSubnets4::selectSubnet(const IOAddress& address,
     return (Subnet4Ptr());
 }
 
+SubnetIDSet
+CfgSubnets4::getLinks(const IOAddress& link_addr, uint8_t& link_len) const {
+    SubnetIDSet links;
+    bool link_len_set = false;
+    for (auto const& subnet : subnets_) {
+        if (!subnet->inRange(link_addr)) {
+            continue;
+        }
+        uint8_t plen = subnet->get().second;
+        if (!link_len_set || (plen < link_len)) {
+            link_len_set = true;
+            link_len = plen;
+        }
+        links.insert(subnet->getID());
+    }
+    return (links);
+}
+
 void
 CfgSubnets4::removeStatistics() {
     using namespace isc::stats;
@@ -512,6 +540,32 @@ CfgSubnets4::removeStatistics() {
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "reclaimed-leases"));
+
+        for (const auto& pool : subnet4->getPools(Lease::TYPE_V4)) {
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "total-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "assigned-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "cumulative-assigned-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "declined-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "reclaimed-declined-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "reclaimed-leases")));
+        }
     }
 }
 
@@ -525,17 +579,41 @@ CfgSubnets4::updateStatistics() {
 
         stats_mgr.setValue(StatsMgr::
                            generateName("subnet", subnet_id, "total-addresses"),
-                                        static_cast<int64_t>
-                                        (subnet4->getPoolCapacity(Lease::TYPE_V4)));
-        std::string name =
-            StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-addresses");
+                                        int64_t(subnet4->getPoolCapacity(Lease::TYPE_V4)));
+        const std::string& name(StatsMgr::generateName("subnet", subnet_id,
+                                                       "cumulative-assigned-addresses"));
         if (!stats_mgr.getObservation(name)) {
             stats_mgr.setValue(name, static_cast<int64_t>(0));
         }
 
-        name = StatsMgr::generateName("subnet", subnet_id, "v4-reservation-conflicts");
-        if (!stats_mgr.getObservation(name)) {
-            stats_mgr.setValue(name, static_cast<int64_t>(0));
+        const std::string& name_reuses(StatsMgr::generateName("subnet", subnet_id,
+                                                              "v4-lease-reuses"));
+        if (!stats_mgr.getObservation(name_reuses)) {
+            stats_mgr.setValue(name_reuses, int64_t(0));
+        }
+
+        const std::string& name_conflicts(StatsMgr::generateName("subnet", subnet_id,
+                                                                 "v4-reservation-conflicts"));
+        if (!stats_mgr.getObservation(name_conflicts)) {
+            stats_mgr.setValue(name_conflicts, static_cast<int64_t>(0));
+        }
+
+        for (const auto& pool : subnet4->getPools(Lease::TYPE_V4)) {
+            const std::string& name_total(StatsMgr::generateName("subnet", subnet_id,
+                                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                                        "total-addresses")));
+            if (!stats_mgr.getObservation(name_total)) {
+                stats_mgr.setValue(name_total, static_cast<int64_t>(pool->getCapacity()));
+            } else {
+                stats_mgr.addValue(name_total, static_cast<int64_t>(pool->getCapacity()));
+            }
+
+            const std::string& name_ca(StatsMgr::generateName("subnet", subnet_id,
+                                                              StatsMgr::generateName("pool", pool->getID(),
+                                                                                     "cumulative-assigned-addresses")));
+            if (!stats_mgr.getObservation(name_ca)) {
+                stats_mgr.setValue(name_ca, static_cast<int64_t>(0));
+            }
         }
     }
 
@@ -545,6 +623,18 @@ CfgSubnets4::updateStatistics() {
     }
 }
 
+void
+CfgSubnets4::initAllocatorsAfterConfigure() {
+    for (auto subnet : subnets_) {
+        subnet->initAllocatorsAfterConfigure();
+    }
+}
+
+void
+CfgSubnets4::clear() {
+    subnets_.clear();
+}
+
 ElementPtr
 CfgSubnets4::toElement() const {
     ElementPtr result = Element::createList();
