@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,8 @@
 using namespace isc::asiolink;
 using namespace isc::data;
 
+using namespace std;
+
 namespace isc {
 namespace dhcp {
 
@@ -150,12 +152,12 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
         other_subnet->getCfgOption()->createOptions(cfg_def);
 
         // Create the options for pool based on the given definitions.
-        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_NA)) {
+        for (const auto& pool : other_subnet->getPoolsWritable(Lease::TYPE_NA)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
         // Create the options for pd pool based on the given definitions.
-        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_PD)) {
+        for (const auto& pool : other_subnet->getPoolsWritable(Lease::TYPE_PD)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
@@ -178,6 +180,8 @@ CfgSubnets6::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks6Ptr networks,
                           << ", network does not exist");
             }
         }
+        // Instantiate the configured allocators and their states.
+        other_subnet->createAllocators();
     }
 }
 
@@ -382,6 +386,24 @@ CfgSubnets6::getSubnet(const SubnetID id) const {
     return (Subnet6Ptr());
 }
 
+SubnetIDSet
+CfgSubnets6::getLinks(const IOAddress& link_addr, uint8_t& link_len) const {
+    SubnetIDSet links;
+    bool link_len_set = false;
+    for (auto const& subnet : subnets_) {
+        if (!subnet->inRange(link_addr)) {
+            continue;
+        }
+        uint8_t plen = subnet->get().second;
+        if (!link_len_set || (plen < link_len)) {
+            link_len_set = true;
+            link_len = plen;
+        }
+        links.insert(subnet->getID());
+    }
+    return (links);
+}
+
 void
 CfgSubnets6::removeStatistics() {
     using namespace isc::stats;
@@ -390,7 +412,8 @@ CfgSubnets6::removeStatistics() {
     // For each v6 subnet currently configured, remove the statistics.
     for (auto const& subnet6 : subnets_) {
         SubnetID subnet_id = subnet6->getID();
-        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id, "total-nas"));
+        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                             "total-nas"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "assigned-nas"));
@@ -398,7 +421,8 @@ CfgSubnets6::removeStatistics() {
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "cumulative-assigned-nas"));
 
-        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id, "total-pds"));
+        stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                             "total-pds"));
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "assigned-pds"));
@@ -414,6 +438,50 @@ CfgSubnets6::removeStatistics() {
 
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "reclaimed-leases"));
+
+        for (const auto& pool : subnet6->getPools(Lease::TYPE_NA)) {
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "total-nas")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "assigned-nas")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "cumulative-assigned-nas")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "declined-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "reclaimed-declined-addresses")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pool", pool->getID(),
+                                                                        "reclaimed-leases")));
+        }
+
+        for (const auto& pool : subnet6->getPools(Lease::TYPE_PD)) {
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                        "total-pds")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                        "assigned-pds")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                        "cumulative-assigned-pds")));
+
+            stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
+                                                 StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                        "reclaimed-leases")));
+        }
     }
 }
 
@@ -428,25 +496,71 @@ CfgSubnets6::updateStatistics() {
 
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "total-nas"),
-                           static_cast<int64_t>
-                           (subnet6->getPoolCapacity(Lease::TYPE_NA)));
+                           subnet6->getPoolCapacity(Lease::TYPE_NA));
 
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "total-pds"),
-                            static_cast<int64_t>
-                            (subnet6->getPoolCapacity(Lease::TYPE_PD)));
+                           subnet6->getPoolCapacity(Lease::TYPE_PD));
 
-        const std::string& name_nas =
-            StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-nas");
+        const std::string& name_nas(StatsMgr::generateName("subnet", subnet_id,
+                                                           "cumulative-assigned-nas"));
         if (!stats_mgr.getObservation(name_nas)) {
             stats_mgr.setValue(name_nas, static_cast<int64_t>(0));
         }
 
-        const std::string& name_pds =
-            StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-pds");
+        const std::string& name_pds(StatsMgr::generateName("subnet", subnet_id,
+                                                           "cumulative-assigned-pds"));
         if (!stats_mgr.getObservation(name_pds)) {
             stats_mgr.setValue(name_pds, static_cast<int64_t>(0));
         }
+
+        string const& name_ia_na_reuses(StatsMgr::generateName("subnet", subnet_id,
+                                                               "v6-ia-na-lease-reuses"));
+        if (!stats_mgr.getObservation(name_ia_na_reuses)) {
+            stats_mgr.setValue(name_ia_na_reuses, int64_t(0));
+        }
+
+        string const& name_ia_pd_reuses(StatsMgr::generateName("subnet", subnet_id,
+                                                               "v6-ia-pd-lease-reuses"));
+        if (!stats_mgr.getObservation(name_ia_pd_reuses)) {
+            stats_mgr.setValue(name_ia_pd_reuses, int64_t(0));
+        }
+
+        for (const auto& pool : subnet6->getPools(Lease::TYPE_NA)) {
+            const std::string& name_total_nas(StatsMgr::generateName("subnet", subnet_id,
+                                                                     StatsMgr::generateName("pool", pool->getID(),
+                                                                                            "total-nas")));
+            if (!stats_mgr.getObservation(name_total_nas)) {
+                stats_mgr.setValue(name_total_nas, pool->getCapacity());
+            } else {
+                stats_mgr.addValue(name_total_nas, pool->getCapacity());
+            }
+
+            const std::string& name_ca_nas(StatsMgr::generateName("subnet", subnet_id,
+                                                                  StatsMgr::generateName("pool", pool->getID(),
+                                                                                         "cumulative-assigned-nas")));
+            if (!stats_mgr.getObservation(name_ca_nas)) {
+                stats_mgr.setValue(name_ca_nas, static_cast<int64_t>(0));
+            }
+        }
+
+        for (const auto& pool : subnet6->getPools(Lease::TYPE_PD)) {
+            const std::string& name_total_pds(StatsMgr::generateName("subnet", subnet_id,
+                                                                     StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                                            "total-pds")));
+            if (!stats_mgr.getObservation(name_total_pds)) {
+                stats_mgr.setValue(name_total_pds, pool->getCapacity());
+            } else {
+                stats_mgr.addValue(name_total_pds, pool->getCapacity());
+            }
+
+            const std::string& name_ca_pds(StatsMgr::generateName("subnet", subnet_id,
+                                                                  StatsMgr::generateName("pd-pool", pool->getID(),
+                                                                                         "cumulative-assigned-pds")));
+            if (!stats_mgr.getObservation(name_ca_pds)) {
+                stats_mgr.setValue(name_ca_pds, static_cast<int64_t>(0));
+            }
+        }
     }
 
     // Only recount the stats if we have subnets.
@@ -455,6 +569,18 @@ CfgSubnets6::updateStatistics() {
     }
 }
 
+void
+CfgSubnets6::initAllocatorsAfterConfigure() {
+    for (auto subnet : subnets_) {
+        subnet->initAllocatorsAfterConfigure();
+    }
+}
+
+void
+CfgSubnets6::clear() {
+    subnets_.clear();
+}
+
 ElementPtr
 CfgSubnets6::toElement() const {
     ElementPtr result = Element::createList();
