@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -15,9 +15,6 @@
 #ifdef HAVE_PGSQL
 #include <dhcpsrv/pgsql_lease_mgr.h>
 #endif
-#ifdef HAVE_CQL
-#include <dhcpsrv/cql_lease_mgr.h>
-#endif
 
 #include <boost/algorithm/string.hpp>
 #include <boost/foreach.hpp>
@@ -35,10 +32,10 @@ using namespace std;
 namespace isc {
 namespace dhcp {
 
-boost::scoped_ptr<LeaseMgr>&
+boost::scoped_ptr<TrackingLeaseMgr>&
 LeaseMgrFactory::getLeaseMgrPtr() {
-    static boost::scoped_ptr<LeaseMgr> leaseMgrPtr;
-    return (leaseMgrPtr);
+    static boost::scoped_ptr<TrackingLeaseMgr> lease_mgr_ptr;
+    return (lease_mgr_ptr);
 }
 
 void
@@ -79,17 +76,6 @@ LeaseMgrFactory::create(const std::string& dbaccess) {
         LOG_ERROR(dhcpsrv_logger, DHCPSRV_UNKNOWN_DB).arg("postgresql");
         isc_throw(InvalidType, "The Kea server has not been compiled with "
                   "support for database type: postgresql");
-#endif
-    }
-    if (parameters[type] == string("cql")) {
-#ifdef HAVE_CQL
-        LOG_INFO(dhcpsrv_logger, DHCPSRV_CQL_DB).arg(redacted);
-        getLeaseMgrPtr().reset(new CqlLeaseMgr(parameters));
-        return;
-#else
-        LOG_ERROR(dhcpsrv_logger, DHCPSRV_UNKNOWN_DB).arg("cql");
-        isc_throw(InvalidType, "The Kea server has not been compiled with "
-                  "support for database type: cql");
 #endif
     }
     if (parameters[type] == string("memfile")) {
@@ -115,14 +101,33 @@ LeaseMgrFactory::destroy() {
     getLeaseMgrPtr().reset();
 }
 
+void
+LeaseMgrFactory::recreate(const std::string& dbaccess, bool preserve_callbacks) {
+    TrackingLeaseMgr::CallbackContainerPtr callbacks;
+    // Preserve the callbacks if needed.
+    if (preserve_callbacks && haveInstance()) {
+        callbacks = instance().callbacks_;
+    }
+
+    // Re-create the manager.
+    destroy();
+    create(dbaccess);
+
+    if (callbacks) {
+        // Copy the callbacks to the new instance. It should be fast
+        // because we merely copy the pointer.
+        instance().callbacks_ = callbacks;
+    }
+}
+
 bool
 LeaseMgrFactory::haveInstance() {
     return (getLeaseMgrPtr().get());
 }
 
-LeaseMgr&
+TrackingLeaseMgr&
 LeaseMgrFactory::instance() {
-    LeaseMgr* lmptr = getLeaseMgrPtr().get();
+    TrackingLeaseMgr* lmptr = getLeaseMgrPtr().get();
     if (lmptr == NULL) {
         isc_throw(NoLeaseManager, "no current lease manager is available");
     }
