@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -31,7 +31,8 @@ SharedNetwork4Parser::SharedNetwork4Parser(bool check_iface)
 }
 
 SharedNetwork4Ptr
-SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
+SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data,
+                            bool encapsulate_options) {
     SharedNetwork4Ptr shared_network;
     try {
 
@@ -75,7 +76,7 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
             auto parser = createOptionDataListParser();
-            parser->parse(cfg_option, json);
+            parser->parse(cfg_option, json, encapsulate_options);
         }
 
         if (shared_network_data->contains("subnet4")) {
@@ -197,6 +198,14 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
 
         // Parse lease cache parameters
         parseCacheParams(shared_network_data, network);
+
+        // Parse allocator params.
+        parseAllocatorParams(shared_network_data, network);
+
+        // Parse offer-lifetime parameter.
+        Network4Ptr network4 = boost::dynamic_pointer_cast<Network4>(shared_network);
+        parseOfferLft(shared_network_data, network4);
+
     } catch (const DhcpConfigError&) {
         // Position was already added
         throw;
@@ -232,7 +241,8 @@ SharedNetwork6Parser::SharedNetwork6Parser(bool check_iface)
 }
 
 SharedNetwork6Ptr
-SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
+SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data,
+                            bool encapsulate_options) {
     SharedNetwork6Ptr shared_network;
     std::string name;
     try {
@@ -316,7 +326,7 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
             auto parser = createOptionDataListParser();
-            parser->parse(cfg_option, json);
+            parser->parse(cfg_option, json, encapsulate_options);
         }
 
         if (shared_network_data->contains("client-class")) {
@@ -377,6 +387,17 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
 
         // Parse lease cache parameters
         parseCacheParams(shared_network_data, network);
+
+        // Parse allocator params.
+        parseAllocatorParams(shared_network_data, network);
+        if (network->getAllocatorType() == "flq") {
+            isc_throw(BadValue, "Free Lease Queue allocator is not supported for IPv6 address pools");
+        }
+
+        // Parse prefix delegation allocator params.
+        auto network6 = boost::dynamic_pointer_cast<Network6>(shared_network);
+        parsePdAllocatorParams(shared_network_data, network6);
+
     } catch (const std::exception& ex) {
         isc_throw(DhcpConfigError, ex.what() << " ("
                   << shared_network_data->getPosition() << ")");
