@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -100,6 +100,9 @@ const SimpleKeywords SimpleParser6::GLOBAL6_PARAMETERS = {
     { "ddns-use-conflict-resolution",     Element::boolean },
     { "compatibility",                    Element::map },
     { "parked-packet-limit",              Element::integer },
+    { "allocator",                        Element::string },
+    { "pd-allocator",                     Element::string },
+    { "ddns-ttl-percent",                 Element::real },
 };
 
 /// @brief This table defines default global values for DHCPv6
@@ -108,7 +111,7 @@ const SimpleKeywords SimpleParser6::GLOBAL6_PARAMETERS = {
 /// in Dhcp6) are optional. If not defined, the following values will be
 /// used.
 const SimpleDefaults SimpleParser6::GLOBAL6_DEFAULTS = {
-    { "preferred-lifetime",               Element::integer, "3600" },
+    // preferred-lifetime is unspecified and defaults to 0.625 of valid-lifetime.
     { "valid-lifetime",                   Element::integer, "7200" },
     { "decline-probation-period",         Element::integer, "86400" }, // 24h
     { "dhcp4o6-port",                     Element::integer, "0" },
@@ -135,7 +138,9 @@ const SimpleDefaults SimpleParser6::GLOBAL6_DEFAULTS = {
     { "reservations-lookup-first",        Element::boolean, "false" },
     { "ddns-update-on-renew",             Element::boolean, "false" },
     { "ddns-use-conflict-resolution",     Element::boolean, "true" },
-    { "parked-packet-limit",              Element::integer, "256" }
+    { "parked-packet-limit",              Element::integer, "256" },
+    { "allocator",                        Element::string,  "iterative" },
+    { "pd-allocator",                     Element::string,  "iterative" },
 };
 
 /// @brief This table defines all option definition parameters.
@@ -179,6 +184,7 @@ const SimpleKeywords SimpleParser6::OPTION6_PARAMETERS = {
     { "space",        Element::string },
     { "csv-format",   Element::boolean },
     { "always-send",  Element::boolean },
+    { "never-send",   Element::boolean },
     { "user-context", Element::map },
     { "comment",      Element::string },
     { "metadata",     Element::map }
@@ -192,7 +198,8 @@ const SimpleKeywords SimpleParser6::OPTION6_PARAMETERS = {
 const SimpleDefaults SimpleParser6::OPTION6_DEFAULTS = {
     { "space",        Element::string,  "dhcp6"}, // DHCP6_OPTION_SPACE
     { "csv-format",   Element::boolean, "true"},
-    { "always-send",  Element::boolean, "false"}
+    { "always-send",  Element::boolean, "false"},
+    { "never-send",   Element::boolean, "false"}
 };
 
 /// @brief This table defines all subnet parameters for DHCPv6.
@@ -243,7 +250,10 @@ const SimpleKeywords SimpleParser6::SUBNET6_PARAMETERS = {
     { "cache-threshold",                Element::real },
     { "cache-max-age",                  Element::integer },
     { "ddns-update-on-renew",           Element::boolean },
-    { "ddns-use-conflict-resolution",   Element::boolean }
+    { "ddns-use-conflict-resolution",   Element::boolean },
+    { "allocator",                      Element::string },
+    { "pd-allocator",                   Element::string },
+    { "ddns-ttl-percent",               Element::real },
 };
 
 /// @brief This table defines default values for each IPv6 subnet.
@@ -265,6 +275,11 @@ const SimpleDefaults SimpleParser6::SUBNET6_DEFAULTS = {
 ///
 /// This is mostly the same as @ref SUBNET6_DEFAULTS, except the parameters
 /// that can be derived from shared-network, but cannot from global scope.
+const SimpleDefaults SimpleParser6::SHARED_SUBNET6_DEFAULTS = {
+    { "id",               Element::integer, "0" } // 0 means autogenerate
+};
+
+/// @brief This table defines default values for each IPv6 shared network.
 const SimpleDefaults SimpleParser6::SHARED_NETWORK6_DEFAULTS = {
     { "client-class",     Element::string,  "" },
     { "interface",        Element::string,  "" },
@@ -296,7 +311,9 @@ const ParamsList SimpleParser6::INHERIT_TO_SUBNET6 = {
     "t2-percent",
     "store-extended-info",
     "cache-threshold",
-    "cache-max-age"
+    "cache-max-age",
+    "allocator",
+    "pd-allocator"
 };
 
 /// @brief This table defines all pool parameters.
@@ -306,6 +323,7 @@ const ParamsList SimpleParser6::INHERIT_TO_SUBNET6 = {
 /// Order follows pool_param rules in bison grammar.
 const SimpleKeywords SimpleParser6::POOL6_PARAMETERS = {
     { "pool",                   Element::string },
+    { "pool-id",                Element::integer },
     { "option-data",            Element::list },
     { "client-class",           Element::string },
     { "require-client-classes", Element::list },
@@ -323,6 +341,7 @@ const SimpleKeywords SimpleParser6::PD_POOL6_PARAMETERS = {
     { "prefix",                 Element::string },
     { "prefix-len",             Element::integer },
     { "delegated-len",          Element::integer },
+    { "pool-id",                Element::integer },
     { "option-data",            Element::list },
     { "client-class",           Element::string },
     { "require-client-classes", Element::list },
@@ -378,12 +397,10 @@ const SimpleKeywords SimpleParser6::SHARED_NETWORK6_PARAMETERS = {
     { "cache-threshold",                Element::real },
     { "cache-max-age",                  Element::integer },
     { "ddns-update-on-renew",           Element::boolean },
-    { "ddns-use-conflict-resolution",   Element::boolean }
-};
-
-/// @brief This table defines default values for each IPv6 subnet.
-const SimpleDefaults SimpleParser6::SHARED_SUBNET6_DEFAULTS = {
-    { "id",               Element::integer, "0" } // 0 means autogenerate
+    { "ddns-use-conflict-resolution",   Element::boolean },
+    { "allocator",                      Element::string },
+    { "pd-allocator",                   Element::string },
+    { "ddns-ttl-percent",               Element::real },
 };
 
 /// @brief This table defines default values for interfaces for DHCPv6.
@@ -400,7 +417,7 @@ const SimpleDefaults SimpleParser6::DHCP_QUEUE_CONTROL6_DEFAULTS = {
 
 /// @brief This table defines default values for multi-threading in DHCPv6.
 const SimpleDefaults SimpleParser6::DHCP_MULTI_THREADING6_DEFAULTS = {
-    { "enable-multi-threading", Element::boolean, "false" },
+    { "enable-multi-threading", Element::boolean, "true" },
     { "thread-pool-size",       Element::integer, "0" },
     { "packet-queue-size",      Element::integer, "64" }
 };
