@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,6 +14,7 @@
 
 using namespace isc;
 using namespace isc::dhcp;
+using namespace isc::util;
 using namespace asiolink;
 
 namespace {
@@ -30,8 +31,8 @@ void checkMergedNetwork(const CfgSharedNetworks6& networks, const std::string& n
                         const std::vector<SubnetID>& exp_subnets) {
     auto network = networks.getByName(name);
     ASSERT_TRUE(network) << "expected network: " << name << " not found";
-    ASSERT_EQ(exp_valid, network->getValid()) << " network valid lifetime wrong";
-    const Subnet6Collection* subnets = network->getAllSubnets();
+    ASSERT_EQ(exp_valid, network->getValid().get()) << " network valid lifetime wrong";
+    const Subnet6SimpleCollection* subnets = network->getAllSubnets();
     ASSERT_EQ(exp_subnets.size(), subnets->size()) << " wrong number of subnets";
     for (auto exp_id : exp_subnets) {
         ASSERT_TRUE(network->getSubnet(exp_id))
@@ -253,7 +254,11 @@ TEST(CfgSharedNetworks6Test, unparse) {
         "    \"renew-timer\": 100,\n"
         "    \"subnet6\": [ ],\n"
         "    \"preferred-lifetime\": 200,\n"
-        "    \"valid-lifetime\": 300\n,"
+        "    \"min-preferred-lifetime\": 200,\n"
+        "    \"max-preferred-lifetime\": 200,\n"
+        "    \"valid-lifetime\": 300,\n"
+        "    \"min-valid-lifetime\": 300,\n"
+        "    \"max-valid-lifetime\": 300,\n"
         "    \"store-extended-info\": true,\n"
         "    \"cache-max-age\": 80\n"
         "  },\n"
@@ -341,7 +346,7 @@ TEST(CfgSharedNetworks6Test, mergeNetworks) {
     std::string value("Yay!");
     OptionPtr option(new Option(Option::V6, 1));
     option->setData(value.begin(), value.end());
-    ASSERT_NO_THROW(network1b->getCfgOption()->add(option, false, "isc"));
+    ASSERT_NO_THROW(network1b->getCfgOption()->add(option, false, false, "isc"));
     ASSERT_NO_THROW(network1b->add(subnet4));
 
     // Network2 we will not touch.
@@ -360,7 +365,7 @@ TEST(CfgSharedNetworks6Test, mergeNetworks) {
     // Should still have 3 networks.
 
     // Network1 should have doubled its valid lifetime but still only have
-    // the orignal two subnets.  Merge should discard assocations on CB
+    // the orignal two subnets.  Merge should discard associations on CB
     // subnets and preserve the associations from existing config.
     ASSERT_EQ(3, cfg_to.getAll()->size());
     ASSERT_NO_FATAL_FAILURE(checkMergedNetwork(cfg_to, "network1", Triplet<uint32_t>(200),
