@@ -1,12 +1,12 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <dhcp/iface_mgr.h>
-#include <dhcp/option_vendor.h>
 #include <dhcp/pkt6.h>
 #include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcp/option6_addrlst.h>
@@ -15,9 +15,9 @@
 #include <dhcp/option_int.h>
 #include <dhcpsrv/dhcp4o6_ipc.h>
 #include <dhcpsrv/testutils/dhcp4o6_test_ipc.h>
-#include <boost/bind.hpp>
-#include <boost/function.hpp>
+
 #include <gtest/gtest.h>
+#include <functional>
 #include <sstream>
 #include <string>
 
@@ -35,7 +35,7 @@ const uint16_t TEST_PORT = 12345;
 const uint16_t TEST_ITERATIONS = 10;
 
 /// @brief Type definition for the function creating DHCP message.
-typedef boost::function<Pkt6Ptr(uint16_t, uint16_t)> CreateMsgFun;
+typedef std::function<Pkt6Ptr(uint16_t, uint16_t)> CreateMsgFun;
 
 /// @brief Define short name for test IPC class.
 typedef Dhcp4o6TestIpc TestIpc;
@@ -167,6 +167,7 @@ Dhcp4o6IpcBaseTest::createDHCPv4o6Message(uint16_t msg_type,
     // this test includes two interfaces: "eth0" and "eth1". Therefore,
     // we pick one or another, depending on the index of the iteration.
     pkt->setIface(concatenate("eth", postfix % 2));
+    pkt->setIndex(ETH0_INDEX + postfix % 2);
 
     // The remote address of the sender of the DHCPv6 packet is carried
     // between the servers in the dedicated option. We use different
@@ -286,6 +287,7 @@ Dhcp4o6IpcBaseTest::testSendReceive(uint16_t iterations_num,
 
         // Check that the interface is correct.
         EXPECT_EQ(concatenate("eth", i % 2), pkt_received->getIface());
+        EXPECT_EQ(ETH0_INDEX + i % 2, pkt_received->getIndex());
 
         // Check that the address conveyed is correct.
         EXPECT_EQ(concatenate("2001:db8:1::", i),
@@ -327,6 +329,7 @@ Dhcp4o6IpcBaseTest::testReceiveError(const Pkt6Ptr& pkt) {
     ASSERT_NO_THROW(ipc_dest.open());
 
     pkt->setIface("eth0");
+    pkt->setIndex(ETH0_INDEX);
     pkt->setRemoteAddr(IOAddress("2001:db8:1::1"));
     pkt->setRemotePort(TEST_PORT);
     pkt->addOption(createDHCPv4MsgOption(TestIpc::ENDPOINT_TYPE_V6));
@@ -463,13 +466,11 @@ TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutVendorOption) {
 TEST_F(Dhcp4o6IpcBaseTest, receiveInvalidEnterpriseId) {
     Pkt6Ptr pkt(new Pkt6(DHCPV6_DHCPV4_QUERY, 0));
     OptionVendorPtr option_vendor(new OptionVendor(Option::V6, 1));
-    option_vendor->addOption(
-        OptionStringPtr(new OptionString(Option::V6, ISC_V6_4O6_INTERFACE,
-                                         "eth0")));
-    option_vendor->addOption(
-        Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
-                                             IOAddress("2001:db8:1::1")))
-    );
+    option_vendor->addOption(OptionStringPtr(new OptionString(Option::V6,
+                                                              ISC_V6_4O6_INTERFACE,
+                                                              "eth0")));
+    option_vendor->addOption(Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
+                                                                  IOAddress("2001:db8:1::1"))));
 
     pkt->addOption(option_vendor);
     testReceiveError(pkt);
@@ -479,16 +480,12 @@ TEST_F(Dhcp4o6IpcBaseTest, receiveInvalidEnterpriseId) {
 // interface option is not present.
 TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutInterfaceOption) {
     Pkt6Ptr pkt(new Pkt6(DHCPV6_DHCPV4_QUERY, 0));
-    OptionVendorPtr option_vendor(new OptionVendor(Option::V6,
-                                                   ENTERPRISE_ID_ISC));
-    option_vendor->addOption(
-        Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
-                                             IOAddress("2001:db8:1::1")))
-    );
-    option_vendor->addOption(
-        OptionUint16Ptr(new OptionUint16(Option::V6,
-                                         ISC_V6_4O6_SRC_PORT,
-                                         TEST_PORT)));
+    OptionVendorPtr option_vendor(new OptionVendor(Option::V6, ENTERPRISE_ID_ISC));
+    option_vendor->addOption(Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
+                                                                  IOAddress("2001:db8:1::1"))));
+    option_vendor->addOption(OptionUint16Ptr(new OptionUint16(Option::V6,
+                                                              ISC_V6_4O6_SRC_PORT,
+                                                              TEST_PORT)));
 
     pkt->addOption(option_vendor);
     testReceiveError(pkt);
@@ -499,19 +496,15 @@ TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutInterfaceOption) {
 // system.
 TEST_F(Dhcp4o6IpcBaseTest, receiveWithInvalidInterface) {
     Pkt6Ptr pkt(new Pkt6(DHCPV6_DHCPV4_QUERY, 0));
-    OptionVendorPtr option_vendor(new OptionVendor(Option::V6,
-                                                   ENTERPRISE_ID_ISC));
-    option_vendor->addOption(
-        OptionStringPtr(new OptionString(Option::V6, ISC_V6_4O6_INTERFACE,
-                                         "ethX")));
-    option_vendor->addOption(
-        Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
-                                             IOAddress("2001:db8:1::1")))
-    );
-    option_vendor->addOption(
-        OptionUint16Ptr(new OptionUint16(Option::V6,
-                                         ISC_V6_4O6_SRC_PORT,
-                                         TEST_PORT)));
+    OptionVendorPtr option_vendor(new OptionVendor(Option::V6, ENTERPRISE_ID_ISC));
+    option_vendor->addOption(OptionStringPtr(new OptionString(Option::V6,
+                                                              ISC_V6_4O6_INTERFACE,
+                                                              "ethX")));
+    option_vendor->addOption(Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
+                                                                  IOAddress("2001:db8:1::1"))));
+    option_vendor->addOption(OptionUint16Ptr(new OptionUint16(Option::V6,
+                                                              ISC_V6_4O6_SRC_PORT,
+                                                              TEST_PORT)));
 
     pkt->addOption(option_vendor);
     testReceiveError(pkt);
@@ -522,15 +515,13 @@ TEST_F(Dhcp4o6IpcBaseTest, receiveWithInvalidInterface) {
 // source address option is not present.
 TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutSourceAddressOption) {
     Pkt6Ptr pkt(new Pkt6(DHCPV6_DHCPV4_QUERY, 0));
-    OptionVendorPtr option_vendor(new OptionVendor(Option::V6,
-                                                   ENTERPRISE_ID_ISC));
-    option_vendor->addOption(
-        OptionStringPtr(new OptionString(Option::V6, ISC_V6_4O6_INTERFACE,
-                                         "eth0")));
-    option_vendor->addOption(
-        OptionUint16Ptr(new OptionUint16(Option::V6,
-                                         ISC_V6_4O6_SRC_PORT,
-                                         TEST_PORT)));
+    OptionVendorPtr option_vendor(new OptionVendor(Option::V6, ENTERPRISE_ID_ISC));
+    option_vendor->addOption(OptionStringPtr(new OptionString(Option::V6,
+                                             ISC_V6_4O6_INTERFACE,
+                                             "eth0")));
+    option_vendor->addOption(OptionUint16Ptr(new OptionUint16(Option::V6,
+                                                              ISC_V6_4O6_SRC_PORT,
+                                                              TEST_PORT)));
 
     pkt->addOption(option_vendor);
     testReceiveError(pkt);
@@ -540,15 +531,12 @@ TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutSourceAddressOption) {
 // source port option is not present.
 TEST_F(Dhcp4o6IpcBaseTest, receiveWithoutSourcePortOption) {
     Pkt6Ptr pkt(new Pkt6(DHCPV6_DHCPV4_QUERY, 0));
-    OptionVendorPtr option_vendor(new OptionVendor(Option::V6,
-                                                   ENTERPRISE_ID_ISC));
-    option_vendor->addOption(
-        OptionStringPtr(new OptionString(Option::V6, ISC_V6_4O6_INTERFACE,
-                                         "eth0")));
-    option_vendor->addOption(
-        Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
-                                             IOAddress("2001:db8:1::1")))
-    );
+    OptionVendorPtr option_vendor(new OptionVendor(Option::V6, ENTERPRISE_ID_ISC));
+    option_vendor->addOption(OptionStringPtr(new OptionString(Option::V6,
+                                                              ISC_V6_4O6_INTERFACE,
+                                                              "eth0")));
+    option_vendor->addOption(Option6AddrLstPtr(new Option6AddrLst(ISC_V6_4O6_SRC_ADDRESS,
+                                                                  IOAddress("2001:db8:1::1"))));
 
     pkt->addOption(option_vendor);
     testReceiveError(pkt);
