@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,15 +12,12 @@
 #include <dhcpsrv/pool.h>
 #include <testutils/test_to_element.h>
 
-#include <boost/scoped_ptr.hpp>
-
 #include <gtest/gtest.h>
 
 #include <iostream>
 #include <vector>
 #include <sstream>
 
-using boost::scoped_ptr;
 using namespace isc;
 using namespace isc::dhcp;
 using namespace isc::data;
@@ -28,7 +25,7 @@ using namespace isc::asiolink;
 
 namespace {
 
-TEST(Pool4Test, constructor_first_last) {
+TEST(Pool4Test, constructorFirstLast) {
 
     // let's construct 192.0.2.1-192.0.2.255 pool
     Pool4 pool1(IOAddress("192.0.2.1"), IOAddress("192.0.2.255"));
@@ -48,7 +45,7 @@ TEST(Pool4Test, constructor_first_last) {
                  BadValue);
 }
 
-TEST(Pool4Test, constructor_prefix_len) {
+TEST(Pool4Test, constructorPrefixLen) {
 
     // let's construct 2001:db8:1::/96 pool
     Pool4 pool1(IOAddress("192.0.2.0"), 25);
@@ -94,26 +91,6 @@ TEST(Pool4Test, leasesCount) {
     EXPECT_EQ(16777216, pool4.getCapacity());
 }
 
-// This test creates 100 pools and verifies that their IDs are unique.
-TEST(Pool4Test, unique_id) {
-
-    const int num_pools = 100;
-    std::vector<Pool4Ptr> pools;
-
-    for (int i = 0; i < num_pools; ++i) {
-        pools.push_back(Pool4Ptr(new Pool4(IOAddress("192.0.2.0"),
-                                           IOAddress("192.0.2.255"))));
-    }
-
-    for (int i = 0; i < num_pools; ++i) {
-        for (int j = i + 1; j < num_pools; ++j) {
-            if (pools[i]->getId() == pools[j]->getId()) {
-                FAIL() << "Pool-ids must be unique";
-            }
-        }
-    }
-}
-
 // Simple check if toText returns reasonable values
 TEST(Pool4Test, toText) {
     Pool4 pool1(IOAddress("192.0.2.7"), IOAddress("192.0.2.17"));
@@ -143,9 +120,11 @@ TEST(Pool4Test, toElement) {
     isc::test::runToElementTest<Pool4>(expected2, pool2);
 
     Pool4 pool3(IOAddress("192.0.2.0"), IOAddress("192.0.2.127"));
+    pool3.setID(5);
     std::string expected3 = "{"
         " \"pool\": \"192.0.2.0/25\", "
-        " \"option-data\": [ ] "
+        " \"option-data\": [ ], "
+        " \"pool-id\": 5 "
         "}";
     isc::test::runToElementTest<Pool4>(expected3, pool3);
 }
@@ -159,14 +138,15 @@ TEST(Pool4Test, addOptions) {
     // Differentiate options by their codes (100-109)
     for (uint16_t code = 100; code < 110; ++code) {
         OptionPtr option(new Option(Option::V4, code, OptionBuffer(10, 0xFF)));
-        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, "dhcp4"));
+        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, false,
+                                                  "dhcp4"));
     }
 
     // Add 7 options to another option space. The option codes partially overlap
     // with option codes that we have added to dhcp4 option space.
     for (uint16_t code = 105; code < 112; ++code) {
         OptionPtr option(new Option(Option::V4, code, OptionBuffer(10, 0xFF)));
-        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, "isc"));
+        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, false, "isc"));
     }
 
     // Get options from the pool and check if all 10 are there.
@@ -287,29 +267,7 @@ TEST(Pool4Test, requiredClasses) {
     EXPECT_TRUE(pool->getRequiredClasses().contains("foo"));
 }
 
-// This test checks that handling for last allocated address/prefix is valid.
-TEST(Pool4Test, lastAllocated) {
-    // Create a pool.
-    IOAddress first("192.0.2.0");
-    Pool4Ptr pool(new Pool4(first, IOAddress("192.0.2.255")));
-
-    // Initial values are first invalid.
-    EXPECT_EQ(first.toText(), pool->getLastAllocated().toText());
-    EXPECT_FALSE(pool->isLastAllocatedValid());
-
-    // Now set last allocated
-    IOAddress addr("192.0.2.100");
-    EXPECT_NO_THROW(pool->setLastAllocated(addr));
-    EXPECT_EQ(addr.toText(), pool->getLastAllocated().toText());
-    EXPECT_TRUE(pool->isLastAllocatedValid());
-
-    // Reset makes it invalid and does not touch address
-    pool->resetLastAllocated();
-    EXPECT_EQ(addr.toText(), pool->getLastAllocated().toText());
-    EXPECT_FALSE(pool->isLastAllocatedValid());
-}
-
-TEST(Pool6Test, constructor_first_last) {
+TEST(Pool6Test, constructorFirstLast) {
 
     // let's construct 2001:db8:1:: - 2001:db8:1::ffff:ffff:ffff:ffff pool
     Pool6 pool1(Lease::TYPE_NA, IOAddress("2001:db8:1::"),
@@ -332,7 +290,7 @@ TEST(Pool6Test, constructor_first_last) {
                        IOAddress("2001:db8::1")), BadValue);
 }
 
-TEST(Pool6Test, constructor_prefix_len) {
+TEST(Pool6Test, constructorPrefixLen) {
 
     // let's construct 2001:db8:1::/96 pool
     Pool6 pool1(Lease::TYPE_NA, IOAddress("2001:db8:1::"), 96);
@@ -358,7 +316,7 @@ TEST(Pool6Test, constructor_prefix_len) {
                  BadValue);
 }
 
-TEST(Pool6Test, in_range) {
+TEST(Pool6Test, inRange) {
    Pool6 pool1(Lease::TYPE_NA, IOAddress("2001:db8:1::1"),
                IOAddress("2001:db8:1::f"));
 
@@ -385,14 +343,21 @@ TEST(Pool6Test, PD) {
     EXPECT_THROW(Pool6 pool2(Lease::TYPE_PD, IOAddress("2001:db8:1::1"),
                              IOAddress("2001:db8:1::f")), BadValue);
 
+    // Check that it's not allowed to specify bigger prefix address than the
+    // pool prefix length
+    // Should not be able to compute prefix if first address is not starting
+    // from prefix length 32 (2001:db8::)
+    EXPECT_THROW(Pool6 pool3(Lease::TYPE_PD, IOAddress("2001:db8:1::"),
+                             32, 56), BadValue);
+
     // Check that it's not allowed to delegate bigger prefix than the pool
     // Let's try to split /64 prefix into /56 chunks (should be impossible)
-    EXPECT_THROW(Pool6 pool3(Lease::TYPE_PD, IOAddress("2001:db8:1::"),
+    EXPECT_THROW(Pool6 pool4(Lease::TYPE_PD, IOAddress("2001:db8:1::"),
                              64, 56), BadValue);
 
     // It should be possible to have a pool split into just a single chunk
     // Let's try to split 2001:db8:1::/77 into a single /77 delegated prefix
-    EXPECT_NO_THROW(Pool6 pool4(Lease::TYPE_PD, IOAddress("2001:db8:1::"),
+    EXPECT_NO_THROW(Pool6 pool5(Lease::TYPE_PD, IOAddress("2001:db8:1::"),
                                 77, 77));
 }
 
@@ -453,7 +418,7 @@ TEST(Pool6Test, PDExclude) {
                  BadValue);
 
     // Excluded prefix must be an IPv6 prefix.
-    EXPECT_THROW(Pool6(IOAddress("10::"), 8, 16,
+    EXPECT_THROW(Pool6(IOAddress("100::"), 8, 16,
                        IOAddress("10.0.0.0"), 24),
                  BadValue);
 
@@ -496,27 +461,6 @@ TEST(Pool6Test, TA) {
     EXPECT_EQ("2001:db8:1::f", pool2->getLastAddress().toText());
 }
 
-// This test creates 100 pools and verifies that their IDs are unique.
-TEST(Pool6Test, unique_id) {
-
-    const int num_pools = 100;
-    std::vector<Pool6Ptr> pools;
-
-    for (int i = 0; i < num_pools; ++i) {
-        pools.push_back(Pool6Ptr(new Pool6(Lease::TYPE_NA, IOAddress("2001:db8:1::"),
-                                           IOAddress("2001:db8:1::ffff:ffff:ffff:ffff"))));
-    }
-
-    for (int i = 0; i < num_pools; ++i) {
-        for (int j = i + 1; j < num_pools; ++j) {
-            if (pools[i]->getId() == pools[j]->getId()) {
-                FAIL() << "Pool-ids must be unique";
-            }
-        }
-    }
-
-}
-
 // Simple check if toText returns reasonable values
 TEST(Pool6Test, toText) {
     Pool6 pool1(Lease::TYPE_NA, IOAddress("2001:db8::1"),
@@ -573,9 +517,11 @@ TEST(Pool6Test, toElement) {
 
     Pool6 pool4(Lease::TYPE_NA, IOAddress("2001:db8::"),
                 IOAddress("2001:db8::ffff"));
+    pool4.setID(5);
     std::string expected4 = "{"
         " \"pool\": \"2001:db8::/112\", "
-        " \"option-data\": [ ] "
+        " \"option-data\": [ ], "
+        " \"pool-id\": 5 "
         "}";
     isc::test::runToElementTest<Pool6>(expected4, pool4);
 }
@@ -598,14 +544,15 @@ TEST(Pool6Test, addOptions) {
     // Differentiate options by their codes (100-109)
     for (uint16_t code = 100; code < 110; ++code) {
         OptionPtr option(new Option(Option::V6, code, OptionBuffer(10, 0xFF)));
-        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, "dhcp6"));
+        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, false,
+                                                  "dhcp6"));
     }
 
     // Add 7 options to another option space. The option codes partially overlap
     // with option codes that we have added to dhcp6 option space.
     for (uint16_t code = 105; code < 112; ++code) {
         OptionPtr option(new Option(Option::V6, code, OptionBuffer(10, 0xFF)));
-        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, "isc"));
+        ASSERT_NO_THROW(pool->getCfgOption()->add(option, false, false, "isc"));
     }
 
     // Get options from the pool and check if all 10 are there.
@@ -726,26 +673,4 @@ TEST(Pool6Test, requiredClasses) {
     EXPECT_TRUE(pool.getRequiredClasses().contains("foo"));
 }
 
-// This test checks that handling for last allocated address/prefix is valid.
-TEST(Pool6Test, lastAllocated) {
-    // Create a pool.
-    IOAddress first("2001:db8::1");
-    Pool6 pool(Lease::TYPE_NA, first, IOAddress("2001:db8::200"));
-
-    // Initial values are first invalid.
-    EXPECT_EQ(first.toText(), pool.getLastAllocated().toText());
-    EXPECT_FALSE(pool.isLastAllocatedValid());
-
-    // Now set last allocated
-    IOAddress addr("2001:db8::100");
-    EXPECT_NO_THROW(pool.setLastAllocated(addr));
-    EXPECT_EQ(addr.toText(), pool.getLastAllocated().toText());
-    EXPECT_TRUE(pool.isLastAllocatedValid());
-
-    // Reset makes it invalid and does not touch address
-    pool.resetLastAllocated();
-    EXPECT_EQ(addr.toText(), pool.getLastAllocated().toText());
-    EXPECT_FALSE(pool.isLastAllocatedValid());
-}
-
-}; // end of anonymous namespace
+}  // end of anonymous namespace
