@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -142,6 +142,12 @@ public:
                         isc::data::ConstElementPtr user_context =
                         isc::data::ConstElementPtr()) const;
 
+    /// @brief Adds an IPv6 address to the host.
+    ///
+    /// @param host pointer to the host instance.
+    /// @param address an IPv6 address to be added as a string.
+    void addIPv6Address(const HostPtr& host, const std::string& address) const;
+
     /// @brief Pointer to the host data source
     HostDataSourcePtr hdsptr_;
 
@@ -513,6 +519,11 @@ public:
     /// Uses gtest macros to report failures.
     void testMultipleHosts6();
 
+    /// @brief Tests that hosts can be updated.
+    ///
+    /// Uses gtest macros to report failures.
+    void testUpdate();
+
     /// @brief Returns DUID with identical content as specified HW address
     ///
     /// This method does not have any sense in real life and is only useful
@@ -735,6 +746,23 @@ protected:
                   const isc::asiolink::IOAddress& address,
                   const uint8_t prefix_len = 128);
 
+    /// @brief Inserts IPv6 reservation into the host data source.
+    ///
+    /// @param data_source Reference to the data source to which the reservation
+    /// should be inserted.
+    /// @param duid Pointer to the DUID to be associated with the reservation.
+    /// @param subnet_id IPv6 subnet id.
+    /// @param addresses IPv6 addresses/prefixes to be reserved.
+    /// @param prefix_len Prefix length. The default value is 128 which
+    /// indicates that the reservation is for an IPv6 address rather than a
+    /// prefix. Notice that this is common for all addresses in given vector
+    /// of addresses.
+    void addHost6(BaseHostDataSource& data_source,
+                  const DuidPtr& duid,
+                  const SubnetID& subnet_id,
+                  const std::vector<isc::asiolink::IOAddress>& addresses,
+                  const uint8_t prefix_len = 128);
+
     /// @brief This test verifies that HostMgr returns all reservations for the
     /// specified HW address.
     ///
@@ -924,6 +952,92 @@ protected:
     void testGetAll6BySubnetIP(BaseHostDataSource& data_source1,
                                BaseHostDataSource& data_source2);
 
+    /// @brief This test verifies that HostMgr returns all reservations for the
+    /// specified IPv6 address.
+    ///
+    /// If reservations are added to different host data sources, it is expected
+    /// that the @c HostMgr will retrieve reservations from both of them.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testGetAll6ByIP(BaseHostDataSource& data_source1,
+                         BaseHostDataSource& data_source2);
+
+    /// @brief This test verifies that HostMgr returns all reservations for the
+    /// specified IPv6 prefix/es.
+    ///
+    /// If reservations are added to different host data sources, it is expected
+    /// that the @c HostMgr will retrieve reservations from both of them.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testGetAll6ByIpPrefix(BaseHostDataSource& data_source1, BaseHostDataSource& data_source2);
+
+    /// @brief This test verifies that HostMgr adds the reservations to any
+    /// data source.
+    ///
+    /// The reservations are added to the external database (alternate sources)
+    /// by default but the primary source may be changed on demand too.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testAdd(BaseHostDataSource& data_source1,
+                 BaseHostDataSource& data_source2);
+
+    /// @brief This test verifies that HostMgr deletes the reservations by
+    /// the subnet ID and subnet address.
+    ///
+    /// The reservations are deleted from the external database (alternate
+    /// sources) only by default but the primary source may be changed on
+    /// demand too.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testDeleteByIDAndAddress(BaseHostDataSource& data_source1,
+                                  BaseHostDataSource& data_source2);
+
+    /// @brief This test verifies that HostMgr deletes the IPv4 reservations by
+    /// the subnet ID and identifier.
+    ///
+    /// The reservations are deleted from the external database (alternate
+    /// sources) only by default but the primary source may be changed on
+    /// demand too.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testDelete4ByIDAndIdentifier(BaseHostDataSource& data_source1,
+                                      BaseHostDataSource& data_source2);
+
+    /// @brief This test verifies that HostMgr deletes the IPv6 reservations by
+    /// the subnet ID and identifier.
+    ///
+    /// The reservations are deleted from the external database (alternate
+    /// sources) only by default but the primary source may be changed on
+    /// demand too.
+    ///
+    /// @param data_source1 Host data source to which first reservation is
+    /// inserted.
+    /// @param data_source2 Host data source to which second reservation is
+    /// inserted.
+    void testDelete6ByIDAndIdentifier(BaseHostDataSource& data_source1,
+                                      BaseHostDataSource& data_source2);
+
+    /// @brief Utility function that returns true if a given data source
+    /// is primary (it isn't an alternate source).
+    /// @param data_source  Host data source to check.
+    /// @return True if the data source is primary. Otherwise, false.
+    bool isPrimaryDataSource(const BaseHostDataSource& data_source) const;
+
     /// @brief HW addresses to be used by the tests.
     std::vector<HWAddrPtr> hwaddrs_;
     /// @brief DUIDs to be used by the tests.
