@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -62,14 +62,16 @@ public:
                      const uint8_t* salt_data, size_t salt_length) :
         algorithm_(algorithm), iterations_(iterations),
         salt_data_(NULL), salt_length_(salt_length),
-        digest_(DEFAULT_DIGEST_LENGTH), obuf_(Name::MAX_WIRE)
-    {
+        digest_(DEFAULT_DIGEST_LENGTH), obuf_(Name::MAX_WIRE) {
         if (algorithm_ != NSEC3_HASH_SHA1) {
             isc_throw(UnknownNSEC3HashAlgorithm, "Unknown NSEC3 algorithm: " <<
                       static_cast<unsigned int>(algorithm_));
         }
 
         if (salt_length > 0) {
+            if (salt_data == NULL) {
+                isc_throw(isc::BadValue, "salt data is NULL");
+            }
             salt_data_ = static_cast<uint8_t*>(std::malloc(salt_length));
             if (salt_data_ == NULL) {
                 throw std::bad_alloc();
