@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,12 +26,10 @@ namespace hooks {
 
 // Constructor
 CalloutManager::CalloutManager(int num_libraries)
-    : server_hooks_(ServerHooks::getServerHooks()),
-      current_hook_(-1), current_library_(-1),
+    : server_hooks_(ServerHooks::getServerHooks()), current_library_(-1),
       hook_vector_(ServerHooks::getServerHooks().getCount()),
-      library_handle_(this), pre_library_handle_(this, 0),
-      post_library_handle_(this, INT_MAX), num_libraries_(num_libraries)
-{
+      library_handle_(*this), pre_library_handle_(*this, 0),
+      post_library_handle_(*this, INT_MAX), num_libraries_(num_libraries) {
     if (num_libraries < 0) {
         isc_throw(isc::BadValue, "number of libraries passed to the "
                   "CalloutManager must be >= 0");
@@ -42,7 +40,6 @@ CalloutManager::CalloutManager(int num_libraries)
 // (n is the number of libraries), 0 (pre-user library callouts), or INT_MAX
 // (post-user library callouts).  It can also be -1 to indicate an invalid
 // value.
-
 void
 CalloutManager::checkLibraryIndex(int library_index) const {
     if (((library_index >= -1) && (library_index <= num_libraries_)) ||
@@ -56,15 +53,16 @@ CalloutManager::checkLibraryIndex(int library_index) const {
 }
 
 // Register a callout for the current library.
-
 void
-CalloutManager::registerCallout(const std::string& name, CalloutPtr callout) {
+CalloutManager::registerCallout(const std::string& name,
+                                CalloutPtr callout,
+                                int library_index) {
     // Note the registration.
     LOG_DEBUG(callouts_logger, HOOKS_DBG_CALLS, HOOKS_CALLOUT_REGISTRATION)
-        .arg(current_library_).arg(name);
+        .arg(library_index).arg(name);
 
     // Sanity check that the current library index is set to a valid value.
-    checkLibraryIndex(current_library_);
+    checkLibraryIndex(library_index);
 
     // New hooks could have been registered since the manager was constructed.
     ensureHookLibsVectorSize();
@@ -78,10 +76,10 @@ CalloutManager::registerCallout(const std::string& name, CalloutPtr callout) {
     // the present index.
     for (CalloutVector::iterator i = hook_vector_[hook_index].begin();
          i != hook_vector_[hook_index].end(); ++i) {
-        if (i->first > current_library_) {
+        if (i->first > library_index) {
             // Found an element whose library index number is greater than the
             // current index, so insert the new element ahead of this one.
-            hook_vector_[hook_index].insert(i, make_pair(current_library_,
+            hook_vector_[hook_index].insert(i, make_pair(library_index,
                                                          callout));
             return;
         }
@@ -90,11 +88,10 @@ CalloutManager::registerCallout(const std::string& name, CalloutPtr callout) {
     // Reached the end of the vector, so there is no element in the (possibly
     // empty) set of callouts with a library index greater than the current
     // library index.  Inset the callout at the end of the list.
-    hook_vector_[hook_index].push_back(make_pair(current_library_, callout));
+    hook_vector_[hook_index].push_back(make_pair(library_index, callout));
 }
 
 // Check if callouts are present for a given hook index.
-
 bool
 CalloutManager::calloutsPresent(int hook_index) const {
     // Validate the hook index.
@@ -127,12 +124,9 @@ CalloutManager::commandHandlersPresent(const std::string& command_name) const {
     return (false);
 }
 
-
 // Call all the callouts for a given hook.
-
 void
 CalloutManager::callCallouts(int hook_index, CalloutHandle& callout_handle) {
-
     // Clear the "skip" flag so we don't carry state from a previous call.
     // This is done regardless of whether callouts are present to avoid passing
     // any state from the previous call of callCallouts().
@@ -144,14 +138,7 @@ CalloutManager::callCallouts(int hook_index, CalloutHandle& callout_handle) {
 
         // Set the current hook index.  This is used should a callout wish to
         // determine to what hook it is attached.
-        current_hook_ = hook_index;
-
-        // Duplicate the callout vector for this hook and work through that.
-        // This step is needed because we allow dynamic registration and
-        // deregistration of callouts.  If a callout attached to a hook modified
-        // the list of callouts on that hook, the underlying CalloutVector would
-        // change and potentially affect the iteration through that vector.
-        CalloutVector callouts(hook_vector_[hook_index]);
+        callout_handle.setCurrentHook(hook_index);
 
         // This object will be used to measure execution time of each callout
         // and the total time spent in callouts for this hook point.
@@ -159,15 +146,16 @@ CalloutManager::callCallouts(int hook_index, CalloutHandle& callout_handle) {
 
         // Mark that the callouts begin for the hook.
         LOG_DEBUG(callouts_logger, HOOKS_DBG_CALLS, HOOKS_CALLOUTS_BEGIN)
-            .arg(server_hooks_.getName(current_hook_));
+            .arg(server_hooks_.getName(callout_handle.getCurrentHook()));
 
         // Call all the callouts.
-        for (CalloutVector::const_iterator i = callouts.begin();
-             i != callouts.end(); ++i) {
-            // In case the callout tries to register or deregister a callout,
-            // set the current library index to the index associated with the
-            // library that registered the callout being called.
-            current_library_ = i->first;
+        for (CalloutVector::const_iterator i = hook_vector_[hook_index].begin();
+             i != hook_vector_[hook_index].end(); ++i) {
+            // In case the callout requires access to the context associated
+            // with the library, set the current library index to the index
+            // associated with the library that registered the callout being
+            // called.
+            callout_handle.setCurrentLibrary(i->first);
 
             // Call the callout
             try {
@@ -176,14 +164,15 @@ CalloutManager::callCallouts(int hook_index, CalloutHandle& callout_handle) {
                 stopwatch.stop();
                 if (status == 0) {
                     LOG_DEBUG(callouts_logger, HOOKS_DBG_EXTENDED_CALLS,
-                              HOOKS_CALLOUT_CALLED).arg(current_library_)
-                        .arg(server_hooks_.getName(current_hook_))
+                              HOOKS_CALLOUT_CALLED)
+                        .arg(callout_handle.getCurrentLibrary())
+                        .arg(server_hooks_.getName(callout_handle.getCurrentHook()))
                         .arg(PointerConverter(i->second).dlsymPtr())
                         .arg(stopwatch.logFormatLastDuration());
                 } else {
                     LOG_ERROR(callouts_logger, HOOKS_CALLOUT_ERROR)
-                        .arg(current_library_)
-                        .arg(server_hooks_.getName(current_hook_))
+                        .arg(server_hooks_.getName(callout_handle.getCurrentHook()))
+                        .arg(callout_handle.getCurrentLibrary())
                         .arg(PointerConverter(i->second).dlsymPtr())
                         .arg(stopwatch.logFormatLastDuration());
                 }
@@ -191,27 +180,39 @@ CalloutManager::callCallouts(int hook_index, CalloutHandle& callout_handle) {
                 // If an exception occurred, the stopwatch.stop() hasn't been
                 // called, so we have to call it here.
                 stopwatch.stop();
-                // Any exception, not just ones based on isc::Exception
+                // Any exception, just ones based on isc::Exception
                 LOG_ERROR(callouts_logger, HOOKS_CALLOUT_EXCEPTION)
-                    .arg(current_library_)
-                    .arg(server_hooks_.getName(current_hook_))
+                    .arg(server_hooks_.getName(callout_handle.getCurrentHook()))
+                    .arg(callout_handle.getCurrentLibrary())
                     .arg(PointerConverter(i->second).dlsymPtr())
                     .arg(e.what())
                     .arg(stopwatch.logFormatLastDuration());
+                callout_handle.setStatus(CalloutHandle::NEXT_STEP_DROP);
+            } catch (...) {
+                // If an exception occurred, the stopwatch.stop() hasn't been
+                // called, so we have to call it here.
+                stopwatch.stop();
+                // Any exception, not just ones based on isc::Exception
+                LOG_ERROR(callouts_logger, HOOKS_CALLOUT_EXCEPTION)
+                    .arg(server_hooks_.getName(callout_handle.getCurrentHook()))
+                    .arg(callout_handle.getCurrentLibrary())
+                    .arg(PointerConverter(i->second).dlsymPtr())
+                    .arg("Unspecified exception")
+                    .arg(stopwatch.logFormatLastDuration());
+                callout_handle.setStatus(CalloutHandle::NEXT_STEP_DROP);
             }
-
         }
 
         // Mark end of callout execution. Include the total execution
         // time for callouts.
         LOG_DEBUG(callouts_logger, HOOKS_DBG_CALLS, HOOKS_CALLOUTS_COMPLETE)
-            .arg(server_hooks_.getName(current_hook_))
+            .arg(server_hooks_.getName(callout_handle.getCurrentHook()))
             .arg(stopwatch.logFormatTotalDuration());
 
         // Reset the current hook and library indexes to an invalid value to
         // catch any programming errors.
-        current_hook_ = -1;
-        current_library_ = -1;
+        callout_handle.setCurrentHook(-1);
+        callout_handle.setCurrentLibrary(-1);
     }
 }
 
@@ -222,19 +223,17 @@ CalloutManager::callCommandHandlers(const std::string& command_name,
     // This will throw an exception if the hook point doesn't exist.
     // The caller should check if the hook point exists by calling
     // commandHandlersPresent.
-    int index = ServerHooks::getServerHooks().getIndex(
-                    ServerHooks::commandToHookName(command_name));
+    int index = ServerHooks::getServerHooks().getIndex(ServerHooks::commandToHookName(command_name));
     // Call the handlers for this command.
     callCallouts(index, callout_handle);
 }
 
-
 // Deregister a callout registered by the current library on a particular hook.
-
 bool
-CalloutManager::deregisterCallout(const std::string& name, CalloutPtr callout) {
+CalloutManager::deregisterCallout(const std::string& name, CalloutPtr callout,
+                                  int library_index) {
     // Sanity check that the current library index is set to a valid value.
-    checkLibraryIndex(current_library_);
+    checkLibraryIndex(library_index);
 
     // New hooks could have been registered since the manager was constructed.
     ensureHookLibsVectorSize();
@@ -250,7 +249,7 @@ CalloutManager::deregisterCallout(const std::string& name, CalloutPtr callout) {
 
     /// Construct a CalloutEntry matching the current library and the callout
     /// we want to remove.
-    CalloutEntry target(current_library_, callout);
+    CalloutEntry target(library_index, callout);
 
     /// To decide if any entries were removed, we'll record the initial size
     /// of the callout vector for the hook, and compare it with the size after
@@ -268,25 +267,24 @@ CalloutManager::deregisterCallout(const std::string& name, CalloutPtr callout) {
     // all the matching elements.
     hook_vector_[hook_index].erase(remove_if(hook_vector_[hook_index].begin(),
                                              hook_vector_[hook_index].end(),
-                                             bind1st(equal_to<CalloutEntry>(),
-                                                     target)),
+                                             [&target] (CalloutEntry x) {
+                                                 return (x == target); }),
                                    hook_vector_[hook_index].end());
 
     // Return an indication of whether anything was removed.
     bool removed = initial_size != hook_vector_[hook_index].size();
     if (removed) {
         LOG_DEBUG(callouts_logger, HOOKS_DBG_EXTENDED_CALLS,
-                  HOOKS_CALLOUT_DEREGISTERED).arg(current_library_).arg(name);
+                  HOOKS_CALLOUT_DEREGISTERED).arg(library_index).arg(name);
     }
 
     return (removed);
 }
 
 // Deregister all callouts on a given hook.
-
 bool
-CalloutManager::deregisterAllCallouts(const std::string& name) {
-
+CalloutManager::deregisterAllCallouts(const std::string& name,
+                                      int library_index) {
     // New hooks could have been registered since the manager was constructed.
     ensureHookLibsVectorSize();
 
@@ -296,7 +294,7 @@ CalloutManager::deregisterAllCallouts(const std::string& name) {
 
     /// Construct a CalloutEntry matching the current library (the callout
     /// pointer is NULL as we are not checking that).
-    CalloutEntry target(current_library_, static_cast<CalloutPtr>(0));
+    CalloutEntry target(library_index, static_cast<CalloutPtr>(0));
 
     /// To decide if any entries were removed, we'll record the initial size
     /// of the callout vector for the hook, and compare it with the size after
@@ -306,16 +304,16 @@ CalloutManager::deregisterAllCallouts(const std::string& name) {
     // Remove all callouts matching this library.
     hook_vector_[hook_index].erase(remove_if(hook_vector_[hook_index].begin(),
                                              hook_vector_[hook_index].end(),
-                                             bind1st(CalloutLibraryEqual(),
-                                                     target)),
+                                             [&target] (CalloutEntry x) {
+                                                 return (x.first == target.first);
+                                             }),
                                    hook_vector_[hook_index].end());
 
     // Return an indication of whether anything was removed.
     bool removed = initial_size != hook_vector_[hook_index].size();
     if (removed) {
         LOG_DEBUG(callouts_logger, HOOKS_DBG_EXTENDED_CALLS,
-                  HOOKS_ALL_CALLOUTS_DEREGISTERED).arg(current_library_)
-                                                .arg(name);
+                  HOOKS_ALL_CALLOUTS_DEREGISTERED).arg(library_index).arg(name);
     }
 
     return (removed);
@@ -323,7 +321,6 @@ CalloutManager::deregisterAllCallouts(const std::string& name) {
 
 void
 CalloutManager::registerCommandHook(const std::string& command_name) {
-
     // New hooks could have been registered since the manager was constructed.
     ensureHookLibsVectorSize();
 
