@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,7 @@
 
 #include <asiolink/asio_wrapper.h>
 #include <asiolink/interval_timer.h>
+#include <asiolink/io_service_thread_pool.h>
 #include <asiolink/tls_socket.h>
 #include <http/client.h>
 #include <http/http_log.h>
@@ -819,7 +820,7 @@ private:
         /// @param tls_context server TLS context of this destination
         /// @param max_connections maximum number of concurrent connections
         /// allowed for in the list URL
-        Destination(Url url, TlsContextPtr tls_context, size_t max_connections)
+        Destination(Url const& url, TlsContextPtr tls_context, size_t max_connections)
             : url_(url), tls_context_(tls_context),
               max_connections_(max_connections), connections_(), queue_(),
               last_queue_warn_time_(min_date_time), last_queue_size_(0) {
@@ -989,7 +990,7 @@ private:
         /// to be growing it will emit a warning log.
         ///
         /// @param desc RequestDescriptor to queue.
-        void pushRequest(RequestDescriptor desc) {
+        void pushRequest(RequestDescriptor const& desc) {
             queue_.push(desc);
             size_t size = queue_.size();
             // If the queue size is larger than the threshold and growing, issue a
@@ -1789,14 +1790,14 @@ public:
             // Create our own private IOService.
             thread_io_service_.reset(new IOService());
 
-            // Create the thread pool.
-            thread_pool_.reset(new HttpThreadPool(thread_io_service_, thread_pool_size_,
-                                                  defer_thread_start));
-
             // Create the connection pool. Note that we use the thread_pool_size
             // as the maximum connections per URL value.
             conn_pool_.reset(new ConnectionPool(*thread_io_service_, thread_pool_size_));
 
+            // Create the thread pool.
+            thread_pool_.reset(new IoServiceThreadPool(thread_io_service_, thread_pool_size_,
+                                                       defer_thread_start));
+
             LOG_DEBUG(http_logger, isc::log::DBGLVL_TRACE_BASIC, HTTP_CLIENT_MT_STARTED)
                      .arg(thread_pool_size_);
         } else {
@@ -1943,17 +1944,15 @@ private:
 
     /// @brief Pool of threads used to service connections in multi-threaded
     /// mode.
-    HttpThreadPoolPtr thread_pool_;
+    IoServiceThreadPoolPtr thread_pool_;
 };
 
-HttpClient::HttpClient(IOService& io_service, size_t thread_pool_size,
-                       bool defer_thread_start /* = false */) {
-    if (thread_pool_size > 0) {
-        if (!MultiThreadingMgr::instance().getMode()) {
-            isc_throw(InvalidOperation,
-                      "HttpClient thread_pool_size must be zero"
-                      "when Kea core multi-threading is disabled");
-        }
+HttpClient::HttpClient(IOService& io_service, bool multi_threading_enabled,
+                       size_t thread_pool_size, bool defer_thread_start) {
+    if (!multi_threading_enabled && thread_pool_size) {
+        isc_throw(InvalidOperation,
+                  "HttpClient thread_pool_size must be zero "
+                  "when Kea core multi-threading is disabled");
     }
 
     impl_.reset(new HttpClientImpl(io_service, thread_pool_size,
