@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -1064,7 +1064,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
@@ -1122,7 +1122,7 @@ public:
         ASSERT_NO_THROW(listener2_.start());
 
         // Create the client. It will be communicating with the two servers.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URLs on which the servers are available.
         Url url1("http://127.0.0.1:18123");
@@ -1179,7 +1179,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL on which the server is available.
         Url url("http://127.0.0.1:18123");
@@ -1239,7 +1239,7 @@ public:
         ASSERT_NO_THROW(listener3_.start());
 
         // Create the client that will communicate with this server.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of this server.
         Url url("http://127.0.0.1:18125");
@@ -1294,7 +1294,7 @@ public:
     /// server is unreachable.
     void testUnreachable () {
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server. This server is down.
         Url url("http://127.0.0.1:18123");
@@ -1323,7 +1323,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -1360,7 +1360,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -1425,7 +1425,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -1499,7 +1499,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -1582,7 +1582,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
@@ -1677,7 +1677,7 @@ public:
         ASSERT_NO_THROW(listener_.start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
