@@ -139,7 +139,7 @@ PgSqlConnection::getVersion(const ParameterMap& parameters) {
     PgSqlConnection conn(parameters);
 
     // Open the database.
-    conn.openDatabase();
+    conn.openDatabaseInternal(false);
 
     const char* version_sql =  "SELECT version, minor FROM schema_version;";
     PgSqlResult r(PQexec(conn.conn_, version_sql));
@@ -182,6 +182,11 @@ PgSqlConnection::prepareStatements(const PgSqlTaggedStatement* start_statement,
 
 std::string
 PgSqlConnection::getConnParameters() {
+    return (getConnParametersInternal(false));
+}
+
+std::string
+PgSqlConnection::getConnParametersInternal(bool logging) {
     string dbconnparameters;
     string shost = "localhost";
     try {
@@ -249,11 +254,20 @@ PgSqlConnection::getConnParameters() {
         isc_throw(DbInvalidTimeout, ex.what());
     }
 
-    // Append timeouts.
+    // Append connection timeout.
     std::ostringstream oss;
     oss << " connect_timeout = " << connect_timeout;
+
     if (tcp_user_timeout > 0) {
+// tcp_user_timeout parameter is a PostgreSQL 12+ feature.
+#ifdef HAVE_PGSQL_TCP_USER_TIMEOUT
         oss << " tcp_user_timeout = " << tcp_user_timeout * 1000;
+        static_cast<void>(logging);
+#else
+        if (logging) {
+            DB_LOG_WARN(PGSQL_TCP_USER_TIMEOUT_UNSUPPORTED).arg();
+        }
+#endif
     }
     dbconnparameters += oss.str();
 
@@ -262,7 +276,12 @@ PgSqlConnection::getConnParameters() {
 
 void
 PgSqlConnection::openDatabase() {
-    std::string dbconnparameters = getConnParameters();
+    openDatabaseInternal(true);
+}
+
+void
+PgSqlConnection::openDatabaseInternal(bool logging) {
+    std::string dbconnparameters = getConnParametersInternal(logging);
     // Connect to Postgres, saving the low level connection pointer
     // in the holder object
     PGconn* new_conn = PQconnectdb(dbconnparameters.c_str());
