@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,11 +6,14 @@
 
 #include <config.h>
 #include <cc/command_interpreter.h>
-#include <cfgrpt/config_report.h>
+#include <process/cfgrpt/config_report.h>
+#include <cryptolink/crypto_hash.h>
 #include <exceptions/exceptions.h>
+#include <config/command_mgr.h>
 #include <hooks/hooks_manager.h>
 #include <log/logger.h>
 #include <log/logger_support.h>
+#include <util/encode/hex.h>
 #include <process/daemon.h>
 #include <process/d_log.h>
 #include <process/d_controller.h>
@@ -18,13 +21,15 @@
 #include <kea_version.h>
 #include <functional>
 #include <sstream>
+#include <string>
 #include <unistd.h>
 #include <signal.h>
 
 using namespace isc::asiolink;
-using namespace isc::data;
 using namespace isc::config;
+using namespace isc::data;
 using namespace isc::hooks;
+using namespace isc::util;
 namespace ph = std::placeholders;
 
 namespace isc {
@@ -410,7 +415,7 @@ DControllerBase::configFromFile() {
         process_->getCfgMgr()->getContext()->applyLoggingCfg();
 
         // build an error result
-        ConstElementPtr error = createAnswer(COMMAND_ERROR,
+        ConstElementPtr error = createAnswer(CONTROL_RESULT_ERROR,
                  std::string("Configuration parsing failed: ") + ex.what());
         return (error);
     }
@@ -447,9 +452,21 @@ DControllerBase::checkConfig(ConstElementPtr new_config) {
 ConstElementPtr
 DControllerBase::configGetHandler(const std::string&,
                                   ConstElementPtr /*args*/) {
-    ConstElementPtr config = process_->getCfgMgr()->getContext()->toElement();
+    ElementPtr config = process_->getCfgMgr()->getContext()->toElement();
+    std::string hash = BaseCommandMgr::getHash(config);
+    config->set("hash", Element::create(hash));
 
-    return (createAnswer(COMMAND_SUCCESS, config));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, config));
+}
+
+ConstElementPtr
+DControllerBase::configHashGetHandler(const std::string&,
+                                      ConstElementPtr /*args*/) {
+    ConstElementPtr config = process_->getCfgMgr()->getContext()->toElement();
+    std::string hash = BaseCommandMgr::getHash(config);
+    ElementPtr params = Element::createMap();
+    params->set("hash", Element::create(hash));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, params));
 }
 
 ConstElementPtr
@@ -459,12 +476,12 @@ DControllerBase::configWriteHandler(const std::string&,
 
     if (args) {
         if (args->getType() != Element::map) {
-            return (createAnswer(COMMAND_ERROR, "Argument must be a map"));
+            return (createAnswer(CONTROL_RESULT_ERROR, "Argument must be a map"));
         }
         ConstElementPtr filename_param = args->get("filename");
         if (filename_param) {
             if (filename_param->getType() != Element::string) {
-                return (createAnswer(COMMAND_ERROR,
+                return (createAnswer(CONTROL_RESULT_ERROR,
                                      "passed parameter 'filename' "
                                      "is not a string"));
             }
@@ -477,13 +494,12 @@ DControllerBase::configWriteHandler(const std::string&,
         // whatever we remember
         filename = getConfigFile();
         if (filename.empty()) {
-            return (createAnswer(COMMAND_ERROR,
+            return (createAnswer(CONTROL_RESULT_ERROR,
                                  "Unable to determine filename."
                                  "Please specify filename explicitly."));
         }
     }
 
-
     // Ok, it's time to write the file.
     size_t size = 0;
     ElementPtr cfg = process_->getCfgMgr()->getContext()->toElement();
@@ -491,12 +507,12 @@ DControllerBase::configWriteHandler(const std::string&,
     try {
         size = writeConfigFile(filename, cfg);
     } catch (const isc::Exception& ex) {
-        return (createAnswer(COMMAND_ERROR,
+        return (createAnswer(CONTROL_RESULT_ERROR,
                              std::string("Error during write-config:")
                              + ex.what()));
     }
     if (size == 0) {
-        return (createAnswer(COMMAND_ERROR,
+        return (createAnswer(CONTROL_RESULT_ERROR,
                              "Error writing configuration to " + filename));
     }
 
@@ -533,7 +549,7 @@ DControllerBase::handleOtherObjects(ConstElementPtr args) {
 
 ConstElementPtr
 DControllerBase::configTestHandler(const std::string&, ConstElementPtr args) {
-    const int status_code = COMMAND_ERROR; // 1 indicates an error
+    const int status_code = CONTROL_RESULT_ERROR; // 1 indicates an error
     ConstElementPtr module_config;
     std::string app_name = getAppName();
     std::string message;
@@ -566,7 +582,6 @@ DControllerBase::configTestHandler(const std::string&, ConstElementPtr args) {
         return (result);
     }
 
-
     // We are starting the configuration process so we should remove any
     // staging configuration that has been created during previous
     // configuration attempts.
@@ -585,7 +600,7 @@ DControllerBase::configReloadHandler(const std::string&, ConstElementPtr) {
 
 ConstElementPtr
 DControllerBase::configSetHandler(const std::string&, ConstElementPtr args) {
-    const int status_code = COMMAND_ERROR; // 1 indicates an error
+    const int status_code = CONTROL_RESULT_ERROR; // 1 indicates an error
     ConstElementPtr module_config;
     std::string app_name = getAppName();
     std::string message;
@@ -643,7 +658,7 @@ DControllerBase::configSetHandler(const std::string&, ConstElementPtr args) {
         process_->getCfgMgr()->getContext()->applyLoggingCfg();
 
         // build an error result
-        ConstElementPtr error = createAnswer(COMMAND_ERROR,
+        ConstElementPtr error = createAnswer(CONTROL_RESULT_ERROR,
                  std::string("Configuration parsing failed: ") + ex.what());
         return (error);
     }
@@ -655,7 +670,7 @@ DControllerBase::serverTagGetHandler(const std::string&, ConstElementPtr) {
     ElementPtr response = Element::createMap();
     response->set("server-tag", Element::create(tag));
 
-    return (createAnswer(COMMAND_SUCCESS, response));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, response));
 }
 
 ConstElementPtr
@@ -675,7 +690,7 @@ DControllerBase::statusGetHandler(const std::string&, ConstElementPtr) {
         status->set("reload", Element::create(reload.total_seconds()));
     }
 
-    return (createAnswer(COMMAND_SUCCESS, status));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, status));
 }
 
 ConstElementPtr
@@ -686,13 +701,13 @@ DControllerBase::versionGetHandler(const std::string&, ConstElementPtr) {
     ElementPtr extended = Element::create(getVersion(true));
     ElementPtr arguments = Element::createMap();
     arguments->set("extended", extended);
-    answer = createAnswer(COMMAND_SUCCESS, getVersion(false), arguments);
+    answer = createAnswer(CONTROL_RESULT_SUCCESS, getVersion(false), arguments);
     return (answer);
 }
 
 ConstElementPtr
 DControllerBase::buildReportHandler(const std::string&, ConstElementPtr) {
-    return (createAnswer(COMMAND_SUCCESS, isc::detail::getConfigReport()));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, isc::detail::getConfigReport()));
 }
 
 ConstElementPtr
@@ -733,7 +748,7 @@ DControllerBase::shutdownProcess(ConstElementPtr args) {
     // Not really a failure, but this condition is worth noting. In reality
     // it should be pretty hard to cause this.
     LOG_WARN(dctl_logger, DCTL_NOT_RUNNING).arg(app_name_);
-    return (createAnswer(COMMAND_SUCCESS, "Process has not been initialized"));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, "Process has not been initialized"));
 }
 
 void
@@ -820,10 +835,10 @@ DControllerBase::~DControllerBase() {
         }
         LOG_ERROR(dctl_logger, DCTL_UNLOAD_LIBRARIES_ERROR).arg(msg);
     }
-}
 
-// Refer to config_report so it will be embedded in the binary
-const char* const* d_config_report = isc::detail::config_report;
+    io_signal_set_.reset();
+    getIOService()->poll();
+}
 
 std::string
 DControllerBase::getVersion(bool extended) {
