@@ -1,16 +1,18 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <cc/data.h>
 #include <process/log_parser.h>
 #include <process/process_messages.h>
 #include <exceptions/exceptions.h>
 #include <log/logger_support.h>
 #include <process/d_log.h>
+#include <testutils/gtest_utils.h>
 #include <testutils/io_utils.h>
 
 #include <gtest/gtest.h>
@@ -130,7 +132,7 @@ TEST_F(LoggingTest, parsingConsoleOutput) {
     EXPECT_TRUE(storage->getLoggingInfo()[0].destinations_[0].flush_);
 }
 
-// Checks if the LogConfigParser class fails when the configuration
+// Check that LogConfigParser can parse configuration that
 // lacks a severity entry.
 TEST_F(LoggingTest, parsingNoSeverity) {
 
@@ -158,7 +160,23 @@ TEST_F(LoggingTest, parsingNoSeverity) {
     ConstElementPtr config = Element::fromJSON(config_txt);
     config = config->get("loggers");
 
-    EXPECT_THROW(parser.parseConfiguration(config), BadValue);
+    // No exception should be thrown.
+    EXPECT_NO_THROW_LOG(parser.parseConfiguration(config));
+
+    // Entries should be the ones set.
+    ASSERT_EQ(1, storage->getLoggingInfo().size());
+    LoggingInfo const& logging_info(storage->getLoggingInfo()[0]);
+    EXPECT_EQ("kea", logging_info.name_);
+    EXPECT_EQ(99, logging_info.debuglevel_);
+    ASSERT_EQ(1, logging_info.destinations_.size());
+    EXPECT_EQ("stdout" , logging_info.destinations_[0].output_);
+    EXPECT_TRUE(logging_info.destinations_[0].flush_);
+
+    // Severity should default to DEFAULT.
+    EXPECT_EQ(isc::log::DEFAULT, logging_info.severity_);
+
+    // Pattern should default to empty string.
+    EXPECT_TRUE(logging_info.destinations_[0].pattern_.empty());
 }
 
 // Checks if the LogConfigParser class is able to transform JSON structures
@@ -313,7 +331,7 @@ TEST_F(LoggingTest, multipleLoggingDestinations) {
 // Verifies that log rotation occurs when configured.  We do not
 // worry about contents of the log files, only that rotation occurs.
 // Such details are tested in lib/log.  This test verifies that
-// we can correcty configure logging such that rotation occurs as
+// we can correctly configure logging such that rotation occurs as
 // expected.
 TEST_F(LoggingTest, logRotate) {
     wipeFiles();
@@ -373,9 +391,10 @@ TEST_F(LoggingTest, logRotate) {
     wipeFiles();
 }
 
-// Verifies that a valid output option,'pattern' paress correctly.
+// Verifies that a valid output option,'pattern' parses correctly.
 TEST_F(LoggingTest, validPattern) {
 
+    // Note the backslash must be doubled in the pattern definition.
     const char* config_txt =
     "{ \"loggers\": ["
     "    {"
@@ -383,7 +402,7 @@ TEST_F(LoggingTest, validPattern) {
     "        \"output_options\": ["
     "            {"
     "                \"output\": \"stdout\","
-    "                \"pattern\": \"mylog %m\n\""
+    "                \"pattern\": \"mylog %m\\n\""
     "            }"
     "        ],"
     "        \"severity\": \"INFO\""
@@ -451,6 +470,54 @@ TEST_F(LoggingTest, emptyPattern) {
     EXPECT_TRUE(storage->getLoggingInfo()[0].destinations_[0].pattern_.empty());
 }
 
+void testMaxSize(uint64_t maxsize_candidate, uint64_t expected_maxsize) {
+    std::string const logger(R"(
+    {
+      "loggers": [
+        {
+
+          "debuglevel": 99,
+          "name": "kea",
+          "output_options": [
+            {
+              "output": "kea.test.log",
+              "flush": true,
+              "maxsize": )" + std::to_string(maxsize_candidate) + R"(,
+              "maxver": 2
+            }
+          ],
+          "severity": "DEBUG"
+        }
+      ]
+    }
+    )");
+
+    // Create our server config container.
+    ConfigPtr server_cfg(boost::make_shared<ConfigBase>());
+
+    // LogConfigParser expects a list of loggers, so parse
+    // the JSON text and extract the "loggers" element from it
+    ConstElementPtr config(Element::fromJSON(logger));
+    config = config->get("loggers");
+
+    // Parse the config and then apply it.
+    LogConfigParser parser(server_cfg);
+    ASSERT_NO_THROW(parser.parseConfiguration(config));
+    ASSERT_NO_THROW(server_cfg->applyLoggingCfg());
+
+    EXPECT_EQ(server_cfg->getLoggingInfo()[0].destinations_[0].maxsize_,
+              expected_maxsize);
+}
+
+// Test that maxsize can be configured with high values.
+TEST_F(LoggingTest, maxsize) {
+    testMaxSize(TEST_MAX_SIZE, TEST_MAX_SIZE);
+    testMaxSize(std::numeric_limits<int32_t>::max(), std::numeric_limits<int32_t>::max());
+    testMaxSize(std::numeric_limits<uint32_t>::max(), std::numeric_limits<uint32_t>::max());
+    testMaxSize(1000LL * std::numeric_limits<int32_t>::max(), 1000LL * std::numeric_limits<int32_t>::max());
+    testMaxSize(1000000LL * std::numeric_limits<int32_t>::max(), 1000000LL * std::numeric_limits<int32_t>::max());
+}
+
 /// @todo Add tests for malformed logging configuration
 
 /// @todo There is no easy way to test applyConfiguration() and defaultLogging().
@@ -460,4 +527,4 @@ TEST_F(LoggingTest, emptyPattern) {
 /// check if the file is indeed created or configure stdout destination, then
 /// swap console file descriptors and check that messages are really logged.
 
-};
+}  // namespace
