@@ -1,4 +1,4 @@
-// Copyright (C) 2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #include <config.h>
 
 #include <testutils/log_utils.h>
+#include <cstdlib>
 #include <iostream>
 
 namespace isc {
@@ -32,6 +33,12 @@ LogContentTest::LogContentTest()
     spec.addOutputOption(option);
     LoggerManager manager;
     manager.process(spec);
+
+    // Overwrite the verbose_ default is the KEA_LOG_CHECK_VERBOSE
+    // environment variable exists.
+    if (getenv(KEA_LOG_CHECK_VERBOSE)) {
+        verbose_ = true;
+    }
 }
 
 LogContentTest:: ~LogContentTest() {
@@ -50,8 +57,8 @@ bool LogContentTest::checkFile() {
     while (getline(file, line) && (i != exp_strings_.size())) {
         exp_line = exp_strings_[i];
         if (verbose_) {
-            cout << "Read line  :" << line << endl;
-            cout << "Looking for:" << exp_line << endl;
+            cout << "Read line  : " << line << endl;
+            cout << "Looking for: " << exp_line << endl;
         }
         i++;
         if (string::npos == line.find(exp_line)) {
@@ -74,6 +81,35 @@ bool LogContentTest::checkFile() {
     return (true);
 }
 
+size_t LogContentTest::countFile(const string& exp_string) {
+    ifstream file(LOG_FILE);
+    EXPECT_TRUE(file.is_open());
+    string line;
+    size_t cnt = 0;
+
+    using namespace std;
+
+    if (verbose_) {
+        cout << "Looking for:" << exp_string << endl;
+    }
+    while (getline(file, line)) {
+        if (verbose_) {
+            cout << "Read line  :" << line << endl;
+        }
+        if (line.find(exp_string) != string::npos) {
+            ++cnt;
+        }
+    }
+
+    file.close();
+
+    if (verbose_) {
+        cout << "Final count: " << cnt << endl;
+    }
+
+    return (cnt);
+}
+
 void LogContentTest::remFile() {
     static_cast<void>(remove(LOG_FILE));
 }
@@ -83,9 +119,13 @@ void LogContentTest::addString(const string& new_string) {
 }
 
 // Set up the name of the LOG_FILE for use in checking
-// the debug statements
-const char *LogContentTest::LOG_FILE = "test.log";
+// the debug statements.
+// Must not be the same file name used by test shell scripts.
+const char* LogContentTest::LOG_FILE = "logtest.log";
 
-}; // end of isc::dhcp::test namespace
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+// The environment variable to overwrite the verbose_ default value.
+const char* LogContentTest::KEA_LOG_CHECK_VERBOSE = "KEA_LOG_CHECK_VERBOSE";
+
+} // end of isc::dhcp::test namespace
+} // end of isc::dhcp namespace
+} // end of isc namespace
