@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -452,7 +452,7 @@ TEST(StringUtilTest, decodeFormattedHexString) {
     EXPECT_THROW(decodeFormattedHexString("0a ", decoded),
                  isc::BadValue);
     // '0x' prefix and spaces.
-    EXPECT_THROW(decodeFormattedHexString("x01 02", decoded),
+    EXPECT_THROW(decodeFormattedHexString("0x01 02", decoded),
                  isc::BadValue);
     // '0x' prefix and colons.
     EXPECT_THROW(decodeFormattedHexString("0x01:02", decoded),
@@ -509,15 +509,21 @@ void sanitizeStringTest(
 
 // Verifies StringSantizer class
 TEST(StringUtilTest, stringSanitizer) {
-
     // Bad regular expression should throw.
     StringSanitizerPtr ss;
-    ASSERT_THROW (ss.reset(new StringSanitizer("[bogus-regex","")), BadValue);
+    ASSERT_THROW(ss.reset(new StringSanitizer("[bogus-regex","")), BadValue);
+
+    std::string good_data(StringSanitizer::MAX_DATA_SIZE, '0');
+    std::string bad_data(StringSanitizer::MAX_DATA_SIZE + 1, '0');
+
+    ASSERT_NO_THROW(ss.reset(new StringSanitizer(good_data, good_data)));
+
+    ASSERT_THROW(ss.reset(new StringSanitizer(bad_data, "")), BadValue);
+    ASSERT_THROW(ss.reset(new StringSanitizer("", bad_data)), BadValue);
 
     // List of invalid chars should work: (b,c,2 are invalid)
     sanitizeStringTest("abc.123", "[b-c2]", "*",
                        "a**.1*3");
-
     // Inverted list of valid chars should work: (b,c,2 are valid)
     sanitizeStringTest("abc.123", "[^b-c2]", "*",
                        "*bc**2*");
@@ -549,6 +555,10 @@ TEST(StringUtilTest, stringSanitizer) {
     // Dots as valid chars work.
     sanitizeStringTest("abc.123", "[^A-Za-z0-9_.]", "*",
                        "abc.123");
+
+    std::string withNulls("\000ab\000c.12\0003",10);
+    sanitizeStringTest(withNulls, "[^A-Za-z0-9_.]", "*",
+                       "*ab*c.12*3");
 }
 
 // Verifies templated buffer iterator seekTrimmed() function
@@ -597,4 +607,36 @@ TEST(StringUtilTest, seekTrimmed) {
     EXPECT_EQ(8, std::distance(begin, end));
 }
 
+// Verifies isPrintable predicate on strings.
+TEST(StringUtilTest, stringIsPrintable) {
+    string content;
+
+    // Empty is printable.
+    EXPECT_TRUE(isPrintable(content));
+
+    // Check Abcd.
+    content = "Abcd";
+    EXPECT_TRUE(isPrintable(content));
+
+    // Add a control character (not printable).
+    content += "\a";
+    EXPECT_FALSE(isPrintable(content));
+}
+
+// Verifies isPrintable predicate on byte vectors.
+TEST(StringUtilTest, vectorIsPrintable) {
+    vector<uint8_t> content;
+
+    // Empty is printable.
+    EXPECT_TRUE(isPrintable(content));
+
+    // Check Abcd.
+    content = { 0x41, 0x62, 0x63, 0x64 };
+    EXPECT_TRUE(isPrintable(content));
+
+    // Add a control character (not printable).
+    content.push_back('\a');
+    EXPECT_FALSE(isPrintable(content));
+}
+
 } // end of anonymous namespace
