@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,11 +6,11 @@
 
 #include <config.h>
 
+#include <gtest/gtest.h>
+
+#include <yang/tests/sysrepo_setup.h>
 #include <yang/translator_logger.h>
 #include <yang/yang_models.h>
-#include <yang/tests/sysrepo_setup.h>
-
-#include <gtest/gtest.h>
 
 using namespace std;
 using namespace isc;
@@ -28,21 +28,20 @@ extern char const logger_list[] = "logger list";
 class TranslatorLoggersTestv4 :
     public GenericTranslatorTest<logger_list, TranslatorLoggers> {
 public:
-
-    /// Constructor.
+    /// @brief Constructor
     TranslatorLoggersTestv4() {
         model_ = KEA_DHCP4_SERVER;
     }
-};
+};  // TranslatorLoggersTestv4
+
 class TranslatorLoggersTestv6 :
     public GenericTranslatorTest<logger_list, TranslatorLoggers> {
 public:
-
-    /// Constructor.
+    /// @brief Constructor
     TranslatorLoggersTestv6() {
         model_ = KEA_DHCP6_SERVER;
     }
-};
+};  // TranslatorLoggersTestv6
 
 // This test verifies that an empty logger list can be properly
 // translated from YANG to JSON.
@@ -50,7 +49,7 @@ TEST_F(TranslatorLoggersTestv4, getEmpty) {
     // Get empty.
     const string& xpath = "/kea-dhcp4-server:config";
     ConstElementPtr loggers;
-    EXPECT_NO_THROW(loggers = t_obj_->getLoggers(xpath));
+    EXPECT_NO_THROW_LOG(loggers = translator_->getLoggersFromAbsoluteXpath(xpath));
     ASSERT_FALSE(loggers);
 }
 
@@ -63,15 +62,16 @@ TEST_F(TranslatorLoggersTestv6, get) {
     const string& xseverity = xlogger + "/severity";
     const string& xoption = xlogger + "/output-option[output='/bar']";
     const string& xmaxver = xoption + "/maxver";
-    S_Val s_severity(new Val("WARN", SR_ENUM_T));
-    EXPECT_NO_THROW(sess_->set_item(xseverity.c_str(), s_severity));
+    string const s_severity("WARN");
+    EXPECT_NO_THROW_LOG(sess_->setItem(xseverity, s_severity));
     uint32_t max_ver = 10;
-    S_Val s_maxver(new Val(max_ver));
-    EXPECT_NO_THROW(sess_->set_item(xmaxver.c_str(), s_maxver));
+    string const s_maxver(to_string(max_ver));
+    EXPECT_NO_THROW_LOG(sess_->setItem(xmaxver, s_maxver));
+    sess_->applyChanges();
 
     // Get empty.
     ConstElementPtr loggers;
-    EXPECT_NO_THROW(loggers = t_obj_->getLoggers(xpath));
+    EXPECT_NO_THROW_LOG(loggers = translator_->getLoggersFromAbsoluteXpath(xpath));
     ASSERT_TRUE(loggers);
     ASSERT_EQ(1, loggers->size());
     ConstElementPtr logger = loggers->get(0);
@@ -107,21 +107,21 @@ TEST_F(TranslatorLoggersTestv4, set) {
     // Set a value.
     const string& xpath = "/kea-dhcp4-server:config";
     ElementPtr option = Element::createMap();
-    option->set("output", Element::create(string("/bar")));
+    option->set("output", Element::create("/bar"));
     option->set("maxver", Element::create(10));
     ElementPtr options = Element::createList();
     options->add(option);
     ElementPtr logger = Element::createMap();
-    logger->set("name", Element::create(string("foo")));
-    logger->set("severity", Element::create(string("WARN")));
+    logger->set("name", Element::create("foo"));
+    logger->set("severity", Element::create("WARN"));
     logger->set("output_options", options);
     ElementPtr loggers = Element::createList();
     loggers->add(logger);
-    ASSERT_NO_THROW_LOG(t_obj_->setLoggers(xpath, loggers));
+    ASSERT_NO_THROW_LOG(translator_->setLoggers(xpath, loggers));
 
     // Get it back.
     ConstElementPtr gots;
-    EXPECT_NO_THROW(gots = t_obj_->getLoggers(xpath));
+    EXPECT_NO_THROW_LOG(gots = translator_->getLoggersFromAbsoluteXpath(xpath));
     ASSERT_TRUE(gots);
     ASSERT_EQ(1, gots->size());
     ConstElementPtr got = gots->get(0);
@@ -149,9 +149,6 @@ TEST_F(TranslatorLoggersTestv4, set) {
     ASSERT_TRUE(maxver);
     ASSERT_EQ(Element::integer, maxver->getType());
     EXPECT_EQ(10, maxver->intValue());
-
-    // Check it validates.
-    EXPECT_NO_THROW(sess_->validate());
 }
 
 /// @todo: Implement a test that will cover multiple loggers.
