@@ -1,4 +1,4 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,12 +8,11 @@
 #define ISC_TRANSLATOR_CONFIG_H 1
 
 #include <yang/translator.h>
+#include <yang/translator_class.h>
 #include <yang/translator_control_socket.h>
 #include <yang/translator_database.h>
-#include <yang/translator_class.h>
-#include <yang/translator_shared_network.h>
 #include <yang/translator_logger.h>
-#include <list>
+#include <yang/translator_shared_network.h>
 
 namespace isc {
 namespace yang {
@@ -27,8 +26,13 @@ namespace yang {
 /// @code
 /// "Dhcp4": {
 ///     "valid-lifetime": <valid lifetime>,
+///     "min-valid-lifetime": <minimum valid lifetime>,
+///     "max-valid-lifetime": <maximum valid lifetime>,
 ///     "renew-timer": <renew timer>,
 ///     "rebind-timer": <rebind timer>,
+///     "calculate-tee-times": <calculate T1/T2 times>,
+///     "t1-percent": <T1 percent>,
+///     "t2-percent": <T2 percent>,
 ///     "decline-probation-period": <decline probation period>,
 ///     "subnet4": [ <list of subnet4> ],
 ///     <shared-networks>,
@@ -45,6 +49,8 @@ namespace yang {
 ///     <expired-leases-processing>,
 ///     <dhcp4o6-port>,
 ///     <control-socket>,
+///     <hostname-char-set": <hostname character set>,
+///     <hostname-char-replacement": <hostname character replacement>,
 ///     <dhcp-ddns>,
 ///     "echo-client-id": <echo client id flag>,
 ///     "match-client-id": <match client id flag>,
@@ -59,54 +65,74 @@ namespace yang {
 ///     "reservations": [ <list of host reservations> ],
 ///     <config-control>,
 ///     "server-tag": <server tag>,
-///     "dhcp-queue-control": { <DHCP queue control> }
+///     "dhcp-queue-control": { <DHCP queue control> },
+///     "loggers": [ <list of loggers> ]
 /// },
-/// "Logging": <logging>
 /// @endcode
 ///
 /// YANG syntax for kea-dhcp4-server:config is:
 /// @code
-/// +--rw valid-lifetime?                uint32
-/// +--rw renew-timer?                   uint32
-/// +--rw rebind-timer?                  uint32
-/// +--rw decline-probation-period?      uint32
-/// +--rw subnet4*
-/// +--rw shared-network*
+/// +--rw valid-lifetime?                   uint32
+/// +--rw min-valid-lifetime?               uint32
+/// +--rw max-valid-lifetime?               uint32
+/// +--rw renew-timer?                      uint32
+/// +--rw rebind-timer?                     uint32
+/// +--rw calculate-tee-times?              boolean
+/// +--rw t1-percent?                       decimal64
+/// +--rw t2-percent?                       decimal64
+/// +--rw decline-probation-period?         uint32
+/// +--rw subnet4* [id]
+/// +--rw shared-network* [name]
 /// +--rw interfaces-config
-///    +--rw interfaces*                 string
-///    +--rw dhcp-socket-type?           enumeration
-///    +--rw outbound-interface?         enumeration
-///    +--rw re-detect?                  boolean
-///    +--rw user-context?               string
-/// +--rw lease-database!                <database>
-/// +--rw hosts-database*
-/// +--rw host-reservation-identifiers*  enumeration
-/// +--rw client-class*
-/// +--rw option-def*
-/// +--rw option-data*
-/// +--rw hook-library*
-///    +--rw library                     string
-///    +--rw parameters?                 string
-/// +--rw expired-leases-processing      <expired-leases-processing>
-/// +--rw dhcp4o6-port?                  uint16
-/// +--rw control-socket!                <control-socket>
-/// +--rw dhcp-ddns                      <dhcp-ddns>
-/// +--rw echo-client-id?                boolean
-/// +--rw match-client-id?               boolean
-/// +--rw next-server?                   inet:ipv4-address
-/// +--rw server-hostname?               string
-/// +--rw boot-file-name?                string
-/// +--rw authoritative?                 boolean
-/// +--rw user-context?                  string
+/// +--rw lease-database!
+/// +--rw hosts-database* [database-type]
+/// +--rw host-reservation-identifiers*     host-identifier-type
+/// +--rw client-class* [name]
+/// +--rw option-def* [code space]
+/// +--rw option-data* [code space]
+/// +--rw hook-library* [library]
+/// +--rw expired-leases-processing
+/// +--rw dhcp4o6-port?                     uint16
+/// +--rw control-socket!
+/// +--rw hostname-char-set?                string
+/// +--rw hostname-char-replacement?        string
+/// +--rw dhcp-ddns
+/// +--rw echo-client-id?                   boolean
+/// +--rw match-client-id?                  boolean
+/// +--rw next-server?                      inet:ipv4-address
+/// +--rw server-hostname?                  string
+/// +--rw boot-file-name?                   string
+/// +--rw authoritative?                    boolean
+/// +--rw user-context?                     user-context
 /// +--rw sanity-checks
-///    +--rw lease-checks?               enumeration
-/// +--rw reservation-mode?              enumeration
-/// +--rw host*
+/// +--rw reservation-mode?                 host-reservation-mode
+/// +--rw host* [identifier-type identifier]
 /// +--rw config-control
-///    +--rw config-database*
-/// +--rw server-tag                     string
-/// +--rw dhcp-queue-control             string
-/// +--rw logger*
+/// +--rw server-tag?                       string
+/// +--rw dhcp-queue-control?               string
+/// +--rw logger* [name]
+/// +--rw cache-max-age?                    uint32
+/// +--rw cache-threshold?                  decimal64
+/// +--rw compatibility
+/// +--rw ddns-generated-prefix?            string
+/// +--rw ddns-override-client-update?      boolean
+/// +--rw ddns-override-no-update?          boolean
+/// +--rw ddns-qualifying-suffix?           string
+/// +--rw ddns-replace-client-name?         string
+/// +--rw ddns-send-updates?                boolean
+/// +--rw ddns-update-on-renew?             boolean
+/// +--rw ddns-use-conflict-resolution?     boolean
+/// +--rw ip-reservations-unique?           boolean
+/// +--rw early-global-reservations-lookup? boolean
+/// +--rw reservations-lookup-first?        boolean
+/// +--rw multi-threading
+/// +--rw parked-packet-limit?              uint32
+/// +--rw reservations-global?              boolean
+/// +--rw reservations-in-subnet?           boolean
+/// +--rw reservations-out-of-pool?         boolean
+/// +--rw statistic-default-sample-age?     uint32
+/// +--rw statistic-default-sample-count?   uint32
+/// +--rw store-extended-info?              boolean
 /// @endcode
 ///
 /// Example of kea-dhcp6 simple configuration:
@@ -164,10 +190,18 @@ namespace yang {
 /// JSON syntax for kea-dhcp6 is:
 /// @code
 /// "Dhcp6": {
+///     "data-directory": <data directory>,
 ///     "preferred-lifetime": <preferred lifetime>,
+///     "min-preferred-lifetime": <minimum preferred lifetime>,
+///     "max-preferred-lifetime": <maximum preferred lifetime>,
 ///     "valid-lifetime": <valid lifetime>,
+///     "min-valid-lifetime": <minimum valid lifetime>,
+///     "max-valid-lifetime": <maximum valid lifetime>,
 ///     "renew-timer": <renew timer>,
 ///     "rebind-timer": <rebind timer>,
+///     "calculate-tee-times": <calculate T1/T2 times>,
+///     "t1-percent": <T1 percent>,
+///     "t2-percent": <T2 percent>,
 ///     "decline-probation-period": <decline probation period>,
 ///     "subnet6": [ <list of subnet6> ],
 ///     <shared-networks>,
@@ -187,6 +221,8 @@ namespace yang {
 ///     <server-id>,
 ///     <dhcp4o6-port>,
 ///     <control-socket>,
+///     <hostname-char-set": <hostname character set>,
+///     <hostname-char-replacement": <hostname character replacement>,
 ///     <dhcp-ddns>,
 ///     <user-context>,
 ///     <comment>
@@ -195,51 +231,75 @@ namespace yang {
 ///     "reservations": [ <list of host reservations> ],
 ///     <config-control>,
 ///     "server-tag": <server tag>,
-///     "dhcp-queue-control": { <DHCP queue control> }
+///     "dhcp-queue-control": { <DHCP queue control> },
+///     "loggers": [ <list of loggers> ]
 /// },
-/// "Logging": <logging>
 /// @endcode
 ///
 /// YANG syntax for kea-dhcp6-server:config is:
 /// @code
-/// +--rw preferred-lifetime?            uint32
-/// +--rw valid-lifetime?                uint32
-/// +--rw renew-timer?                   uint32
-/// +--rw rebind-timer?                  uint32
-/// +--rw decline-probation-period?      uint32
-/// +--rw subnet6*
-/// +--rw shared-network*
+/// +--rw data-directory?                   string
+/// +--rw preferred-lifetime?               uint32
+/// +--rw min-preferred-lifetime?           uint32
+/// +--rw max-preferred-lifetime?           uint32
+/// +--rw valid-lifetime?                   uint32
+/// +--rw min-valid-lifetime?               uint32
+/// +--rw max-valid-lifetime?               uint32
+/// +--rw renew-timer?                      uint32
+/// +--rw rebind-timer?                     uint32
+/// +--rw calculate-tee-times?              boolean
+/// +--rw t1-percent?                       decimal64
+/// +--rw t2-percent?                       decimal64
+/// +--rw decline-probation-period?         uint32
+/// +--rw subnet6* [id]
+/// +--rw shared-network* [name]
 /// +--rw interfaces-config
-///    +--rw interfaces*                 string
-///    +--rw re-detect?                  boolean
-///    +--rw user-context?               string
-/// +--rw lease-database!                <database>
-/// +--rw hosts-database*
-/// +--rw relay-supplied-options*        string
-/// +--rw mac-sources*                   string
-/// +--rw host-reservation-identifiers*  enumeration
-/// +--rw client-class*
-/// +--rw option-def*
-/// +--rw option-data*
-/// +--rw hook-library*
-///    +--rw library                     string
-///    +--rw parameters?                 string
-/// +--rw expired-leases-processing      <expired-leases-processing>
-/// +--rw server-id                      <server-id>
-/// +--rw dhcp4o6-port?                  uint16
-/// +--rw control-socket!                <control-socket>
-/// +--rw dhcp-ddns                      <dhcp-ddns>
-/// +--rw echo-client-id?                boolean
-/// +--rw user-context?                  string
+/// +--rw lease-database!
+/// +--rw hosts-database* [database-type]
+/// +--rw relay-supplied-options*           string
+/// +--rw mac-sources*                      string
+/// +--rw host-reservation-identifiers*     host-identifier-type
+/// +--rw client-class* [name]
+/// +--rw option-def* [code space]
+/// +--rw option-data* [code space]
+/// +--rw hook-library* [library]
+/// +--rw expired-leases-processing
+/// +--rw server-id!
+/// +--rw dhcp4o6-port?                     uint16
+/// +--rw control-socket!
+/// +--rw hostname-char-set?                string
+/// +--rw hostname-char-replacement?        string
+/// +--rw dhcp-ddns
+/// +--rw user-context?                     user-context
 /// +--rw sanity-checks
-///    +--rw lease-checks?               enumeration
-/// +--rw reservation-mode?              enumeration
-/// +--rw host*
+/// +--rw reservation-mode?                 host-reservation-mode
+/// +--rw host* [identifier-type identifier]
 /// +--rw config-control
-///    +--rw config-database*
-/// +--rw server-tag                     string
-/// +--rw dhcp-queue-control             string
-/// +--rw logger*
+/// +--rw server-tag?                       string
+/// +--rw dhcp-queue-control?               string
+/// +--rw logger* [name]
+/// +--rw cache-max-age?                    uint32
+/// +--rw cache-threshold?                  decimal64
+/// +--rw compatibility
+/// +--rw ddns-generated-prefix?            string
+/// +--rw ddns-override-client-update?      boolean
+/// +--rw ddns-override-no-update?          boolean
+/// +--rw ddns-qualifying-suffix?           string
+/// +--rw ddns-replace-client-name?         string
+/// +--rw ddns-send-updates?                boolean
+/// +--rw ddns-update-on-renew?             boolean
+/// +--rw ddns-use-conflict-resolution?     boolean
+/// +--rw ip-reservations-unique?           boolean
+/// +--rw early-global-reservations-lookup? boolean
+/// +--rw reservations-lookup-first?        boolean
+/// +--rw multi-threading
+/// +--rw parked-packet-limit?              uint32
+/// +--rw reservations-global?              boolean
+/// +--rw reservations-in-subnet?           boolean
+/// +--rw reservations-out-of-pool?         boolean
+/// +--rw statistic-default-sample-age?     uint32
+/// +--rw statistic-default-sample-count?   uint32
+/// +--rw store-extended-info?              boolean
 /// @endcode
 ///
 /// Example of kea-dhcp6 simple configuration:
@@ -293,55 +353,6 @@ namespace yang {
 /// </config>
 /// @endcode
 
-/// JSON syntax for the logging part of kea servers is:
-/// @code
-/// "Logging": {
-///     "loggers": [ <list of loggers> '
-/// }
-/// @endcode
-///
-/// Example of Logging simple configuration:
-/// @code
-/// {
-///     ...
-///     "Logging":
-///     {
-///         "loggers":
-///         [
-///             {
-///                 "name": "kea-dhcp6",
-///                 "output_options":
-///                 [
-///                     {
-///                         "output": "stderr"
-///                     }
-///                 ],
-///                 "severity": "DEBUG",
-///                 "debuglevel": 99
-///             }
-///         ]
-///     }
-/// }
-/// @endcode
-///
-/// The same configuration wrote into YANG datastore using @c setConfig()
-/// with a kea server model and exported to XML format:
-/// @code
-/// <config xmlns="urn:ietf:params:xml:ns:yang:kea-dhcp4-server">
-///   ...
-///   <logger>
-///     <name>kea-dhcp6</name>
-///     <output-options>
-///       <option>
-///         <output>stderr</output>
-///       </option>
-///     </output-options>
-///     <debuglevel>99</debuglevel>
-///     <severity>DEBUG</severity>
-///   </logger>
-/// </config>
-/// @endcode
-
 /// Inheritance graph between translators is:
 ///
 ///        +-----------------------------------------+
@@ -396,34 +407,27 @@ class TranslatorConfig : virtual public TranslatorControlSocket,
     virtual public TranslatorSharedNetworks,
     virtual public TranslatorLoggers {
 public:
-
     /// @brief Constructor.
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-#ifndef HAVE_PRE_0_7_6_SYSREPO
-    TranslatorConfig(sysrepo::S_Session session, const std::string& model);
-#else
-    TranslatorConfig(S_Session session, const std::string& model);
-#endif
+    TranslatorConfig(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
-    virtual ~TranslatorConfig();
+    virtual ~TranslatorConfig() = default;
 
-    /// @brief Get and translate the whole DHCP server configuration
-    /// from YANG to JSON.
+    /// @brief Translate the whole DHCP server configuration from YANG to JSON.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getConfig();
 
-    /// @brief Translate and sets the DHCP server configuration
-    /// from JSON to YANG.
+    /// @brief Translate and set the DHCP server configuration from JSON to YANG.
     ///
     /// Null elem argument removes the config containers.
     ///
     /// @param elem The JSON element.
-    void setConfig(isc::data::ConstElementPtr elem);
+    void setConfig(isc::data::ElementPtr elem);
 
 protected:
     /// @brief getConfig for ietf-dhcpv6-server.
@@ -432,7 +436,7 @@ protected:
     /// only partially and nothing else.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getConfigIetf6();
 
     /// @brief delConfig for ietf-dhcpv6-server.
@@ -450,31 +454,32 @@ protected:
     /// @brief getConfig for kea-dhcp4-server.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getConfigKea4();
 
     /// @brief getConfig for kea-dhcp6-server.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getConfigKea6();
 
     /// @brief getServer common part for kea-dhcp[46]-server:config.
     ///
-    /// @param xpath The xpath of the server.
+    /// @param data_node the YANG node representing the server configuration
+    ///
     /// @return JSON representation of the server.
-    isc::data::ElementPtr getServerKeaDhcpCommon(const std::string& xpath);
+    isc::data::ElementPtr getServerKeaDhcpCommon(libyang::DataNode const& data_node);
 
     /// @brief getServer for kea-dhcp4-server:config.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getServerKeaDhcp4();
 
     /// @brief getServer for kea-dhcp6-server:config.
     ///
     /// @return JSON representation of the config.
-    /// @throw SysrepoError when sysrepo raises an error.
+    /// @throw NetconfError when sysrepo raises an error.
     isc::data::ElementPtr getServerKeaDhcp6();
 
     /// @brief delConfig for kea-dhcp[46]-server.
@@ -497,29 +502,42 @@ protected:
     void setServerKeaDhcpCommon(const std::string& xpath,
                                 isc::data::ConstElementPtr elem);
 
+    isc::data::ElementPtr getHook(libyang::DataNode const& data_node);
+
     /// @brief Retrieves hooks configuration from sysrepo.
     ///
-    /// @param xpath path to hooks configuration.
+    /// @param data_node the YANG node representing the hook libraries
+    ///
     /// @return ElementList with hooks configuration.
-    isc::data::ElementPtr getHooksKea(const std::string& xpath);
+    isc::data::ElementPtr getHooksKea(libyang::DataNode const& data_node);
 
     /// @brief Retrieves expired leases processing parameters from sysrepo.
     ///
-    /// @param xpath path to expired leases configuration.
+    /// @param data_node the YANG node representing the configuration for expired lease processing
+    ///
     /// @return ElementList with expired leases configuration.
-    isc::data::ElementPtr getExpiredKea(const std::string& xpath);
+    isc::data::ElementPtr getExpiredKea(libyang::DataNode const& data_node);
 
     /// @brief Retrieves DDNS configuration from sysrepo
     ///
-    /// @param xpath path to dhcp-ddns configuration.
+    /// @param data_node the YANG node representing dhcp-ddns configuration
+    ///
     /// @return ElementList with dhcp-ddns configuration.
-    isc::data::ElementPtr getDdnsKea(const std::string& xpath);
+    isc::data::ElementPtr getDdnsKea(libyang::DataNode const& data_node);
 
     /// @brief Retrieves configuration control from sysrepo.
     ///
-    /// @param xpath path to configuration control.
+    /// @param data_node the YANG node representing configuration control
+    ///
     /// @return ElementMap with configuration control.
-    isc::data::ElementPtr getConfigControlKea(const std::string& xpath);
+    isc::data::ElementPtr getConfigControlKea(libyang::DataNode const& data_node);
+
+    /// @brief Retrieves interfaces configuration from sysrepo.
+    ///
+    /// @param data_node the YANG node representing the interfaces configuration
+    ///
+    /// @return ElementMap with configuration control.
+    isc::data::ElementPtr getInterfacesKea(libyang::DataNode const& data_node);
 
     /// @brief setServer for kea-dhcp4-server:config.
     ///
@@ -530,25 +548,9 @@ protected:
     ///
     /// @param elem The JSON element.
     void setServerKeaDhcp6(isc::data::ConstElementPtr elem);
+};  // TranslatorConfig
 
-    /// @brief set Logging part for kea-*:config.
-    ///
-    /// @param elem The JSON element.
-    void setServerKeaLogging(isc::data::ConstElementPtr elem);
+}  // namespace yang
+}  // namespace isc
 
-    /// @brief Retrieves an item and stores in the specified storage.
-    ///
-    /// This will attempt to retrieve an item and, if exists, will
-    /// store it in the storage.
-    ///
-    /// @param storage ElementMap (result will be stored here)
-    /// @param xpath xpath location (data will be extracted from sysrepo)
-    /// @param name name of the parameter
-    void getParam(isc::data::ElementPtr& storage, const std::string& xpath,
-                  const std::string& name);
-};
-
-}; // end of namespace isc::yang
-}; // end of namespace isc
-
-#endif // ISC_TRANSLATOR_CONFIG_H
+#endif  // ISC_TRANSLATOR_CONFIG_H
