@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,7 +8,6 @@
 #define ISC_TRANSLATOR_OPTION_DATA_H 1
 
 #include <yang/translator.h>
-#include <list>
 
 namespace isc {
 namespace yang {
@@ -24,6 +23,7 @@ namespace yang {
 ///     "csv-format": <csv format flag>,
 ///     "data": <value>,
 ///     "always-send": <always send flag>,
+///     "never-send": <never send flag>,
 ///     "user-context": { <json map> },
 ///     "comment": "<comment>"
 /// }
@@ -38,6 +38,7 @@ namespace yang {
 ///    +--rw data?           string
 ///    +--rw csv-format?     boolean
 ///    +--rw always-send?    boolean
+///    +--rw never-send?     boolean
 ///    +--rw user-context?   user-context
 /// @endcode
 ///
@@ -49,7 +50,8 @@ namespace yang {
 ///         "space": "dns",
 ///         "csv-format": false,
 ///         "data": "12121212",
-///         "always-send": false
+///         "always-send": false,
+///         "never-send": false
 ///     }
 /// ]
 /// @endcode
@@ -67,31 +69,42 @@ namespace yang {
 ///     option-data[code='100'][space='dns']/csv-format = false
 ///  /kea-dhcp6-server:config/
 ///     option-data[code='100'][space='dns']/always-send = false
+///  /kea-dhcp6-server:config/
+///     option-data[code='100'][space='dns']/never-send = false
 /// @endcode
 
 /// @brief A translator class for converting an option data between
 /// YANG and JSON.
 ///
 /// Currently supports on kea-dhcp[46]-server, not yet ietf-dhcpv6-server.
-class TranslatorOptionData : virtual public TranslatorBasic {
+class TranslatorOptionData : virtual public Translator {
 public:
-
     /// @brief Constructor.
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorOptionData(sysrepo::S_Session session, const std::string& model);
+    TranslatorOptionData(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
-    virtual ~TranslatorOptionData();
+    virtual ~TranslatorOptionData() = default;
 
-    /// @brief Get and translate an option data from YANG to JSON.
+    /// @brief Translate an option data from YANG to JSON.
     ///
+    /// @param data_node the YANG node representing the option data
+    ///
+    /// @return the JSON representation of the option data
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getOptionData(libyang::DataNode const& data_node);
+
+    /// @brief Translate an option data from YANG to JSON.
     ///
     /// @param xpath The xpath of the option data.
+    ///
     /// @return JSON representation of the option data.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getOptionData(const std::string& xpath);
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getOptionDataFromAbsoluteXpath(std::string const& xpath);
 
     /// @brief Translate and set option data from JSON to YANG.
     ///
@@ -103,10 +116,12 @@ public:
 protected:
     /// @brief getOptionData JSON for kea-dhcp[46].
     ///
-    /// @param xpath The xpath of the option data.
+    /// @param data_node the YANG node representing the option data
+    ///
     /// @return JSON representation of the option data.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getOptionDataKea(const std::string& xpath);
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getOptionDataKea(libyang::DataNode const& data_node);
 
     /// @brief setOptionData for kea-dhcp[46].
     ///
@@ -114,7 +129,7 @@ protected:
     /// @param elem The JSON element.
     void setOptionDataKea(const std::string& xpath,
                           isc::data::ConstElementPtr elem);
-};
+};  // TranslatorOptionData
 
 /// @brief A translator class for converting an option data list between
 /// YANG and JSON.
@@ -122,22 +137,33 @@ protected:
 /// Currently supports on kea-dhcp[46]-server, not yet ietf-dhcpv6-server.
 class TranslatorOptionDataList : virtual public TranslatorOptionData {
 public:
-
     /// @brief Constructor.
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorOptionDataList(sysrepo::S_Session session,
+    TranslatorOptionDataList(sysrepo::Session session,
                              const std::string& model);
 
     /// @brief Destructor.
-    virtual ~TranslatorOptionDataList();
+    virtual ~TranslatorOptionDataList() = default;
 
-    /// @brief Get and translate option data list from YANG to JSON.
+    /// @brief Translate option data list from YANG to JSON.
+    ///
+    /// @param data_node the YANG node representing the list of option data
+    ///
+    /// @return the JSON representation of the list of option data
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ConstElementPtr getOptionDataList(libyang::DataNode const& data_node);
+
+    /// @brief Translate option data list from YANG to JSON.
     ///
     /// @param xpath The xpath of the option data list.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ConstElementPtr getOptionDataList(const std::string& xpath);
+    ///
+    /// @return the JSON representation of the list of option data
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ConstElementPtr getOptionDataListFromAbsoluteXpath(std::string const& xpath);
 
     /// @brief Translate and set option data list from JSON to YANG.
     ///
@@ -149,9 +175,10 @@ public:
 protected:
     /// @brief getOptionDataList for kea-dhcp[46].
     ///
-    /// @param xpath The xpath of the option data list.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ConstElementPtr getOptionDataListKea(const std::string& xpath);
+    /// @param data_node the YANG node representing the list of option data
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ConstElementPtr getOptionDataListKea(libyang::DataNode const& data_node);
 
     /// @brief setOptionDataList for kea-dhcp[46].
     ///
@@ -160,9 +187,9 @@ protected:
     /// @throw BadValue on option data without code or space.
     void setOptionDataListKea(const std::string& xpath,
                               isc::data::ConstElementPtr elem);
-};
+};  // TranslatorOptionDataList
 
 }  // namespace yang
 }  // namespace isc
 
-#endif // ISC_TRANSLATOR_OPTION_DATA_H
+#endif  // ISC_TRANSLATOR_OPTION_DATA_H
